﻿--[[========================================================================================
      AdvancedIconSelector-KeywordData provides keyword data associated with World of
      Warcaft icons to World of Warcraft addons.
      
      Copyright (c) 2011 - 2012 David Forrester  (Darthyl of Bronzebeard-US)
        Email: darthyl@hotmail.com
      
      Permission is hereby granted, free of charge, to any person obtaining a copy
      of this software and associated documentation files (the "Software"), to deal
      in the Software without restriction, including without limitation the rights
      to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
      copies of the Software, and to permit persons to whom the Software is
      furnished to do so, subject to the following conditions:
      
      The above copyright notice and this permission notice shall be included in
      all copies or substantial portions of the Software.
      
      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
      OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
      THE SOFTWARE.
    ========================================================================================]]

--[[========================================================================================
      If you are making use of LibAdvancedIconSelector in your addon:
      
      You CAN package this keyword data alongside your addon so that
      AdvancedIconSelector doesn't have to be installed for your icons to
      have keywords, but you'll need to rename this folder so that your addon
      can be installed side-by-side with AIS.  The keyword data should not be
      embedded in your addon, however, as it's designed to be loaded on-demand.
      
      For example, if you want to package keyword data alongside an addon named
      "Foo" that makes use of the icon selector, then you should:
        a) Rename this folder from "AdvancedIconSelector-KeywordData" to "Foo-KeywordData"
        b) Change the title in the TOC to "Foo [KeywordData]" or something similar
        c) Change X-Part-Of in the TOC to "Foo" (used by Addon Control Panel)
        d) Change ADDON_NAME below to "Foo-KeywordData" (but do NOT change MAJOR_VERSION)
        e) Specify keywordAddonName = "Foo-KeywordData" in the options table passed to
             CreateIconSelectorWindow / CreateIconSelectorFrame / CreateSearch.  This will
             tell LAIS to use your keyword library if it's newer than the currently loaded
             / default one.
      
      Do not change the X-Revision number in the TOC unless you've got a very good reason
      to prioritize your keyword library over the one provided by AIS.
    ========================================================================================]]

local ADDON_NAME = "ActionSwap2-IconKeywords"

-- Do NOT change the MAJOR_VERSION string to the name of your addon, however,
-- as this is used to obtain the most up-to-date keyword data, regardless of
-- what addon it came from.
local MAJOR_VERSION = "LibAdvancedIconSelector-KeywordData-1.0"
local MINOR_VERSION = tonumber(GetAddOnMetadata(ADDON_NAME, "X-Revision"))

if not LibStub then error(ADDON_NAME .. " requires LibStub to operate") end
local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local DEBUG = false
function lib:Debug(...)
	if DEBUG then
		print("|cff00ff00[AIS-KWD] [Debug]|r", ...)
	end
end

lib:Debug("Keyword data loaded!")

-- Returns the keyword library revision, as specified by X-Revision in the TOC.
function lib:GetRevision()
	return MINOR_VERSION
end

-- Returns the keywords associated with the given texture filename.
function lib:GetKeywords(texture)
	if texture then
		return lib.keywords[strlower(texture)]
	end
end

--#### Everything below this line has been automatically generated by the Icon Keyword Editor. ####
lib.keywords = {
	["ability_ambush"] = "purple ambush hidden stealth hide hiding rogue leaves spell:feral_instinct(druid) racial:shadowmeld(night_elf)",
	["ability_backstab"] = "red silver pierce puncture bloody steel metal sword blade weapon gushing gash injury spell:dispatch(rogue) tab:combat(rogue) spell:backstab(rogue) spell:deep_wounds(warrior)",
	["ability_bullrush"] = "red tauren bull horned horns furry fuzzy beast animal creature rush roar shout anger angry fury furious rage raging aggression head spell:frenzied_regeneration(druid)",
	["ability_cheapshot"] = "brown eyeball whites sword blade weapon threatened humanoid stabbing spell:cheap_shot(rogue)",
	["ability_creature_amber_01"] = "orange yellow amber goo liquid spit vomit water slime poison flowing",
	["ability_creature_amber_02"] = "orange yellow amber goo liquid poison water pool brew cauldron steaming bubbling bubbles boiling",
	["ability_creature_cursed_01"] = "curse orange runes of power symbols circle emblem disease skul aural magic spell sigil seal",
	["ability_creature_cursed_02"] = "red skull and crossbones jolly roger undeath undead evil cursed deadly poisonous toxic corpse spell:will_of_the_necropolis(death_knight)",
	["ability_creature_cursed_03"] = "curse skull purple dead dying magic head spell:ebon_plaguebringer(death_knight)",
	["ability_creature_cursed_04"] = "red swirly vortex portal maelstrom spiraling magic spell",
	["ability_creature_cursed_05"] = "curse white skull death dead magic spell face",
	["ability_creature_disease_01"] = "green brown rotten apple mushroom fungus fungi diseased",
	["ability_creature_disease_02"] = "skull green dead goo poison toxic disease ectoplasm magic spell head talent:plague_leech(death_knight)",
	["ability_creature_disease_03"] = "green pink poisoned diseased pox head",
	["ability_creature_disease_04"] = "disease green wolf canine dog worg animal furry fuzzy creature beast anger angry fury furious rage raging aggression ectoplasmic ghost apparition spirit baring teeth magic spell head",
	["ability_creature_disease_05"] = "hand purple curse disease shadow unholy magic companion:kirin_tor_familiar",
	["ability_creature_poison_01"] = "poison toxic disease teeth goo green pink saliva bite jaws gums",
	["ability_creature_poison_02"] = "green goo poison water liquid disease toxic nature magic spell power slime sludge",
	["ability_creature_poison_03"] = "posion green rip tear wound claw pierce",
	["ability_creature_poison_04"] = "poisonous green thorns plant painful sharp vines roots",
	["ability_creature_poison_05"] = "green ghoul poison disease ghost teeth apparition head",
	["ability_creature_poison_06"] = "bubbles disease green blood water liquid surface toxic slime sludge spell:master_poisoner(rogue)",
	["ability_criticalstrike"] = "bloody dried covered soaked swinging sword blade weapon moving motion spell:recklessness(warrior)",
	["ability_deathknight_aoedeathgrip"] = "purple violet hands claws gripping grasping reaching pulling spirit ghostly ghastly",
	["ability_deathknight_asphixiate"] = "purple violet human hand gripping grasping holding glowing fire fiery burning hot heat flames flaming blazing scorching searing aura",
	["ability_deathknight_boneshield"] = "bones green skeletal skeleton spell:bone_shield(death_knight)",
	["ability_deathknight_brittlebones"] = "broken skeletal skeleton blue shattered shards fragments spell:brittle_bones(death_knight)",
	["ability_deathknight_deathsiphon"] = "green glowing human hand reaching undead undeath unholy zombie claw poison",
	["ability_deathknight_deathsiphon2"] = "green grass hair flowing beams undead undeath unholy strands",
	["ability_deathknight_desecratedground"] = "green unholy circle sphere patch of ground corruption corrupted desecration desecrated",
	["ability_deathknight_desolation"] = "hand curse blood purple shadow unholy magic spell",
	["ability_deathknight_hemorrhagicfever"] = "red blood horror fever hemorrhage magic spell",
	["ability_deathknight_icygrip"] = "blue undead undeath unholy zombie grip grasping reaching claw ice icy cold frozen freezing frosty",
	["ability_deathknight_pillaroffrost"] = "frozen ice icy cold freezing frosty snowy blue stone spell:pillar_of_frost(death_knight)",
	["ability_deathknight_remorselesswinters"] = "bluegreen white ice icy cold frozen freezing frosty swirling swirly spiraling clouds",
	["ability_deathknight_remorselesswinters2"] = "bluegreen white ice icy cold frozen freezing frosty swirling swirly spiraling clouds ribbons human figure infused",
	["ability_deathknight_roilingblood"] = "green red undead undeath unholy zombie gore skull bone skeletal talent:roiling_blood(death_knight)",
	["ability_deathknight_runicimpowerment"] = "frozen ice icy cold freezing frosty blue blade battleaxe weapon",
	["ability_deathknight_sanguinfortitude"] = "red silver wooden shield armor defense protection blocking blood buckler spell:sanguine_fortitude(death_knight)",
	["ability_deathknight_soulreaper"] = "blue emblem creature undead undeath unholy spell:soul_reaper(death_knight)",
	["ability_deathknight_summongargoyle"] = "black bat vampire fangs teeth creature animal bite head face spell:summon_gargoyle(death_knight)",
	["ability_deathwing_assualtaspects"] = "deathwing neltharion red orange glowing iris eyeball black dragon demonic evil staring stare",
	["ability_deathwing_bloodcorruption_death"] = "bloody dripping drips drops liquid skull red death evil unholy",
	["ability_deathwing_bloodcorruption_earth"] = "red bloody dripping drips drops liquid sphere",
	["ability_deathwing_cataclysm"] = "red orange deathwing neltharion black fire fiery burning hot heat flames flaming blazing scorching searing dragonbreath mouth head",
	["ability_deathwing_fierygrip"] = "red orange fire fiery burning hot heat flames flaming blazing scorching searing beam laser flare pillar power magic spell deathwing neltharion",
	["ability_deathwing_grasping_tendrils"] = "red orange fire fiery burning hot heat flames flaming blazing scorching searing lava magma tendril corruption tentacle deathwing neltharion",
	["ability_deathwing_sealarmorbreachtga"] = "red orange burning tendons searing deathwing neltharion",
	["ability_deathwing_shrapnel"] = "ice icy cold frozen freezing frosty steel metal shrapnel shards icicles",
	["ability_defend"] = "silver steel metal armor defense protection blocking spears weapons defend pet_stance:defensive(hunter) spell:shield_block(warrior)",
	["ability_devour"] = "gray grey devour eat teeth bite",
	["ability_druid_aquaticform"] = "whale tail animal creature critter ocean sea water blue outline magic silhouette spell:aquatic_form(druid)",
	["ability_druid_astralstorm"] = "purple violet arcane energy windy gusts gusty winds leaves cyclone tornado hurricane torrential blowing hurling carrying lifting destroying tearing ripping nature magic powerful destructive force natural storm whirlwind swirly",
	["ability_druid_balanceofpower"] = "green scale level weight spell:balance_of_power(druid)",
	["ability_druid_bash"] = "blue brown bear furry beast animal creature paw claws foot scratch",
	["ability_druid_berserk"] = "cat kitty feline furry fuzzy beast animal creature tiger lion panther orange red roar anger angry fury furious rage raging aggression mouth teeth bite head spell:berserk(druid) pet_command:rabid(hunter)",
	["ability_druid_catform"] = "kitty feline furry fuzzy beast animal creature claw paw blue outline scratch silhouette tab:feral(druid) spell:cat_form(druid)",
	["ability_druid_catformattack"] = "purple blue black cat kitty feline furry fuzzy beast animal creature paw claw scratch panther",
	["ability_druid_challangingroar"] = "furry beast animal creature roar shout anger angry fury furious rage raging aggression red orange head mount:amani_battle_bear",
	["ability_druid_cower"] = "black cat kitty feline furry fuzzy animal creature scared terrified afraid fear blue spell:scare_beast(hunter)",
	["ability_druid_cyclone"] = "green swirly cyclone tornado windy gusty gusts whirlwind swirling spiraling twister twisting vortex nature magic",
	["ability_druid_dash"] = "fast run green animal cat kitty feline furry fuzzy beast creature speed go tail paw spell:dash(druid) pet_command:dash(hunter)",
	["ability_druid_demoralizingroar"] = "brown pink bear furry beast animal creature head roar teeth bite shout",
	["ability_druid_disembowel"] = "claw tip sharp blood animal creature pierce red scratch spell:rake(druid)",
	["ability_druid_dreamstate"] = "wisp power purple head arcane shadow magic spell",
	["ability_druid_earthandsky"] = "green blue silver helmet armor defense protection magic spell power",
	["ability_druid_eclipse"] = "planet lunar solar blue magic spell:mastery:_total_eclipse(druid)",
	["ability_druid_eclipseorange"] = "planet orange lunar solar magic spell:eclipse(druid)",
	["ability_druid_empoweredrejuvination"] = "healing restoration star magic nature power spell:swift_rejuvenation(druid)",
	["ability_druid_empoweredtouch"] = "hand touch light power magic spell healing restoration",
	["ability_druid_enrage"] = "purple tauren bull horned horns furry fuzzy beast animal creature anger angry fury furious raging aggression head baring teeth magic spell:enrage(druid)",
	["ability_druid_ferociousbite"] = "red cat kitty feline furry fuzzy beast animal creature teeth fangs face spell:ferocious_bite(druid) spell:bestial_wrath(hunter)",
	["ability_druid_flightform"] = "flying wings blue bird winged feathered feathers animal creature magic spell:flight_form(druid) spell:swift_flight_form(druid)",
	["ability_druid_flourish"] = "green plant life healing restoration flourish nourishing grass herbs magic nature spell:wild_growth(druid)",
	["ability_druid_forceofnature"] = "treant tree form purple green nature magic spell enchanted enchantment",
	["ability_druid_galewinds"] = "purple tornado gusty gale winds cyclone swirly nature magic spiraling vortex whirlwind",
	["ability_druid_giftoftheearthmother"] = "green healing restoration growing sprout life hand nurturing nurture nature magic power spell:living_seed(druid)",
	["ability_druid_healinginstincts"] = "owl face bird winged feathered feathers animal creature eyes spell:nurturing_instinct(druid)",
	["ability_druid_improvedmoonkinform"] = "moonkin form bird winged feathered feathers animal creature outline silhouette blue purple nature magic spell power head",
	["ability_druid_improvedtreeform"] = "leaf blue life healing restoration tree plant growth nature magic spell power",
	["ability_druid_infectedwound"] = "diseased pox green red blood scratch scar spell:infected_wounds(druid)",
	["ability_druid_kingofthejungle"] = "yellow orange red lion roar teeth head mouth bite feline cat kitty furry fuzzy beast animal creature",
	["ability_druid_lacerate"] = "wound blood red fangs mouth teeth bite spell:lacerate(druid)",
	["ability_druid_lunarguidance"] = "white black priest prayer light moon lunar worgen spell:revive(druid)",
	["ability_druid_manatree"] = "tree life plant healing restoration moon sky green blue nature magic spell power",
	["ability_druid_mangle"] = "red purple black wound scratch blood mangle claw cat kitty feline furry fuzzy beast animal creature rip tear",
	["ability_druid_mangle.tga"] = "red purple wound scratch blood mangle claw cat kitty feline furry fuzzy beast animal creature rip tear spell:maim(druid)",
	["ability_druid_mangle2"] = "red brown claw scratch blood wound animal creature spell:mangle(druid)",
	["ability_druid_mastershapeshifter"] = "black orange shapeshifter animal cat kitty feline furry fuzzy beast creature transformation",
	["ability_druid_maul"] = "blue brown scratch paw claw cat kitty feline furry fuzzy beast animal creature bear spell:maul(druid)",
	["ability_druid_naturalperfection"] = "healing restoration protection protective nature green magic spell power talent:cenarion_ward(druid)",
	["ability_druid_nourish"] = "plant leaf tree restoration healing nourishing green purple nature magic spell:nourish(druid)",
	["ability_druid_owlkinfrenzy"] = "moonkin red orange fire fiery burning hot heat flames flaming blazing scorching searing anger angry fury furious rage raging aggression antlers nature magic spell:owlkin_frenzy(druid)",
	["ability_druid_predatoryinstincts"] = "predator glowing eyes hidden ambush hide stealth spell:killer_instinct(druid)",
	["ability_druid_primalagression"] = "yellow claw bear furry beast animal creature scratch paw rend blood rip",
	["ability_druid_primalprecision"] = "yellow precision spot light point sights marksmanship",
	["ability_druid_primaltenacity"] = "white red brown fangs teeth bloody anger angry fury furious rage raging aggression bite bitten furry animal creature beast worg face",
	["ability_druid_prowl"] = "hidden hide hiding stealth blue ambush spell:prowl(druid)",
	["ability_druid_rake"] = "white rip tear blood wound paw animal creature cat kitty feline furry fuzzy beast wolf canine dog rend racial_passive:flayer(worgen) pet_command:claw(hunter)",
	["ability_druid_ravage"] = "black red rip tear wound claw pierce spell:ravage(druid)",
	["ability_druid_replenish"] = "healing restoration life green nature magic power poison toxic roots surrounding spell:viper_venom(hunter)",
	["ability_druid_skinteeth"] = "cat kitty feline furry fuzzy beast animal creature face head teeth fangs bite glowing eyes red orange spell:savage_roar(druid)",
	["ability_druid_starfall"] = "blue white comets meteors moonlight moonbeams magic power blast burst sky stars arcane nature spell:starfall(druid)",
	["ability_druid_supriseattack"] = "purple black cat kitty feline furry fuzzy beast animal creature prowl hidden hiding ambush grass sneaking hunting face spell:pounce(druid)",
	["ability_druid_swipe"] = "cat kitty feline furry fuzzy beast animal creature claw paw swipe red scratch",
	["ability_druid_tigersroar"] = "orange black cat kitty feline furry fuzzy beast animal creature tiger roar mouth fangs bite anger angry fury furious rage raging aggression face spell:survival_instincts(druid)",
	["ability_druid_travelform"] = "blue outline silhouette horseshoe hoofprint zhevra swift fast running nature magic power spell:travel_form(druid)",
	["ability_druid_treeoflife"] = "blue leaf healing tree life restoration nature magic spell power outline silhouette",
	["ability_druid_twilightswrath"] = "twilight purple power lightning magic arcane shadow nature",
	["ability_druid_typhoon"] = "purple swirly disease poison typoon cyclone torrent tornado windy gusty gusts whirlwind swirling spiraling twister twisting vortex nature magic maelstrom",
	["ability_dualwield"] = "silver steel metal swords blades weapons daggers knives knife cross battle duel passive:dual_wield(death_knight) passive:dual_wield(rogue) spell:ambidexterity(rogue) passive:dual_wield(hunter) passive:dual_wield(shaman)",
	["ability_dualwieldspecialization"] = "blue yellow orange fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty empowered magic weapons swords blades spell:threat_of_thassarian(death_knight)",
	["ability_ensnare"] = "blue brown net catch rope kidnap spell:trap_mastery(hunter)",
	["ability_eyeoftheowl"] = "white owl eye beak flying flight sight vision",
	["ability_fiegndead"] = "gray grey death dead tombstone graveyard zombie ghoul ghost buried",
	["ability_ghoulfrenzy"] = "claw cat kitty feline furry fuzzy beast animal creature rend tear wound blood red spell:rip(druid) pet_command:attack(hunter) spell:flurry(warrior) spell:flurry(shaman)",
	["ability_glaivetoss"] = "silver steel metal blade glaive shuriken thrown tossed",
	["ability_golemstormbolt"] = "white gray grey bullet pellet shoot ammo ammunition spell:scatter_shot(hunter)",
	["ability_golemthunderclap"] = "gray grey purple thunder lightning stone fist punch power earthen nature magic knuckles professions:toughness spell:intimidating_shout(warrior)",
	["ability_gouge"] = "red silver steel metal rend rip tear gash sword blade weapon dagger knife wound blood spell:gouge(rogue)",
	["ability_heroicleap"] = "jump flying flight red passive:imp:_flee(warlock) spell:heroic_leap(warrior)",
	["ability_hibernation"] = "sleepy tired dead closed eyes hibernating",
	["ability_hunter_aimedshot"] = "tip sharp precision precise target point arrowhead spear marksmanship shoot blue bullseye",
	["ability_hunter_animalhandler"] = "animal creature bird winged feathered feathers purple sky hawk hippogryph moonlight",
	["ability_hunter_aspectmastery"] = "blue purple tiger power head face aspect transformation animal cat kitty feline furry fuzzy beast creature",
	["ability_hunter_aspectofthefox"] = "red orange white animal creature canine head ears hearing spell:aspect_of_the_fox(hunter)",
	["ability_hunter_aspectoftheironhawk"] = "silver iron stone granite emblem talons claws foot bird winged feathered feathers animal creature hawk eagle outline silhouette nature magic spell power",
	["ability_hunter_aspectofthemonkey"] = "black yellow banana monkey ape primate fruit gripping animal creature hand",
	["ability_hunter_aspectoftheviper"] = "blue purple eyeball beast viper snake animal creature sight vision face",
	["ability_hunter_assassinate"] = "white skull skeleton skeletal arrow death dead head bones",
	["ability_hunter_assassinate2"] = "tip point arrowhead spear blood red pierce puncture shoot spell:kill_shot(hunter)",
	["ability_hunter_beastcall"] = "brown wooden whistle blow noise sound instrument spell:call_pet_2(hunter) spell:call_pet_3(hunter) spell:call_pet_4(hunter) spell:call_pet_5(hunter)",
	["ability_hunter_beastcall02"] = "brown wooden weapon slingshot stick rubber band passive:control_pet(hunter)",
	["ability_hunter_beastmastery"] = "orange red dinosaur animal creature reptile teeth bite head rage roar savage spell:exotic_beasts(hunter)",
	["ability_hunter_beastsoothe"] = "brown hand slap motion spell:soothe(druid) spell:revive_pet(hunter)",
	["ability_hunter_beasttaming"] = "green whip mace weapon torture hand spikes ball spell:tame_beast(hunter) spell:mastery:_master_of_beasts(hunter)",
	["ability_hunter_beasttraining"] = "brown treat yummie food biscuit training spell:feed_pet(hunter)",
	["ability_hunter_beastwithin"] = "transformation blue red eyeball magic power spell:the_beast_within(hunter)",
	["ability_hunter_bestialdiscipline"] = "animals dinosaur reptile wolf canine dog worg furry fuzzy cat kitty feline fangs transformation beasts rage pack group savage creatures yellow white black panther spell:stampede(hunter)",
	["ability_hunter_blackicetrap"] = "purple blue ice icy cold frozen freezing frosty chain links ring trap handcuffs",
	["ability_hunter_blindingshot"] = "weapon ammunition ammo golden piercing pierce sharp tip arrowhead",
	["ability_hunter_camouflage"] = "green brown hiding hidden hide stealth sneaky ambush rogue leaves",
	["ability_hunter_catlikereflexes"] = "cat kitty feline furry fuzzy beast animal creature panther yellow",
	["ability_hunter_chimerashot2"] = "purple tip arrowhead weapon point sharp pierce puncture staff javelin spear spell:chimera_shot(hunter)",
	["ability_hunter_cobrashot"] = "green silver staff tip point arrowhead weapon spell:cobra_shot(hunter)",
	["ability_hunter_cobrastrikes"] = "purple snake poisonous toxic king deadly venomous fangs bite sting viper companion:cobra_hatchling spell:cobra_strikes(hunter)",
	["ability_hunter_combatexperience"] = "purple brown red horn pet_command:combat_experience(hunter)",
	["ability_hunter_criticalshot"] = "purple arrowhead weapon point sharp pierce puncture spear javelin marksmanship shoot",
	["ability_hunter_crossfire"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell power",
	["ability_hunter_disarmingshot"] = "purple silver disarm arrowhead shoot",
	["ability_hunter_displacement"] = "blue shadow mirror image deception deceit night elf speed dash running spell:camouflage(hunter)",
	["ability_hunter_eagleeye"] = "bird winged feathered feathers animal creature flying sky vision sight beak head spell:eagle_eye(hunter)",
	["ability_hunter_efficiency"] = "silver weapons spears staves staff arrows arrowheads ammunition ammo fireworks rockets spell:concussive_barrage(hunter)",
	["ability_hunter_efficiency_brown"] = "silver weapons spears staves staff arrows arrowheads ammunition ammo fireworks rockets",
	["ability_hunter_explosiveshot"] = "orange yellow red explosives boom explosion fireworks spear javelin arrowhead fuse weapon spell:explosive_shot(hunter)",
	["ability_hunter_ferociousinspiration"] = "red power circle magic spell",
	["ability_hunter_fervor"] = "yellow brown black dinosaur beast animal creature reptile mouth teeth man beard transformation savage face head",
	["ability_hunter_focusedaim"] = "red white silver target arrows arrowheads precision points piercing pierce marksmanship bullseye passive:focused_aim(hunter)",
	["ability_hunter_focusfire"] = "brown silver bow arrowhead weapon shoot aim marksman precision tip point sharp spell:focus_fire(hunter)",
	["ability_hunter_glacialtrap"] = "blue white icy cold frozen freezing frosty iceberg",
	["ability_hunter_goforthethroat"] = "yellow black cat kitty feline furry fuzzy beast animal creature hand death spell:go_for_the_throat(hunter)",
	["ability_hunter_harass"] = "shout red cry anger angry fury furious rage raging aggression upset talent:second_wind(warrior)",
	["ability_hunter_huntervswild"] = "cliff sky mountains standing view scenery landscape background daytime blue",
	["ability_hunter_huntingparty"] = "blue hunter bow",
	["ability_hunter_improvedsteadyshot"] = "yellow silver arrowhead tip sharp point puncture pierce spell:steady_focus(hunter)",
	["ability_hunter_improvedtracking"] = "snowy trek journey walking ice icy cold frozen freezing frosty tracking footprints pawprints white",
	["ability_hunter_invigeration"] = "wolf attack blue black yellow dog canine animal creature outlines silhouettes spell:invigoration(hunter)",
	["ability_hunter_killcommand"] = "red magic order blood spell:kill_command(hunter)",
	["ability_hunter_lockandload"] = "shotgun gunpowder pistol rifle red spell:lock_and_load(hunter)",
	["ability_hunter_longevity"] = "wolf canine dog worg animal furry fuzzy creature beast anger angry fury furious rage raging aggression intimidating intimidation yellow gray grey growling snarling baring teeth head",
	["ability_hunter_longshots"] = "blue sky arrows missed sticking out of ground grass",
	["ability_hunter_markedfordeath"] = "hunter's red marksmanship arrow pointing down",
	["ability_hunter_mastermarksman"] = "brown eyeball sights shoot targetting reticle spell:master_marksman(hunter)",
	["ability_hunter_masterscall"] = "orange white blue cat kitty feline furry fuzzy beast animal creature tiger fangs roar hiss bite running face spell:master's_call(hunter)",
	["ability_hunter_mastertactitian"] = "purple tactics",
	["ability_hunter_mendpet"] = "bloody paw hurt injured injury bandaged claws cat kitty feline furry fuzzy beast animal creature tiger wrapped bandages spell:mend_pet(hunter)",
	["ability_hunter_misdirection"] = "arrowhead shoot pierce puncture swirly blue spiral spell:misdirection(hunter)",
	["ability_hunter_murderofcrows"] = "black ravens crows birds winged feathered feathers animals creatures",
	["ability_hunter_onewithnature"] = "green nature magic life restoration meditation spell:natural_insight(druid)",
	["ability_hunter_pathfinding"] = "compass planet directions blue",
	["ability_hunter_pathfinding2"] = "horse pack mule donkey riding cloaked rider blue",
	["ability_hunter_pet_assist"] = "cat kitty feline furry fuzzy beast animal creature transformation blue red pet_stance:assist(hunter)",
	["ability_hunter_pet_bat"] = "black bat furry animal winged creature fangs vampire teeth face head",
	["ability_hunter_pet_bear"] = "brown bear furry beast animal creature head face teeth anger angry fury furious rage raging aggression hungry",
	["ability_hunter_pet_boar"] = "pink boar head animal creature tusks",
	["ability_hunter_pet_cat"] = "lion fangs teeth cat kitty feline furry fuzzy beast animal creature head spell:predatory_swiftness(druid) spell:call_pet_1(hunter)",
	["ability_hunter_pet_chimera"] = "black yellow beast animal creature pet tongue teeth head talent:crouching_tiger,_hidden_chimera(hunter)",
	["ability_hunter_pet_corehound"] = "red yellow orange magma puppy dog lava animal creature firelands head companion:core_hound_pup",
	["ability_hunter_pet_crab"] = "orange crab pincers sea ocean animal creature monstrous claw companion:strand_crawler",
	["ability_hunter_pet_crocolisk"] = "crocolisk crocodile animal creature reptile bite teeth jaws gray grey head",
	["ability_hunter_pet_devilsaur"] = "raptor dinosaur beast animal creature reptile teeth bite green head",
	["ability_hunter_pet_dragonhawk"] = "bird winged feathered feathers animal creature dragonhawk flying flight jousting silver gray grey head companion:blue_mini_jouster",
	["ability_hunter_pet_gorilla"] = "gorilla grey gray animal creature primate head",
	["ability_hunter_pet_goto"] = "black blue cyan go order signal pointing cat kitty feline furry fuzzy beast animal creature outlines silhouettes pet_command:move_to(hunter)",
	["ability_hunter_pet_hyena"] = "hyena head fangs teeth bite growl anger angry fury furious rage raging aggression animal creature canine yellow snarl baring",
	["ability_hunter_pet_moth"] = "animal creature companion:blue_moth companion:white_moth companion:yellow_moth",
	["ability_hunter_pet_netherray"] = "blue nether ray outland animal creature",
	["ability_hunter_pet_owl"] = "beak owl winged feathered feathers animal creature flying flight head companion:tickbird_hatchling companion:white_tickbird_hatchling",
	["ability_hunter_pet_raptor"] = "raptor dinosaur beast animal creature reptile horn head mount red",
	["ability_hunter_pet_ravager"] = "ravager outland animal creature yellow head",
	["ability_hunter_pet_rhino"] = "brown gray grey rhino head furry hairy animal creature horns",
	["ability_hunter_pet_scorpid"] = "scorpid scorpion animal creature claws pincers yellow",
	["ability_hunter_pet_silithid"] = "red purple silithid bug ahn'qiraj",
	["ability_hunter_pet_spider"] = "spider bug animal creature red head",
	["ability_hunter_pet_sporebat"] = "sporebat outland swamp animal creature blue",
	["ability_hunter_pet_tallstrider"] = "plainstrider tallstrider purple beak head feathered bird winged feathers animal creature",
	["ability_hunter_pet_turtle"] = "green turtle head slow tortoise animal creature spell:polymorph(mage)",
	["ability_hunter_pet_vulture"] = "pink vulture bird winged feathered feathers animal creature beak head companion:gold_mini_jouster",
	["ability_hunter_pet_warpstalker"] = "green warp stalker outland terokkar forest animal creature head",
	["ability_hunter_pet_wasp"] = "wasp blue purple wings winged animal creature outland swamp zangarmarsh",
	["ability_hunter_pet_windserpent"] = "orange wind serpent animal creature beast flying flight outland blade's edge mountains head",
	["ability_hunter_pet_wolf"] = "gray grey wolf canine dog animal furry fuzzy creature beast anger angry fury furious rage raging aggression intimidating intimidation growling snarling baring teeth head racial_passive:viciousness(worgen)",
	["ability_hunter_pet_worm"] = "pink worm slimy animal creature",
	["ability_hunter_piercingshots"] = "silver steel metal chestplate armor defense protection horned horns helmet pierced heart torso arrow weapon shoot passive:piercing_shots(hunter)",
	["ability_hunter_pointofnoescape"] = "blue surrounded arrows trapped",
	["ability_hunter_posthaste"] = "blue brown running exit vehicle leave walking fast swift dash foot boot talent:posthaste(hunter)",
	["ability_hunter_potentvenom"] = "green silver poisoned arrowhead tip deadly shoot toxic spell:widow_venom(hunter)",
	["ability_hunter_quickshot"] = "silver arrowhead spear shoot shot tip point sharp pierce puncture spell:serpent_sting(hunter) passive:improved_serpent_sting(hunter)",
	["ability_hunter_rapidkilling"] = "death graves graveyard dead red moon",
	["ability_hunter_rapidregeneration"] = "silver blue rifle firing shooting shotgun aiming spell:rapid_recuperation(hunter)",
	["ability_hunter_readiness"] = "blue purple time clock sundial slow fast speed magic power spell:readiness(hunter)",
	["ability_hunter_resistanceisfutile"] = "yellow silver surrounded threatened no escape trapped arrows arrowheads",
	["ability_hunter_resourcefulness"] = "blue brown campfire stick smoke camping sparks",
	["ability_hunter_runningshot"] = "shooting shot arrow red outline silhouette marksmanship aim spell:rapid_fire(hunter)",
	["ability_hunter_separationanxiety"] = "pet friends sad leaving dog wolf canine worg animal furry fuzzy creature beast separating black silhouettes outlines spell:kindred_spirits(hunter)",
	["ability_hunter_serpentswiftness"] = "yellow viper snake swift fast speed strike striking venomous slimy animal creature head spell:serpent_spread(hunter)",
	["ability_hunter_sickem"] = "red white wolf canine dog worg animal furry fuzzy creature bite fangs teeth growling snarling anger angry fury furious rage raging aggression intimidating baring head spell:beast_cleave(hunter)",
	["ability_hunter_silenthunter"] = "gray grey hunter targeting reticle owl beast feathered animal creature sight vision precision aim face",
	["ability_hunter_snaketrap"] = "snakes orange blue green deadly venomous animals creatures spell:snake_trap(hunter)",
	["ability_hunter_snipershot"] = "red gray grey black targeting reticle aim precision marksmanship shooting spell:hunter's_mark(hunter)",
	["ability_hunter_snipertraining"] = "red black targeting reticle shoot precision marksmanship",
	["ability_hunter_steadyshot"] = "yellow white arrowhead shoot feathers spell:steady_shot(hunter)",
	["ability_hunter_survivalinstincts"] = "black cat kitty feline furry fuzzy beast animal creature orange purple outline silhouette",
	["ability_hunter_swiftstrike"] = "red silver battleaxe blade weapon blood rend tear rip swift fast swing strike",
	["ability_hunter_thrillofthehunt"] = "green crossbow arrow weapon shoot aim",
	["ability_hunter_traplauncher"] = "red silver bear furry beast animal creature spikes spell:trap_launcher(hunter)",
	["ability_hunter_wildquiver"] = "green gray grey arrows fur spell:mastery:_wild_quiver(hunter)",
	["ability_hunter_zenarchery"] = "aiming bow arrow shooting orange firing spell:careful_aim(hunter)",
	["ability_impalingbolt"] = "purple arrowhead weapon point sharp pierce puncture spear javelin marksmanship shoot spell:arcane_shot(hunter)",
	["ability_kick"] = "brown gray grey boot step foot walking running spell:kick(rogue)",
	["ability_mage_arcanebarrage"] = "blue magic blast burst bolts missiles nukes spells power nature spell:arcane_barrage(mage) spell:shooting_stars(druid)",
	["ability_mage_brainfreeze"] = "white spell:brain_freeze(mage)",
	["ability_mage_burnout"] = "soul drained magic orange dust posession demonic evil mouth",
	["ability_mage_chilledtothebone"] = "cold chilly ice icy frozen freezing frosty skull snowy blue undeath undead scourge corpse",
	["ability_mage_coldasice"] = "blue ice icy cold frozen freezing frosty shield barrier armor defense protection magic spell shattered power shards broken",
	["ability_mage_conjurefoodrank10"] = "mana strudel conjured food warm yummie",
	["ability_mage_conjurefoodrank12"] = "blue potion bottled water liquid remedy tonic medicine healing restoration",
	["ability_mage_conjurefoodrank9"] = "conjured mana pie food whipped cream custard yummie",
	["ability_mage_conjurewater11"] = "blue champagne bottled wine water juice fancy potion medicine tonic remedy restoration",
	["ability_mage_deepfreeze"] = "frozen cold chilly blue ice icy freezing frosty magic power spell:deep_freeze(mage)",
	["ability_mage_fierypayback"] = "fire fiery burning hot heat flames flaming blazing scorching searing fury rage magic power destruction demonic red orange",
	["ability_mage_firestarter"] = "red orange yellow fiery burning hot heat flames flaming blazing scorching searing blast burst magic pillar spell:critical_mass(mage) spell:sunfire(druid)",
	["ability_mage_frostfirebolt"] = "magic ice icy cold frozen freezing frosty fiery burning hot heat flames flaming blazing scorching searing missile nuke power red orange blue spell:frostfire_bolt(mage)",
	["ability_mage_frostjaw"] = "blue human head icy cold frozen freezing frosty trapped iceblock spell:frostjaw(mage)",
	["ability_mage_greaterinvisibility"] = "purple violet blue mage witch invisibility wizard's hat glowing radiant radiance",
	["ability_mage_hotstreak"] = "fire fiery burning hot heat flames flaming blazing scorching searing hand blood red orange magic spell power",
	["ability_mage_improvedpolymorph"] = "white wool critter animal creature sheep lamb polymorph",
	["ability_mage_incantersabsorbtion"] = "rune of power symbol hand red outline silhouette magic spell",
	["ability_mage_invisibility"] = "blue stealth fade cloaked hidden witch magic powerful wizard spell:invisibility(mage) spell:invisibility(rogue)",
	["ability_mage_livingbomb"] = "fire fiery burning hot heat magic red orange power spell:mage_bomb(mage)",
	["ability_mage_missilebarrage"] = "magic power bolts fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty arcane purple spell:mage_bomb(mage)",
	["ability_mage_moltenarmor"] = "fire fiery burning hot heat flames flaming blazing scorching searing shield barrier defense protection magic ward absorb red orange power elemental spell:molten_armor(mage)",
	["ability_mage_netherwindpresence"] = "arcane purple star flower magic power",
	["ability_mage_potentspirit"] = "demonic skull face hooded cloaked shadow unholy purple evil magic power spell:soul_harvest(warlock)",
	["ability_mage_shattershield"] = "blue ice icy cold frozen freezing frosty shield armor defense protection blocking barrier magic spell destroy shatter chilly defend rune of power symbol emblem sigil seal",
	["ability_mage_studentofthemind"] = "brown mind tap thoughts telepathy power magic spell:train_of_thought(priest)",
	["ability_mage_timewarp"] = "purple golden yellow hourglass twisted bent bending sands of bloodlust magic power spell:time_warp(mage)",
	["ability_mage_tormentoftheweak"] = "fear intimidation terror horror blue red cold frozen man chilly ice icy freezing frosty freeze",
	["ability_mage_wintersgrasp"] = "cold chilly ice icy frozen freezing frosty blue skeleton skeletal bones hand claw magic power spell:fingers_of_frost(mage)",
	["ability_mage_worldinflames"] = "fire fiery burning hot heat flames flaming blazing scorching searing orange people ash charred magic spell power",
	["ability_marksmanship"] = "red silver target throwing knife thrown weapon aim precision marksmanship bullseye spell:bombardment(hunter)",
	["ability_meleedamage"] = "silver steel metal blade weapon damage racial_passive:sword_specialization(human) spell:overpower(warrior)",
	["ability_miling"] = "brown mortar and pestle bowl flowers grinding herbs mash inks professions:milling",
	["ability_monk_ascension"] = "white black bluegreen turquoise cyan bear pandaren furry fuzzy beast animal creature female",
	["ability_monk_blackoutkick"] = "dwarf mustache head dizzy confused confusion stars above knocked knockout bumped",
	["ability_monk_boughstrike"] = "brown bluegreen turquoise cyan branch totem wooden battering glowing ramming enchanted enchantment",
	["ability_monk_breathoffire"] = "red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing breathing of spicy",
	["ability_monk_chargingoxwave"] = "yellow horns horned ox animal creature glowing charging charge running ramming",
	["ability_monk_chibrew"] = "bluegreen turquoise cyan jug jar drink liquid flask potion elixir bottle",
	["ability_monk_chicocoon"] = "bluegreen turquoise cyan gong symbol lightning sphere circle orb power magic spell",
	["ability_monk_chiwave"] = "bluegreen turquoise cyan bolt missile nuke magic spell power arcing chi fountain",
	["ability_monk_clashingoxcharge"] = "yellow orange impacting hitting smashing ramming butting heads rams tauren bulls horned horns furry fuzzy beasts animals creatures charging dueling",
	["ability_monk_cracklingjadelightning"] = "bluegreen turquoise cyan lightning thunder bolt magic spell power",
	["ability_monk_cranekick"] = "pink red whirlwind tornado cyclone windy gusty gusts swirling spiraling twister twisting vortex feathers whirling spinning",
	["ability_monk_cranekick_new"] = "green claws crane kicking foot bird winged feathered feathers animal creature",
	["ability_monk_dampenharm"] = "yellow orange golden tauren bull horned horns furry fuzzy beast animal creature radiant radiance",
	["ability_monk_deadlyreach"] = "blue purple brown human hand reaching grasping punching fingers",
	["ability_monk_dpsstance"] = "red pandaren bear furry fuzzy beast animal creature growling angry aggressive aggression anger fierce ferocity",
	["ability_monk_dragonkick"] = "green chinese cloud serpent dragon beast animal creature scaled scales scaly reptile spirit",
	["ability_monk_drunkenhaze"] = "yellow brown root beer mead booze ale alcohol drink liquid mug cup wooden spilling",
	["ability_monk_elusiveale"] = "yellow silver honey pot jug jar paw print root beer mead booze ale alcohol drink liquid",
	["ability_monk_energizingwine"] = "green flask potion elixir bottle jar drink wine alcohol poison glass corked",
	["ability_monk_expelharm"] = "bluegreen turquoise swirly swirls smoke",
	["ability_monk_explodingjadeblossom"] = "green jade flower blossoming explosion exploding glowing radiant radiance petals crystal",
	["ability_monk_flyingdragonkick"] = "bluegreen white turquoise jade cloud serpent dragon beast animal creature scaled scales scaly reptile spirit",
	["ability_monk_forcesphere"] = "light blue cyan turquoise will-o'-the-wisp ball orb circle sphere radiant radiance glowing spirit ghost",
	["ability_monk_fortifyingale"] = "black yellow root beer mead booze ale alcohol drink liquid jug jar flask potion elixir bottle",
	["ability_monk_fortifyingale_new"] = "white yellow root beer mead booze ale alcohol drink liquid jug jar flask potion elixir bottle",
	["ability_monk_guard"] = "golden yellow glowing radiant radiance emblem sigil symbol circle round fire fiery burning hot heat flames flaming blazing scorching searing",
	["ability_monk_healthsphere"] = "bluegreen turquoise cyan water globe ball sphere circle gemstone bubble marble",
	["ability_monk_jab"] = "black bluegreen turquoise cyan pandaren bear furry fuzzy beast animal creature worgen fist punching jab",
	["ability_monk_jasmineforcetea"] = "jug jar potion drink cup",
	["ability_monk_leeroftheox"] = "white yellow tauren bull horned horns furry fuzzy beast animal creature face head glowing eyes skull bone skeletal",
	["ability_monk_legacyoftheemperor"] = "bluegreen bell golden yellow metal brass decorative outlined",
	["ability_monk_legsweep"] = "yellow brown leg foot pawpads pawprint pandaren bear furry fuzzy beast animal creature worgen kicking sweeping swipe tripping falling",
	["ability_monk_palmstrike"] = "black yellow orange bandaged arms fists hands wrapped punching pandaren bear furry fuzzy beast animal creature",
	["ability_monk_paralysis"] = "purple brown human hand reaching grasping punching fingers",
	["ability_monk_powerstrikes"] = "brown yellow wooden battering ramming hitting impacting slamming silver steel metal power",
	["ability_monk_prideofthetiger"] = "bluegreen turquoise cyan yellow golden emblem sigil outlined star circle round tiger cat kitty feline furry fuzzy beast animal creature",
	["ability_monk_provoke"] = "black white red pandaren bear furry fuzzy beast animal creature growling angry anger ferocity fierce face",
	["ability_monk_quipunch"] = "green black white pandaren bear furry fuzzy beast animal creature punch",
	["ability_monk_quitornado"] = "bluegreen cyan turquoise drill spinning spiraling tornado cyclone windy gusty gusts whirlwind swirling twister twisting vortex",
	["ability_monk_renewingmists"] = "bluegreen turquoise cyan starry sky clouds partly cloudy mists",
	["ability_monk_risingsunkick"] = "red yellow orange white kicking sweeping swipe fire fiery burning hot heat flames flaming blazing scorching searing",
	["ability_monk_roll"] = "green arrow roll spin twist rotate turn pandaren bear furry fuzzy beast animal creature monk",
	["ability_monk_roundhousekick"] = "pink black foot paw pandaren bear furry fuzzy beast animal creature kicking stepping",
	["ability_monk_rushingjadewind"] = "bluegreen turquoise cyan tornado cyclone windy gusty gusts whirlwind swirling spiraling twister twisting vortex",
	["ability_monk_shuffle"] = "bluegreen turquoise cyan white outlined pandaren bear furry fuzzy beast animal creature monk anger angry ferocity fierce",
	["ability_monk_soothingmists"] = "bluegreen turquoise cyan glowing beanstalk plant nature magic spell spirits",
	["ability_monk_sparring"] = "yellow orange weapons silver steel metal sword blade wooden staff clashing crossed hitting impacting radiant radiance",
	["ability_monk_spearhand"] = "yellow golden weapon ammunition ammo arrowhead tip sharp point piercing pierce",
	["ability_monk_staffstrike"] = "brown red wooden staff weapon strike hit smack whack",
	["ability_monk_standingkick"] = "yellow orange black white pandaren bear furry fuzzy beast animal creature monk standing feet paws foot",
	["ability_monk_summonserpentstatue"] = "bluegreen turquoise cyan cloud serpent dragon beast animal creature scaled scales scaly reptile head ghastly ghostly spirit",
	["ability_monk_summontigerstatue"] = "white black blue tiger striped cat kitty feline furry fuzzy beast animal creature head face",
	["ability_monk_surgingmist"] = "bluegreen turquoise cloud beam laser powerful explosion",
	["ability_monk_thunderfocustea"] = "blue white mug cup drink liquid smoking steaming candle magic potion",
	["ability_monk_tigereyebrandy"] = "red orange flask potion elixir bottle jar drink glass",
	["ability_monk_tigerpalm"] = "bluegreen turquoise cyan pawpads hit claws swipe cat kitty feline furry fuzzy beast animal creature pandaren bear",
	["ability_monk_tigerslust"] = "yellow golden glowing foot flying running walking speed flight boot shoe",
	["ability_monk_touchofdeath"] = "red symbol seal emblem sigil blood",
	["ability_monk_touchofkarma"] = "white blue cyan symbol seal emblem sigil",
	["ability_monk_uplift"] = "bluegreen turqoise cyan black pandaren bear furry fuzzy beast animal creature monk clouded shrouded silhouette figure",
	["ability_monk_zenmeditation"] = "bluegreen turquoise cyan symbol emblem sigil seal",
	["ability_mount_alliancepvpmount"] = "battle warhorse steed armored helmet blue mount",
	["ability_mount_armoredbluewindrider"] = "wind rider wyvern horde flying flight mount animal creature armored swift fast head blue purple",
	["ability_mount_bigblizzardbear"] = "white polar bear furry beast animal creature goggles mount",
	["ability_mount_blackbattlestrider"] = "mechanostrider gnome gnomish engineering device machine robot mechanical mount",
	["ability_mount_blackdirewolf"] = "canine dog worg animal furry fuzzy creature beast teeth fangs growling snarling intimidating pet head baring mount:black_wolf mount:brown_wolf mount:swift_brown_wolf mount:timber_wolf",
	["ability_mount_blackpanther"] = "black cat kitty feline furry fuzzy beast animal creature head companion:panther_cub",
	["ability_mount_bluewindrider"] = "wyvern horde flying flight furry animal creature head mount:blue_wind_rider",
	["ability_mount_camel_brown"] = "tol'vir head furry animal creature uldum blue mount:brown_riding_camel",
	["ability_mount_camel_gray"] = "tol'vir camel mount head furry animal creature uldum gray grey green",
	["ability_mount_camel_tan"] = "tol'vir head furry animal creature uldum blue mount:tan_riding_camel",
	["ability_mount_celestialhorse"] = "celestial horse stars mount blue sky flying flight",
	["ability_mount_charger"] = "golden yellow fire fiery burning hot heat flames flaming blazing scorching searing hoofprint horseshoe felsteed dreadsteed mount",
	["ability_mount_cloudmount"] = "yellow golden green gate door shield metal circle round",
	["ability_mount_cockatricemount"] = "silvermoon feathered animal creature beak head purple mount:red_hawkstrider",
	["ability_mount_cockatricemount_black"] = "silvermoon feathered animal creature beak head mount:black_hawkstrider",
	["ability_mount_cockatricemount_blue"] = "silvermoon feathered animal creature beak head mount:blue_hawkstrider",
	["ability_mount_cockatricemount_green"] = "hawkstrider silvermoon mount feathered animal creature beak head green",
	["ability_mount_cockatricemount_purple"] = "silvermoon feathered animal creature beak head pink mount:purple_hawkstrider",
	["ability_mount_cockatricemountelite"] = "silvermoon feathered animal creature beak head armored purple mount:swift_pink_hawkstrider",
	["ability_mount_cockatricemountelite_black"] = "hawkstrider silvermoon mount feathered animal creature beak head armored black",
	["ability_mount_cockatricemountelite_blue"] = "hawkstrider silvermoon mount feathered animal creature beak head armored blue",
	["ability_mount_cockatricemountelite_green"] = "silvermoon feathered animal creature beak head armored turquoise mount:swift_green_hawkstrider",
	["ability_mount_cockatricemountelite_purple"] = "silvermoon feathered animal creature beak head armored pink mount:swift_purple_hawkstrider",
	["ability_mount_cockatricemountelite_white"] = "hawkstrider silvermoon mount feathered animal creature beak head armored white",
	["ability_mount_cranemount"] = "bluegreen turquoise cyan crane mount bird winged feathered feathers animal creature beak head",
	["ability_mount_cranemountblue"] = "blue crane mount bird winged feathered feathers animal creature beak head",
	["ability_mount_cranemountpurple"] = "purple violet crane mount bird winged feathered feathers animal creature beak head",
	["ability_mount_drake_albino"] = "drake mount dragon flight flying head animal creature blue turquoise white",
	["ability_mount_drake_azure"] = "drake mount dragon flight flying head animal creature blue",
	["ability_mount_drake_blue"] = "dragon flight flying head animal creature turquoise blue green mount:albino_drake",
	["ability_mount_drake_bronze"] = "yellow dragon flight flying head animal creature gold mount:bronze_drake_mount",
	["ability_mount_drake_proto"] = "dragon northrend creature head red companion:proto-drake_whelp mount:violet_proto-drake",
	["ability_mount_drake_red"] = "dragon flight flying head animal creature mount:red_drake",
	["ability_mount_drake_twilight"] = "purple dragon flight flying head animal creature twilight mount:black_drake_mount",
	["ability_mount_dreadsteed"] = "yellow orange dreadsteed mount hoofprint horseshoe felsteed",
	["ability_mount_ebonblade"] = "winged steed of ebon blade flying mount flight bird feathered feathers animal creature skeleton skeletal bones undead scourge corpse head",
	["ability_mount_ebongryphon"] = "gryphon alliance bird winged feathered feathers animal creature beak head flight flying mount ebon black burgundy",
	["ability_mount_feldrake"] = "brown green fel demonic evil vicious fierce ferocity anger angry drake mount dragon flight flying head animal creature",
	["ability_mount_fireravengodmount"] = "fire fiery burning hot heat flaming blazing scorching searing bird winged feathered feathers animal creature flying flight raven beak head mount spell:flames_of_xoroth(warlock)",
	["ability_mount_flyingcarpet"] = "flying carpet elegant mount tailoring cloth flight red",
	["ability_mount_fossilizedraptor"] = "white gray grey bones skeleton skeletal undead corpse mount:fossilized_raptor",
	["ability_mount_frostyflyingcarpet"] = "flying carpet elegant mount tailoring cloth flight frozen cold ice icy freezing frosty blue",
	["ability_mount_goatmount"] = "silver green gray grey iron metal bell goat mount",
	["ability_mount_goatmountblack"] = "black purple violet iron metal bell goat mount",
	["ability_mount_goatmountbrown"] = "red iron metal bell goat mount",
	["ability_mount_goatmountwhite"] = "white bluegreen cyan turquoise iron metal bell goat mount",
	["ability_mount_goldengryphon"] = "gryphon alliance bird winged feathered feathers animal creature beak head flight flying mount golden brown white yellow",
	["ability_mount_greenwindrider"] = "wyvern horde flying flight animal creature purple mount:green_wind_rider",
	["ability_mount_gryphon_01"] = "gryphon alliance bird winged feathered feathers animal creature beak head flight flying mount armored blue white swift fast",
	["ability_mount_gyrocoptor"] = "blue gnomish gnome engineering gyrocoptor gyrocopter device flying machine flight",
	["ability_mount_gyrocoptorelite"] = "silver gnomish gnome engineering gyrocoptor gyrocopter device flying machine flight face angry anger teeth",
	["ability_mount_hordepvpmount"] = "warbear polar furry beast animal creature red white mount head",
	["ability_mount_hordescorpionamber"] = "orange amber black scorpion scorpid beast animal creature mount",
	["ability_mount_jungletiger"] = "animal feline cat kitty furry fuzzy beast creature head mount striped stripes orange spell:tiger's_fury(druid) spell:aspect_of_the_cheetah(hunter)",
	["ability_mount_kodo_01"] = "animal beast creature head grey mount:gray_kodo mount:great_gray_kodo mount:great_white_kodo mount:white_kodo",
	["ability_mount_kodo_02"] = "kodo animal beast creature mount head green",
	["ability_mount_kodo_03"] = "animal beast creature head mount:brown_kodo mount:great_brown_kodo",
	["ability_mount_kodosunwalkerelite"] = "pink kodo animal beast creature mount head sunwalker swift fast",
	["ability_mount_kotobrewfest"] = "blue brown kodo animal beast creature mount head brewfest",
	["ability_mount_magnificentflyingcarpet"] = "purple flying carpet elegant mount tailoring cloth flight magnificent swift fast",
	["ability_mount_mammoth_black"] = "mammoth furry hairy animal creature big tusks trunk head black",
	["ability_mount_mammoth_black_3seater"] = "mammoth furry hairy animal creature big tusks trunk head 3 seats vehicle blue",
	["ability_mount_mammoth_brown"] = "furry hairy animal creature big tusks trunk head brown mount:wooly_mammoth",
	["ability_mount_mammoth_brown_3seater"] = "furry hairy animal creature big tusks trunk head 3 seats vehicle brown mount:traveler's_tundra_mammoth",
	["ability_mount_mammoth_white"] = "mammoth furry hairy animal creature big tusks trunk head white",
	["ability_mount_mammoth_white_3seater"] = "mammoth furry hairy animal creature big tusks trunk head 3 seats vehicle white",
	["ability_mount_mechastrider"] = "blue gnomish gnome engineering robot device machine mechanostrider",
	["ability_mount_mountainram"] = "ram furry animal creature mountain goat head white",
	["ability_mount_netherdrakeelite"] = "nether drake flying flight mount black purple",
	["ability_mount_netherdrakepurple"] = "nether drake flying flight mount black purple",
	["ability_mount_nightmarehorse"] = "horse dreadsteed undead glowing red eyes head mane mount",
	["ability_mount_onyxpanther"] = "blue black purple green yellow golden panther cat kitty feline furry fuzzy beast animal creature helmet head face glowing eyes mount",
	["ability_mount_onyxpanther_black"] = "purple violet panther cat kitty feline furry fuzzy beast animal creature helmet head face glowing eyes mount",
	["ability_mount_onyxpanther_blue"] = "blue panther cat kitty feline furry fuzzy beast animal creature helmet head face glowing eyes mount",
	["ability_mount_onyxpanther_red"] = "red panther cat kitty feline furry fuzzy beast animal creature helmet head face glowing eyes mount",
	["ability_mount_onyxpanther_yellow"] = "yellow panther cat kitty feline furry fuzzy beast animal creature helmet head face glowing eyes mount",
	["ability_mount_pandaranmount"] = "red cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountblack"] = "blue cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountblue"] = "green cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountbrown"] = "orange blue cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepic"] = "red black cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepicblack"] = "orange black cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepicblue"] = "blue red cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepicbrown"] = "orange brown bluegreen cyan turquoise cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepicpurple"] = "purple yellow green cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountepicred"] = "red green cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountgreen"] = "gray grey orange cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountpurple"] = "purple cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandaranmountred"] = "red green cloud serpent dragon beast animal creature scaled scales scaly reptile head face mount glowing eyes",
	["ability_mount_pandarenkitemount"] = "red kite mount flying",
	["ability_mount_pandarenkitemount_blue"] = "blue golden yellow kite mount flying",
	["ability_mount_pandarenphoenix"] = "white phoenix bird winged feathered feathers animal creature mount flying head glowing eyes",
	["ability_mount_pandarenphoenix_green"] = "purple white green phoenix bird winged feathered feathers animal creature mount flying head glowing eyes",
	["ability_mount_pandarenphoenix_purple"] = "purple green orange phoenix bird winged feathered feathers animal creature mount flying head glowing eyes",
	["ability_mount_pandarenphoenix_red"] = "blue black phoenix bird winged feathered feathers animal creature mount flying head glowing eyes",
	["ability_mount_pandarenphoenix_yellow"] = "white yellow golden phoenix bird winged feathered feathers animal creature mount flying head glowing eyes",
	["ability_mount_pegasus"] = "blue pegasus horse flying flight mount",
	["ability_mount_pinktiger"] = "tiger animal feline cat kitty furry fuzzy beast creature head mount striped stripes snowy ice frost pink purple",
	["ability_mount_polarbear_black"] = "armored polar bear furry beast animal creature teeth anger angry fury furious rage raging aggression intimidating mount head black blue",
	["ability_mount_polarbear_brown"] = "polar furry beast animal creature teeth anger angry fury furious rage raging aggression intimidating head mount:armored_brown_bear",
	["ability_mount_polarbear_white"] = "armored polar bear furry beast animal creature teeth anger angry fury furious rage raging aggression intimidating mount head white",
	["ability_mount_quilenflyingmount"] = "white green quilen flying mount",
	["ability_mount_quilenmount"] = "white quilen flying mount",
	["ability_mount_raptor"] = "dinosaur beast animal creature reptile horn head green companion:obsidian_hatchling mount:emerald_raptor mount:swift_blue_raptor mount:swift_olive_raptor mount:swift_orange_raptor mount:turquoise_raptor mount:violet_raptor",
	["ability_mount_razorscale"] = "dragon northrend creature head blue mount:ironbound_proto-drake",
	["ability_mount_reddrakemount"] = "alexstrasza red dragon head",
	["ability_mount_redfrostwyrm_01"] = "wyrm glowing red eyes creature horns dragon northrend head mount:icebound_frostbrood_vanquisher",
	["ability_mount_ridingelekk"] = "elekk outland nagrand tusks trunk animal creature elephant mount head white gray grey",
	["ability_mount_ridingelekk_grey"] = "elekk outland nagrand tusks trunk animal creature elephant mount head white gray grey",
	["ability_mount_ridingelekk_purple"] = "elekk outland nagrand tusks trunk animal creature elephant mount head purple",
	["ability_mount_ridingelekkelite"] = "elekk outland nagrand tusks trunk animal creature elephant mount head swift armored elite purple pink",
	["ability_mount_ridingelekkelite_blue"] = "elekk outland nagrand tusks trunk animal creature elephant mount head swift armored elite blue",
	["ability_mount_ridingelekkelite_green"] = "elekk outland nagrand tusks trunk animal creature elephant mount head swift armored elite green",
	["ability_mount_ridingelekkelite_purple"] = "elekk outland nagrand tusks trunk animal creature elephant mount head swift armored elite purple",
	["ability_mount_ridinghorse"] = "horse furry animal creature brown head",
	["ability_mount_rocketmount"] = "goblin engineering rocket missile launching firing blastoff liftoff seat red",
	["ability_mount_rocketmount2"] = "goblin engineering shark missile face painted green silver",
	["ability_mount_rocketmount3"] = "blue gray grey silver steel metal rocket bomb explosives firework firecracker flying mount",
	["ability_mount_rocketmount4"] = "blue orange white steel metal rocket bomb explosives missile firework firecracker flying mount",
	["ability_mount_rocketmountblue"] = "goblin engineering rocket missile launching firing blastoff liftoff seat blue",
	["ability_mount_seahorse"] = "vashj'ir ocean swimming animal creature blue head mount:abyssal_seahorse",
	["ability_mount_siberiantigermount"] = "brown blue siberian tiger striped cat kitty feline furry fuzzy beast animal creature mount head",
	["ability_mount_siberiantigermountgreen"] = "green blue siberian tiger striped cat kitty feline furry fuzzy beast animal creature mount head",
	["ability_mount_siberiantigermountred"] = "red purple blue siberian tiger striped cat kitty feline furry fuzzy beast animal creature mount head",
	["ability_mount_snowygryphon"] = "gryphon alliance bird winged feathered feathers animal creature beak head flight flying mount snowy ice icy cold frozen freezing frosty white",
	["ability_mount_spectralgryphon"] = "blue purple white spectral ghostly ghastly spectre gryphon bird feathers head beak glowing eyes",
	["ability_mount_spectraltiger"] = "blue tiger animal cat kitty feline furry fuzzy beast creature spectral ghost spirit apparition undead mount striped stripes",
	["ability_mount_spectralwyvern"] = "blue white orange spectral ghostly ghastly spectre head lion mane glowing eyes",
	["ability_mount_swiftgreenwindrider"] = "wyvern horde flying flight animal creature armored fast head mount:swift_green_wind_rider",
	["ability_mount_swiftpurplewindrider"] = "wyvern horde flying flight animal creature fast head mount:armored_blue_wind_rider mount:swift_purple_wind_rider",
	["ability_mount_swiftredwindrider"] = "wyvern horde flying flight animal creature armored fast head mount:swift_red_wind_rider",
	["ability_mount_swiftyellowwindrider"] = "wyvern horde flying flight animal creature armored fast head golden mount:swift_yellow_wind_rider",
	["ability_mount_tawnywindrider"] = "wyvern horde flying flight furry animal creature head golden yellow mount:tawny_wind_rider",
	["ability_mount_travellersyakmount"] = "green brown yak beast animal creature mount head",
	["ability_mount_twilightdrakemount_black"] = "twilight drake mount flying flight armored black green",
	["ability_mount_twilightdrakemount_blue"] = "twilight drake mount flying flight armored blue",
	["ability_mount_twilightdrakemount_bronze"] = "twilight drake mount flying flight armored yellow blue",
	["ability_mount_twilightdrakemount_green"] = "twilight drake mount flying flight armored green purple",
	["ability_mount_twilightdrakemount_red"] = "twilight drake mount flying flight armored yellow",
	["ability_mount_tyraelmount"] = "blue horse armored plate steel metal head spiritual ghostly ghastly ethereal mount:tyrael's_charger",
	["ability_mount_undeadhorse"] = "skeleton undead demon head creature animal bones corpse mount:black_skeletal_horse mount:blue_skeletal_horse mount:brown_skeletal_horse mount:green_skeletal_warhorse mount:ochre_skeletal_warhorse mount:purple_skeletal_warhorse mount:red_skeletal_horse spell:on_a_pale_horse(death_knight)",
	["ability_mount_warhippogryph"] = "green alliance night elf flying flight firelands mount:flameward_hippogryph",
	["ability_mount_waterstridermount"] = "blue brown war strider animal creature",
	["ability_mount_whitedirewolf"] = "canine dog worg animal furry fuzzy creature beast teeth fangs growling snarling intimidating pet head baring white mount:dire_wolf mount:swift_gray_wolf mount:swift_timber_wolf",
	["ability_mount_whitetiger"] = "tiger animal feline cat kitty furry fuzzy beast creature head mount striped stripes white ice icy cold frozen freezing frosty snowy spell:aspect_of_the_pack(hunter)",
	["ability_mount_wyvern_01"] = "wind rider wyvern horde flying flight mount animal creature armored swift fast head",
	["ability_mount_yakmount"] = "brown blue yak beast animal creature mount head",
	["ability_mount_yakmountblack"] = "purple violet black yak beast animal creature mount head",
	["ability_mount_yakmountbrown"] = "green brown yak beast animal creature mount head",
	["ability_mount_yakmountgrey"] = "orange brown yak beast animal creature mount head",
	["ability_mount_yakmountwhite"] = "black brown bluegreen cyan turquoise yak beast animal creature mount head",
	["ability_paladin_artofwar"] = "yellow light power divine holy magic circle aura spell:the_art_of_war(paladin)",
	["ability_paladin_beaconoflight"] = "beam magic holy power divine healing restoration prayer yellow spell:beacon_of_light(paladin)",
	["ability_paladin_blessedhands"] = "yellow power holy divine magic healing restoration spell:mastery:_hand_of_light(paladin)",
	["ability_paladin_blessedmending"] = "hands light beam prayer holy power magic healing restoration",
	["ability_paladin_blindinglight"] = "golden yellow radiant radiance glowing hammer mace weapon spell:blinding_light(paladin)",
	["ability_paladin_blindinglight2"] = "golden yellow radiant radiance glowing hammer mace weapon",
	["ability_paladin_clemency"] = "brown yellow golden chains metal handcuffs breaking broken freedom",
	["ability_paladin_conviction"] = "paladin golden yellow armor helmet defense protection",
	["ability_paladin_divinestorm"] = "glowing yellow golden metal sword blade weapon power holy shining shine shiny radiating radiance swirly enchanted enchantment spiral spell:divine_storm(paladin)",
	["ability_paladin_enlightenedjudgements"] = "silver hammer mace weapon shiny paladin",
	["ability_paladin_gaurdedbythelight"] = "golen yellow holy light divine healing restoration prayer shield armor defense protection blocking magic",
	["ability_paladin_hammeroftherighteous"] = "silver golden yellow stone mace weapon light holy power magic enchanted enchantment divine spell:hammer_of_the_righteous(paladin)",
	["ability_paladin_holyavenger"] = "yellow golden angel winged wings halo radiant radiance light glowing",
	["ability_paladin_infusionoflight"] = "holy magic dust yellow golden power healing restoration divine spell:infusion_of_light(paladin)",
	["ability_paladin_judgementblue"] = "blue shiny silver hammer mace weapon steel metal paladin smashing striking attacking motion movement",
	["ability_paladin_judgementofthepure"] = "yellow golden judgement hammer light holy divine weapon",
	["ability_paladin_judgementofthewise"] = "blue judgement hammer light holy divine weapon",
	["ability_paladin_judgementred"] = "red shiny silver hammer mace weapon steel metal smashing striking attacking motion movement spell:judgments_of_the_bold(paladin)",
	["ability_paladin_judgementsofthejust"] = "yellow golden seal sigil emblem steel metal hammer mace weapon light holy divine power",
	["ability_paladin_longarmofthelaw"] = "golden yellow reaching touching radiant radiance light rays glowing angelic holy talent:long_arm_of_the_law(paladin)",
	["ability_paladin_protectoroftheinnocent"] = "golden hammer mace weapon purple yellow",
	["ability_paladin_righteousvengeance"] = "paladin yellow holy power magic blessed",
	["ability_paladin_sacredcleansing"] = "healing touch light holy power magic restoration yellow golden spell:sacred_cleansing(paladin)",
	["ability_paladin_sanctifiedwrath"] = "red golden yellow hammer mace weapon",
	["ability_paladin_selflesshealer"] = "yellow golden glowing radiant radiance human naked healing restoration",
	["ability_paladin_sheathoflight"] = "golden metal blade weapon yellow holy power divine spell:sword_of_light(paladin)",
	["ability_paladin_shieldofthetemplar"] = "shield armor defense protection blocking barrier magic spell light golden yellow divine power enchanted enchantment",
	["ability_paladin_shieldofvengeance"] = "armor defense protection blocking golden purple yellow spell:shield_of_the_righteous(paladin)",
	["ability_paladin_speedoflight"] = "yellow golden fast running speeding silhouette figure talent:speed_of_light(paladin)",
	["ability_paladin_swiftretribution"] = "enchanted enchantment steel metal sword blade weapon purple light divine holy magic power",
	["ability_paladin_touchedbylight"] = "bubble armor yellow purple golden power magic defense spell:guarded_by_the_light(paladin)",
	["ability_paladin_turnevil"] = "yellow green goblin orc skull bone skeletal magic spell:turn_evil(paladin)",
	["ability_paladin_veneration"] = "yellow golden winged feathered feathers animal creature glowing radiant radiance wings hummingbird holy angelic talent:pursuit_of_justice(paladin)",
	["ability_parry"] = "silver steel metal swords blades weapons daggers knives knife clash battle duel attack blue passive:parry(death_knight) passive:parry(rogue) passive:parry(paladin) spell:parry(warrior)",
	["ability_pet_baneling"] = "purple green baneling starcraft zerg animal creature",
	["ability_physical_taunt"] = "mouth teeth fangs anger angry fury furious rage raging aggression bite intimidate intimidating red spell:growl(druid) spell:beast_lore(hunter) pet_command:growl(hunter)",
	["ability_piercedamage"] = "silver arrowhead weapon tip point shoot",
	["ability_poisonarrow"] = "poison toxic arrowhead tip point venom green goo deadly weapon",
	["ability_poisons"] = "toxic arrowhead tip point venom green goo deadly weapon spell:improved_poisons(rogue)",
	["ability_poisonsting"] = "purple green caterpillar venomous poisonous bite sting worm deadly spell:crippling_poison(rogue)",
	["ability_priest_angelicbulwark"] = "yellow golden shield winged wings metal radiant radiance glowing holy",
	["ability_priest_angelicfeather"] = "yellow golden quill fast speed flight talent:angelic_feather(priest)",
	["ability_priest_archangel"] = "golden yellow holy light power magic wings spirit dying dead revive healing restoration spell:archangel(priest)",
	["ability_priest_atonement"] = "armor yellow spell:atonement(priest)",
	["ability_priest_bindingprayers"] = "purple book reading praying",
	["ability_priest_cascade"] = "yellow golden sunlight rays point source beams starlight",
	["ability_priest_cascade_shadow"] = "blue purple twilight rays point source sun beams starlight",
	["ability_priest_darkness"] = "shadowy unholy cloaked hidden mysterious pale face purple shadows rogue evil",
	["ability_priest_evangelism"] = "yellow golden purple hammer mace weapon gemmed power magic spell enchanted enchantment holy light",
	["ability_priest_flashoflight"] = "yellow golden rays sunlight lightbeam fire fiery burning hot heat flames flaming blazing scorching searing holy",
	["ability_priest_focusedwill"] = "prayer hands red blue willpower light magic",
	["ability_priest_halo"] = "yellow golden round orbs sun star glowing radiant radiance lights halo ring",
	["ability_priest_halo_shadow"] = "purple violet round orbs sun star glowing radiant radiance lights halo ring",
	["ability_priest_heavanlyvoice"] = "blue summoned demonic creature horns singing yellow head succubus",
	["ability_priest_pathofthedevout"] = "blue white cyan turquoise winged wings feathered feathers heart life angelic lifting flying flight",
	["ability_priest_phantasm"] = "black white ghost spirit figure hooded shrouded spectre glowing eyes silhouette assassin rogue talent:phantasm(priest)",
	["ability_priest_reflectiveshield"] = "golden yellow shield armor defense protection blocking purple beam spell power light holy reflected reflecting magic",
	["ability_priest_shadowyapparition"] = "unholy figure glowing red eyes purple cloaked evil spell:shadowy_apparitions(priest)",
	["ability_priest_silence"] = "shadow unholy purple face mask green eyes magic power spell:silence(priest)",
	["ability_priest_soulwarding"] = "golden yellow holy shield armor defense protection blocking winged wings flying flight power radiance radiant healing restoration",
	["ability_priest_surgeofdarkness"] = "purple violet circle ring magic spell curse",
	["ability_priest_voidshift"] = "yellow orange arrows refresh rotate rotating spinning turning",
	["ability_racial_aberration"] = "green broken ripped skull undead disease corpse plague magic spell power scourge",
	["ability_racial_avatar"] = "dwarf yelling blue white shouting spell:berserker_stance(warrior)",
	["ability_racial_bearform"] = "blue outline silhouette transformation pawprint animal creature footprint furry beast nature magic power spell:bear_form(druid) tab:guardian(druid)",
	["ability_racial_bestdealsanywhere"] = "green money dollar signs symbols cash gold banker auctioneer sales vendor treasure racial_passive:best_deals_anywhere(goblin)",
	["ability_racial_betterlivingthroughchemistry"] = "green vial potion science scientific purple radiance radiant brilliance discovery learning racial_passive:better_living_through_chemistry(goblin)",
	["ability_racial_bloodrage"] = "red roar shouting storm giant anger angry fury furious raging aggression intimidation yelling spell:enrage(warrior)",
	["ability_racial_cannibalize"] = "red ghoul teeth eating diseased gums scourge corpse spell:primal_fury(druid) spell:savage_defense(druid) racial:cannibalize(undead)",
	["ability_racial_darkflight"] = "foot paw running dashing walking claws purple black furry animal creature stepping racial:darkflight(worgen)",
	["ability_racial_flayer"] = "brown claw hand reaching grasping",
	["ability_racial_packhobgoblin"] = "golden yellow money cash bank treasure chest treasury racial:pack_hobgoblin(goblin)",
	["ability_racial_rocketbarrage"] = "red blue rocket missile bomb goblin engineering weapon",
	["ability_racial_rocketjump"] = "engineering boots flying flight boost blast outline silhouette red racial:rocket_jump(goblin)",
	["ability_racial_runningwild"] = "wolf canine dog animal furry fuzzy creature beast fleeing outline blue black silhouette racial:running_wild(worgen)",
	["ability_racial_shadowmeld"] = "purple face racial_passive:quickness(night_elf)",
	["ability_racial_timeismoney"] = "white purple golden yellow clock cash pocketwatch racial_passive:time_is_money(goblin)",
	["ability_racial_twoforms"] = "purple brown black transformation worgen switch forms human magic spell power",
	["ability_racial_ultravision"] = "purple green heat vision centaur cenarius magic spell power racial_passive:elusiveness(night_elf)",
	["ability_racial_viciousness"] = "black pink worgen glowing eyes wolf canine dog animal furry fuzzy creature beast face head",
	["ability_repair"] = "silver hammer mace weapon nail hard work pounding repairing fixing mending building constructing carpentry tools",
	["ability_rewindtime"] = "orange undo gears rewind revert reset reverse clockwork machine mechanical cogs turning spinning",
	["ability_rhyolith_immolation"] = "fire fiery burning hot heat red orange magma lava liquid core",
	["ability_rhyolith_lavapool"] = "fire fiery burning hot heat red orange magma lava liquid pool volcano mountain surface",
	["ability_rhyolith_magmaflow_wave"] = "fire fiery burning hot heat red orange",
	["ability_rhyolith_magmaflow_whole"] = "fire fiery burning hot heat red orange magma lava liquid pool volcano mountain flowing",
	["ability_rhyolith_volcano"] = "fire fiery burning hot heat red orange magma lava liquid pool volcano mountain erupting",
	["ability_rogue_ambush"] = "silver steel metal dagger blade weapon knife sword stab red orange spell:ambush(rogue) spell:heroic_strike(warrior)",
	["ability_rogue_blackjack"] = "mace weapon spikes blood bashing gold",
	["ability_rogue_bladetwisting"] = "silver steel metal dagger blade weapon knife sword spinning twirling rotating",
	["ability_rogue_bloodsplatter"] = "blood spattering piercing puncturing wounding stabbing knife silver steel metal dagger blade weapon fist red",
	["ability_rogue_bloodyeye"] = "bloody eyeball wounded stabbed gash scarred spell:scent_of_blood(death_knight)",
	["ability_rogue_cheatdeath"] = "silver steel metal sword blade weapon lucky break green missed boots legs between",
	["ability_rogue_combatexpertise"] = "target throwing weapon knife silver steel metal dagger blade bullseye hit aim precision marksmanship",
	["ability_rogue_combatexpertisetga"] = "target throwing weapon knife silver steel metal dagger blade bullseye hit aim precision marksmanship",
	["ability_rogue_combatreadiness"] = "golden metal yellow swords blades weapons daggers knives knife shield armor defense protection blocking talent:combat_readiness(rogue)",
	["ability_rogue_cuttothechase"] = "weapon knife poison toxic silver steel metal dagger blade blood green swing strike deadly spell:cut_to_the_chase(rogue)",
	["ability_rogue_deadenednerves"] = "green brain nerves nervous system connections tree",
	["ability_rogue_deadliness"] = "red cloaked shadowy shadowed unholy figure glowing eyes knife silver steel metal dagger blade weapon stab assassinate assassination evil spell:vendetta(rogue)",
	["ability_rogue_deadlybrew"] = "deadly silver steel metal poison toxic daggers blades weapons knives knife swords double two green",
	["ability_rogue_deadlymomentum"] = "purple flying skull teeth head moving undead evil magic spell power scourge corpse",
	["ability_rogue_deviouspoisons"] = "deadly poison toxic pool skull undead glowing eyes green evil scourge corpse",
	["ability_rogue_dirtydeeds"] = "glowing red eyes black shadowy unholy figure cloaked hidden shadowed knife silver steel metal dagger blade weapon teeth evil grin",
	["ability_rogue_disembowel"] = "green silver poison toxic guts steel metal sword weapon dagger knife stabbed spell:envenom(rogue) passive:swiftblade's_cunning(rogue)",
	["ability_rogue_disguise"] = "white porcelain mask false hidden identity disguise secret facade deception deceipt",
	["ability_rogue_dismantle"] = "yellow brown golden metal steel sword blade weapon shield armor defense protection blocking light spell:dismantle(rogue)",
	["ability_rogue_distract"] = "bomb fuse lit tossed thrown weapon explosivespurple spell:distract(rogue)",
	["ability_rogue_dualweild"] = "toxic knives knife green goo double two spell:deadly_poison(rogue)",
	["ability_rogue_energeticrecovery"] = "hooded cloaked shadowy unholy figure energy green yellow power magic light spell",
	["ability_rogue_envelopingshadows"] = "shadows unholy monsters demons creatures purple faces evil spell:shadow_walk(rogue)",
	["ability_rogue_eviscerate"] = "silver steel metal man threatened execution sword blade weapon tab:assassination(rogue) spell:eviscerate(rogue)",
	["ability_rogue_fanofknives"] = "silver steel metal thrown throwing purple daggers blades weapons knife swords array multiple spell:fan_of_knives(rogue)",
	["ability_rogue_feigndeath"] = "blue-green cyan skull and crossbones jolly roger undeath undead evil cursed deadly poisonous toxic corpse spell:feign_death(hunter)",
	["ability_rogue_feint"] = "silver steel metal shadows blue sword blade weapon dagger knife spell:feint(rogue) spell:disengage(hunter)",
	["ability_rogue_findweakness"] = "green targeting reticle spot beast demonic creature spell:find_weakness(rogue)",
	["ability_rogue_fleetfooted"] = "boot grass running ground walking fast speedy swift fleet footed green brown",
	["ability_rogue_focusedattacks"] = "dagger blade weapon knife purple steel metal sword attack spell:blindside(rogue)",
	["ability_rogue_garrote"] = "barbed wire fence bloody barbs stained red spell:garrote(rogue)",
	["ability_rogue_honoramongstthieves"] = "triangle steel metal swords blades weapons yellow daggers knives knife hands pact partners friends spell:honor_among_thieves(rogue)",
	["ability_rogue_hungerforblood"] = "silver green orcish licking steel metal dagger blade weapon knife sword bloody mouth yellow thirst guts spell:taste_for_blood(warrior)",
	["ability_rogue_improvedrecuperate"] = "green hooded cloaked shadowy unholy figure evil glowing eyes light power magic spell rogue",
	["ability_rogue_imrovedrecuperate"] = "blue green man naked bare chest light power magic spell recovery restoration healing",
	["ability_rogue_kidneyshot"] = "bare fist knuckles impact blow punch blood red clenched power spell:kidney_shot(rogue)",
	["ability_rogue_masterofsubtlety"] = "purple cloaked shadowy unholy figure evil warlock contemplating thinking hiding hidden sneaking spell:master_of_subtlety(rogue)",
	["ability_rogue_murderspree"] = "silver steel metal swords blades weapons daggers knives knife red yellow orange mask spell:killing_spree(rogue)",
	["ability_rogue_nervesofsteel"] = "hardened nerves purple brain connected connections",
	["ability_rogue_preparation"] = "blue war paint man face fingers preparation",
	["ability_rogue_preyontheweak"] = "blue hand suffocation strangulation strangle reaching snake viper fangs tongue venomous python spell:bandit's_guile(rogue)",
	["ability_rogue_quickrecovery"] = "green surfing back on his feet getting up recovering swift fast bounce rebound passive:fleet_footed(rogue)",
	["ability_rogue_recuperate"] = "bloody arm gushing wounded blue scarred scratched muscles clenched fist knuckles spell:recuperate(rogue)",
	["ability_rogue_redirect"] = "missile knife bouncing ricocheting blade silver steel metal dagger weapon throwing thrown fired orange fiery burning hot heat spell:redirect(rogue)",
	["ability_rogue_reinforcedleather"] = "silver brown mace weapon nail carpentry hitting hammering smashing impact fixing repairing maintaining building constructing tools",
	["ability_rogue_restlessblades"] = "silver steel metal daggers weapons knives knife swords spell:restless_blades(rogue)",
	["ability_rogue_rupture"] = "black red pierce puncture wound stab knife blade fist weapon knuckles tear rip spell:rupture(rogue)",
	["ability_rogue_ruthlessness"] = "blood wound spattered",
	["ability_rogue_sanguinaryvein"] = "blood vessels veins arteries red purple blue spell:sanguinary_vein(rogue)",
	["ability_rogue_sealfate"] = "scythe red silver blade farming tool weapon",
	["ability_rogue_shadowdance"] = "shadows unholy demons demonic creatures blue red black ghosts spirits apparitions dancing flying floating evil spell:shadow_dance(rogue)",
	["ability_rogue_shadowstep"] = "red purple blue shadowy unholy cloaked figure moving sneaking hiding hidden evil mourning veiled black",
	["ability_rogue_shadowstrikes"] = "blue outlines silhouettes swords blades weapons daggers knives knife crossed conflicting conflicted battle attack spell:mutilate(rogue)",
	["ability_rogue_shroudofconcealment"] = "light blue cyan turquoise hooded figure human enchanted enchantment shrouded spell:shroud_of_concealment(rogue)",
	["ability_rogue_sinistercalling"] = "blue red orange fire fiery burning hot heat flames flaming blazing scorching searing face glowing eyes evil demonic creature power spell:assassin's_resolve(rogue) spell:sinister_calling(rogue)",
	["ability_rogue_slaughterfromtheshadows"] = "assassination assassinate bloody silver steel metal sword blade weapon dagger knife sneaking shadowy unholy figure yellow red evil black",
	["ability_rogue_slicedice"] = "silver red orange yellow attack swing cleave steel metal sword blade weapon dagger knife trailing glowing spell:slice_and_dice(rogue) spell:sweeping_strikes(warrior)",
	["ability_rogue_smoke"] = "hidden obscured vision sight cloud dust powder pellet explosives spell:smoke_bomb(rogue)",
	["ability_rogue_sprint"] = "swift fast flying boots heel winged orange red dash running shoes passive:felguard:_pursuit(warlock) spell:sprint(rogue)",
	["ability_rogue_stayofexecution"] = "purple yellow masked cloaked figure evil leader signal hooded sentence wrath criminal crime spell:seal_fate(rogue)",
	["ability_rogue_sturdyrecuperate"] = "green outline silhouette person light power radiance radiant glowing healing restoration magic spell:energetic_recovery(rogue)",
	["ability_rogue_surpriseattack"] = "silver ambush stab steel metal sword blade weapon dagger knife sneak attack from behind head green person man",
	["ability_rogue_surpriseattack2"] = "blue rogue stealthed stealthy hidden ambush sneak attack surprise dagger hooded shrouded figure",
	["ability_rogue_throwingspecialization"] = "yellow seeds daggers thrown weapons blades knives knife light",
	["ability_rogue_tricksofthetrade"] = "pocket swiss army utility knifes tools goo green shiny metal steel sharp spell:tricks_of_the_trade(rogue)",
	["ability_rogue_trip"] = "falling tripping running chain booby trap boot foot yellow gray grey impact stumble stumbling racial:escape_artist(gnome)",
	["ability_rogue_turnthetables"] = "steel metal swords blades weapons clashing battle battling yellow swinging moving motion strike striking counterattack surprise",
	["ability_rogue_unfairadvantage"] = "strike weapon purple pink sneak attack",
	["ability_rogue_vendetta"] = "red skull glowing green eyes evil demonic creature undead poison toxic deadly disease plague scourge corpse head face spell:scarlet_fever(death_knight)",
	["ability_rogue_venomouswounds"] = "green poison toxic goo liquid gash scar scratch infected disease spell:venomous_wounds(rogue)",
	["ability_rogue_versatility"] = "yellow silver gray grey steel metal sword blade weapon seeking zig zagging swerving",
	["ability_rogue_vigor"] = "light yellow purple cloaked figure hooded shrouded shadowy unholy evil glowing eyes rogue",
	["ability_rogue_waylay"] = "silver fire fiery burning hot heat flames flaming blazing scorching searing forging forged steel metal sword blade weapon dagger knife yellow orange red",
	["ability_rogue_wrongfullyaccused"] = "sad unhappy cloaked hooded figure shadowy unholy misunderstood gray grey sentenced judged mourning loss",
	["ability_sap"] = "mace weapon spikes blood bashing silver spell:sap(rogue)",
	["ability_seal"] = "white black baby seal critter creature animal furry face pet_stance:passive(hunter)",
	["ability_searingarrow"] = "spearhead fire fiery burning hot heat flames flaming blazing scorching searing yellow orange arrow weapon",
	["ability_shaman_ancestralguidance"] = "blue moon shrouded hooded figure spiritual circle sphere orb staff monk silhouette",
	["ability_shaman_ascendance"] = "white red bluegreen turquoise cyan elemental elements orbs marbles circles spheres smoking fire fiery burning hot heat flames flaming blazing scorching searing",
	["ability_shaman_astralshift"] = "red purple sky space wave blast magical explosion cosmic cosmos spell:spirit_shell(priest) talent:astral_shift(shaman)",
	["ability_shaman_cleansespirit"] = "black hand claw light power magic tauren spell:nature's_cure(druid) spell:cleanse_spirit(shaman) spell:purify_spirit(shaman)",
	["ability_shaman_echooftheelements"] = "red yellow orange blue green bolts missiles nukes magic spells power fireballs",
	["ability_shaman_fortifyingwaters"] = "white blue purple banner glowing radiant radiance circle aura",
	["ability_shaman_freedomwolf"] = "purple pink moon wolf canine dog animal furry fuzzy creature beast sphere orb circle howling worgen",
	["ability_shaman_healingtide"] = "brown bluegreen cyan turquoise wooden totem shaman aura glowing",
	["ability_shaman_heroism"] = "red battleaxe blade weapon gemmed elegant swinging moving motion arm hand",
	["ability_shaman_lavalash"] = "hammer mace weapon fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty elements elemental smashing swinging moving motion nature's power magic enchanted enchantment red yellow orange blue spell:lava_lash(shaman)",
	["ability_shaman_multitotemactivation"] = "bluegreen totems auras lights swirly swirls swirling",
	["ability_shaman_repulsiontotem"] = "bluegreen turquoise cyan totem light radiant radiance glowing",
	["ability_shaman_stonebulwark"] = "brown gray grey green talent:stone_bulwark_totem(shaman)",
	["ability_shaman_stormstrike"] = "blue silver steel metal mace weapon striking swinging moving motion lightning thunder nature's power enchanted enchantment magic elemental spell:stormstrike(shaman)",
	["ability_shaman_totemcooldownrefund"] = "green blue shaman totem target center circle reticle aura",
	["ability_shaman_totemrelocation"] = "blue green cyan turquoise brown red totems",
	["ability_shaman_tranquilmindtotem"] = "purple brown blue wooden spell:stormlash_totem(shaman)",
	["ability_shaman_watershield"] = "blue ice icy cold frozen freezing frosty bubbles sea ocean liquid barrier armor defense protection magic ward absorb absorption elemental power spell:water_shield(shaman) spell:resurgence(shaman)",
	["ability_shaman_windwalktotem"] = "brown green white gray grey windy air breeze talent:windwalk_totem(shaman)",
	["ability_shockwave"] = "fire fiery burning hot heat flames flaming blazing scorching searing battleaxe blade weapon yellow orange red spell:hamstring(warrior)",
	["ability_shootwand"] = "wand weapon ranged tip spear arrowhead glowing enchanted enchantment magic power light diamond gemstone arcane ice icy cold frozen freezing frosty white blue spell:shoot(mage) spell:shoot(warlock) spell:shoot(priest)",
	["ability_smash"] = "brown tree log hammer mace weapon striking smashing hitting blow impact swinging moving motion",
	["ability_spy"] = "brown silver map x marks the spot steel metal dagger blade weapon knife sword trails parchment paper papyrus writing battle plan strategy passive:detect_traps(rogue)",
	["ability_stealth"] = "cloaked hooded assassination assassinate hiding hidden green blue leaves face evil spell:stealth(rogue) tab:subtlety(rogue) talent:nightstalker(rogue)",
	["ability_steelmelee"] = "silver steel metal sword blade weapon dagger knife",
	["ability_suffocate"] = "blue pink drowning asphyxiation suffocation bulging eyes bubbles sea ocean fatigue",
	["ability_tetanus"] = "red orange goo infection disease wound spill liquid",
	["ability_theblackarrow"] = "black tip arrowhead point weapon spearhead purple talent:silencing_shot(hunter)",
	["ability_throw"] = "silver throwing knife steel metal dagger blade weapon purple spell:throw(rogue) spell:throw(warrior)",
	["ability_thunderbolt"] = "paladin thrown weapon thor's hammer mace thunder lightning smash power enchanted enchantment gray grey blue",
	["ability_thunderclap"] = "humanoid thunder smashing striking impacting hitting thor's hammer mace weapon lightning power magic enchanted enchantment blow swinging moving motion hand arm wristband gray grey blue",
	["ability_toughness"] = "blue outline silhouette hulk muscles strength strong toughness power light impressive display spell:rallying_cry(warrior)",
	["ability_townwatch"] = "town portal valley mountain scenery background landscape daytime trees circle sky globe vision sight telescope scrying green blue white",
	["ability_tracking"] = "black red animal tracks bear furry beast creature wolf canine dog worg fuzzy pawprints footprints spell:track_humanoids(druid) spell:tracking(hunter) pet_command:follow(hunter) spell:spirit_walk(shaman)",
	["ability_trueshot"] = "arrowhead tip spearhead white spell:trueshot_aura(hunter)",
	["ability_upgrademoonglaive"] = "glaive thrown weapon blade spikes spinning purple spell:multi-shot(hunter)",
	["ability_vanish"] = "ghost released spirit apparition glowing eyes wisp shadow unholy aura grey gray smoke powder cloud spell:vanish(rogue)",
	["ability_vehicle_demolisherflamecatapult"] = "red yellow orange fiery burning hot heat flames flaming blazing scorching searing iron metal cannonball stone bullet fireball catapult ammunition ammo weapon",
	["ability_vehicle_demolisherram"] = "red orange battering ram spikes smashing siege weapon",
	["ability_vehicle_electrocharge"] = "blue lightning magic spell power awesome thunder electricity sparks electrocution crackle crackling striking hitting",
	["ability_vehicle_launchplayer"] = "leap flying flight person man sword shield fired launched thrown tossed blue outline silhouette flinged",
	["ability_vehicle_liquidpyrite"] = "fire fiery burning hot heat flames flaming blazing scorching searing wooden barrel ammunition ammo explosives bursting exploding blowing up yellow orange red",
	["ability_vehicle_liquidpyrite_blue"] = "fire fiery burning hot heat flames flaming blazing scorching searing wooden barrel ammunition ammo explosives bursting exploding blowing up blue purple",
	["ability_vehicle_loadselfcatapult"] = "red brown ammunition ammo loading putting place placing catapult siege weapon launch fire throw toss fling",
	["ability_vehicle_oiljets"] = "oil blob goo black silver engineering liquid",
	["ability_vehicle_plaguebarrel"] = "liquid green goo wooden barrel plague disease",
	["ability_vehicle_playerloaded"] = "green brown launch fire toss player fling catapult siege weapon",
	["ability_vehicle_powertowheels"] = "red siege weapon battering ram spikes smashing vehicle engineering machine",
	["ability_vehicle_reloadammo"] = "black iron metal cannonball reload ammunition ammo gunpowder packing stick",
	["ability_vehicle_rocketboost"] = "red motorcycle bike goblin engineering riding seat vehicle machine",
	["ability_vehicle_shellshieldgenerator"] = "honeycomb hexagon yellow shield barrier armor defense protection magic spell light power cell wall technology bubble",
	["ability_vehicle_siegeenginecannon"] = "cannonball fired firing launching launched blast iron metal power explosives exploding",
	["ability_vehicle_siegeenginecharge"] = "red siege weapon knock down walls ram smash battering vehicle engineering machine",
	["ability_vehicle_siegeengineram"] = "silver siege vehicle machine engineering battering ram parts bolts",
	["ability_vehicle_sonicshockwave"] = "golden yellow sonic blast burst shockwave explosion magic radiance radiant powerful holy restoration sunlight healing spell:solar_beam(druid)",
	["ability_warlock_ancientgrimoire"] = "pink purple green bookmark tome gemmed elegant fancy pages",
	["ability_warlock_avoidance"] = "red green purple blue summoned demonic felguard creature blade weapon head evasion dodge swift fast movement",
	["ability_warlock_backdraft"] = "demonic power magic hand fire fiery burning hot heat flames flaming blazing scorching searing swirly yellow orange red undead undeath corpse spiral spell:backdraft(warlock)",
	["ability_warlock_baneofhavoc"] = "red skull demonic creature undead glowing eyes power magic undeath evil head scourge corpse spell:havoc(warlock)",
	["ability_warlock_burningembers"] = "blazing fireballs fiery hot heat flames flaming scorching searing comets meteors coals red orange magic spell power passive:burning_embers(warlock)",
	["ability_warlock_chaosbolt"] = "poison toxic creature monster worm demonic evil teeth green terrifying beast head spell:chaos_bolt(warlock)",
	["ability_warlock_coil2"] = "purple magic spell blast waves pulses pulsing glowing passive:metamorphosis:_chaos_wave(warlock)",
	["ability_warlock_coil3"] = "purple crystals shards glowing",
	["ability_warlock_cremation"] = "fire fiery burning hot heat flames flaming blazing scorching searing red skeleton skeletal skull undead undeath corpse funeral ceremony bones",
	["ability_warlock_darkarts"] = "witch doctor skull staff purple blue demonic evil hex jinx curse",
	["ability_warlock_demonicempowerment"] = "fire fiery burning hot heat flames flaming blazing scorching searing yellow red orange monster beast demonic creature evil glowing head spell:command_demon(warlock)",
	["ability_warlock_demonicpower"] = "green summoned demonic evil monster creature beast winged wings purple power glowing shadow unholy fel magic spell:carrion_swarm(warlock)",
	["ability_warlock_empoweredimp"] = "fire fiery burning hot heat flames flaming blazing scorching searing imp summoned demonic creature evil beast chaotic chaos yellow orange red",
	["ability_warlock_eradication"] = "purple skull white undeath undead anger angry furious rage raging aggression teeth glowing eyes power magic creature evil shadow unholy head jinxed cursed scourge corpse passive:demonic_fury(warlock)",
	["ability_warlock_everlastingaffliction"] = "purple swirly fire fiery burning hot heat flames flaming blazing scorching searing shadow unholy magic spell:malefic_grasp(warlock)",
	["ability_warlock_fireandbrimstone"] = "fiery burning hot heat flames flaming blazing scorching searing orange yellow skull demonic creature undeath undead evil power magic shadow unholy head scourge corpse spell:fire_and_brimstone(warlock)",
	["ability_warlock_handofguldan"] = "purple pink fireball rocks trailing comet meteorite fiery burning hot heat flames flaming blazing scorching searing spell:hand_of_gul'dan(warlock)",
	["ability_warlock_haunt"] = "purple green skull demonic creature undeath undead powerful fel magic light shadow unholy evil head scourge corpse spell:haunt(warlock)",
	["ability_warlock_howlofterror"] = "purple shout fear curse face glowing evil talent:howl_of_terror(warlock)",
	["ability_warlock_impoweredimp"] = "summoned demonic creature glowing green eyes fel magic shadow unholy evil chaotic chaos teeth horns head empowered spell:wild_imps(warlock)",
	["ability_warlock_improveddemonictactics"] = "red glowing skull head undeath undead horns demonic creature evil spell shadow unholy powerful scourge corpse",
	["ability_warlock_improvedsoulleech"] = "demonic creature glowing eyes evil shadow unholy fear terror soul mouth scream horror purple insanity anger angry fury furious rage raging aggression head fierce cursed jinxed magic spell power",
	["ability_warlock_inferno"] = "fiery burning hot heat flames flaming blazing scorching searing yellow orange waves raging swirly swirls inferno firestorm magic spell power passive:metamorphosis:_immolation_aura(warlock)",
	["ability_warlock_jinx"] = "souls purple spirits dead death sad anguish agony hell outlines silhouettes shadows unholy fire fiery burning hot heat flames flaming blazing scorching searing tortured corrupted jinxed cursed evil magic power spell:mastery:_potent_afflictions(warlock)",
	["ability_warlock_moltencore"] = "fire fiery burning hot heat flames flaming blazing scorching searing yellow orange red rock melting glowing embers coals center passive:molten_core(warlock)",
	["ability_warlock_mortalcoil"] = "rainbow green gas cloud poison bolt missile nuke magic spell power talent:mortal_coil(warlock)",
	["ability_warlock_shadowflame"] = "shadowflame unholy purple skull undead undeath breath dragon drake head spit venom poison toxic curse evil magic spell power scourge corpse",
	["ability_warlock_shadowfurytga"] = "purple glowing radiant radiance demon winged creature evil talent:shadowfury(warlock)",
	["ability_warlock_soullink"] = "purple blue chains spiritual ethereal binding bound linked",
	["ability_warlock_soulsiphon"] = "purple soul undeath undead spirit glowing eyes skull swirly spiral magic spell power scourge corpse",
	["ability_warlock_soulswap"] = "shadow unholy demonic creature evil tortured corrupted purple dead death spirit head pain agony magic power spell:soul_swap(warlock)",
	["ability_warlock_whiplash"] = "bullwhip hand purple torture weapon passive:succubus:_whiplash(warlock)",
	["ability_warrior_battleshout"] = "head shouting anger angry fury furious rage raging aggression intimidation pain agony vigorous roar orange purple spell:battle_shout(warrior)",
	["ability_warrior_bladestorm"] = "silver red orange purple steel metal swords blades weapons ground array multiple passive:felguard:_felstorm(warlock)",
	["ability_warrior_bloodbath"] = "silver bloody red steel metal sword blade weapon tip point dagger knife",
	["ability_warrior_bloodfrenzy"] = "bloody fist clenched red punching power strength strong wounded bleeding knuckles",
	["ability_warrior_bloodnova"] = "blast burst explosion bloody red orange wound",
	["ability_warrior_bloodsurge"] = "bloody arm wounded cut scratched fist clenched blue scarred orcish spell:bloodsurge(warrior)",
	["ability_warrior_bullrush"] = "tauren bull horned horns furry fuzzy beast animal creature cow nosering glowing red eyes hair strong strength head face anger angry fury furious rage raging aggression talent:juggernaut(warrior)",
	["ability_warrior_challange"] = "silver crossed steel metal swords blades weapons daggers knives knife battle attack red orange conflicting conflicted spell:die_by_the_sword(warrior)",
	["ability_warrior_charge"] = "silver red steel metal sword blade weapon dagger knife stab puncture pierce thrust forward into spell:charge(warrior)",
	["ability_warrior_cleave"] = "silver green steel metal battleaxe blade weapon swinging moving motion striking cleaving spell:cleave(warrior)",
	["ability_warrior_colossussmash"] = "broken bloody shield armor defense protection blocking yellow golden silver steel metal defend smashed destroyed cracked spell:colossus_smash(warrior)",
	["ability_warrior_commandingshout"] = "night elf female woman girl screaming shouting yelling terrified horrified anger angry fury furious rage raging aggression white hair purple old aged highborne",
	["ability_warrior_criticalblock"] = "yellow golden steel metal shield armor defense protection blocking defend spell:mastery:_critical_block(warrior)",
	["ability_warrior_decisivestrike"] = "black silver purple spiked mace ball weapon swinging moving motion striking steel sphere passive:relentless_strikes(rogue) spell:slam(warrior)",
	["ability_warrior_defensivestance"] = "silver steel metal shield armor defense protection blocking defend riveted rivets bolts radiance blue radiant spell:defensive_stance(warrior)",
	["ability_warrior_devastate"] = "silver punch warped bloody fist clenched red steel metal sword blade weapon dagger knife power strength strong blow impact knuckles spell:victory_rush(warrior)",
	["ability_warrior_disarm"] = "silver brown hand taking take steel metal sword blade weapon dagger knife stealing overpowering passive:voidwalker:_disarm(warlock) spell:disarm(warrior)",
	["ability_warrior_dragonroar"] = "orange gray grey stone steel metal dragon's roar cloud serpent beast animal creature scaled scales scaly reptile tongue head",
	["ability_warrior_endlessrage"] = "orcish green anger angry furious rage raging aggression glowing red eyes teeth mouth bloodlust shouting yelling nosering spell:mastery:_unshackled_fury(warrior)",
	["ability_warrior_focusedrage"] = "red eye anger angry fury furious raging aggression focused sight vision precision accuracy aim marksmanship talent:enraged_regeneration(warrior)",
	["ability_warrior_furiousresolve"] = "green yellow orange fire fiery burning hot heat flames flaming blazing scorching searing orcish red eyes anger angry fury furious rage raging aggression",
	["ability_warrior_improveddisciplines"] = "silver steel metal armor helmet defense protection glowing red eyes horns horned spell:sudden_death(warrior)",
	["ability_warrior_incite"] = "golden yellow metal glowing light power magic enchanted enchantment sword blade weapon gemmed steel",
	["ability_warrior_innerrage"] = "holy light power magic winged wings helmet armor defense protection red yellow orange spell rage fury anger angry",
	["ability_warrior_intensifyrage"] = "muscles naked bare chest man red glowing impressive workout bodybuilding anger angry fury furious rage raging aggression",
	["ability_warrior_intervene"] = "silver intercepted dagger blade weapon knife steel metal sword blocked prevented stopped plate armored strike swinging striking moving motion",
	["ability_warrior_offensivestance"] = "silver spikey metal steel maces crossed attack conflicting conflicted red weapons spell:battle_stance(warrior)",
	["ability_warrior_punishingblow"] = "red orange yellow dagger weapon knife steel metal sword swinging striking attacking glowing moving motion spell:blade_flurry(rogue) spell:mastery:_strikes_of_opportunity(warrior)",
	["ability_warrior_rallyingcry"] = "shouting yelling roaring booming voice head mouth blue brown spell:commanding_shout(warrior)",
	["ability_warrior_rampage"] = "red anger angry fury furious rage raging aggression bloody wounded dying exploding mouth shouting yelling man orcish spell:crazed_berserker(warrior)",
	["ability_warrior_revenge"] = "brown white eye sight vision precision marksmanship aim anger angry fury furious rage raging aggression spell:vitality(rogue) spell:revenge(warrior)",
	["ability_warrior_riposte"] = "black blue stabbing ice spearhead magic frost crystaline shield spell:expose_armor(rogue)",
	["ability_warrior_safeguard"] = "golden yellow shield armor defense protection blocking metal blue glowing enchanted enchantment riveted rivets bolts",
	["ability_warrior_savageblow"] = "silver blue melee weapon battleaxe blade steel striking swinging moving motion spell:mortal_strike(warrior)",
	["ability_warrior_secondwind"] = "clenched fist red power strength strong punching blow knuckles",
	["ability_warrior_shatteringthrow"] = "silver ice icy cold frozen freezing frosty steel metal sword blade weapon shield barrier armor glowing enchanted enchantment light thrown dagger knife blue spell:shattering_throw(warrior)",
	["ability_warrior_shieldbash"] = "golden yellow silver orange shield armor defense protection blocking steel metal bash strike defending interrupting holding",
	["ability_warrior_shieldbreak"] = "red orange yellow silver broken breaking cracked cracking steel metal shield armor defense protection blocking exploding glowing defend",
	["ability_warrior_shieldguard"] = "brown blocking shield armor protection defending holding wooden protecting spell:bastion_of_defense(warrior)",
	["ability_warrior_shieldmastery"] = "silver steel metal riveted rivets shield armor defense protection blocking fire fiery burning hot flames flaming blazing scorching searing enchanted enchantment glowing heated red orange yellow molten defend",
	["ability_warrior_shieldreflection"] = "silver reflecting beam light reflected steel metal shield armor defense protection blocking defend spell:spell_reflection(warrior)",
	["ability_warrior_shieldwall"] = "wooden emblem purple blue moving motion armor defending defense blocking protection spell:shield_wall(warrior)",
	["ability_warrior_shockwave"] = "silver steel metal sword blade weapon blue glowing golden blast burst power magic yellow shockwave impact smashed shattered disenchanting disenchanted enchantment lightning thunder",
	["ability_warrior_stalwartprotector"] = "silver steel metal sword blade weapon plate armor helmet defense protection glowing blue enchanted enchantment power lightning thunder spell:ultimatum(warrior)",
	["ability_warrior_strengthofarms"] = "orcish strength strong muscles bodybuilding yellow green clenched fist punching knuckles",
	["ability_warrior_sunder"] = "broken silver steel metal sword blade weapon dagger knife destroyed busted shattered sundered smashed brown spell:sunder_armor(warrior)",
	["ability_warrior_swordandboard"] = "silver steel metal blade weapon attached to shield armor defense protection blocking blue glowing enchanted enchantment spell:sword_and_board(warrior)",
	["ability_warrior_throwdown"] = "head hitting ground falling tripping pinned impacting thrown human red orange yellow",
	["ability_warrior_throwdown_2"] = "head hitting ground falling tripping pinned impacting thrown night elf green purple red orange",
	["ability_warrior_titansgrip"] = "gray grey silver red orange clenched fist punching fire fiery burning hot heat flames flaming blazing scorching searing stone metal steel power magic might holding clenching enchanted enchantment knuckles passive:felguard:_axe_toss(warlock) spell:titan's_grip(warrior)",
	["ability_warrior_trauma"] = "red silver bloody spikey ball spikes metal steel impacting hitting wounding gushing wrecking",
	["ability_warrior_unrelentingassault"] = "silver glowing fire fiery burning hot heat flames flaming blazing scorching searing steel metal sword blade weapon helmet armor defense protection yellow orange",
	["ability_warrior_victoryrush"] = "yellow fire fiery burning hot heat flames flaming blazing scorching searing light holy power magic enchanted enchantment defense defending pushing through blocking protected protection armor shielded silhouette outline bubble spell:intervene(warrior)",
	["ability_warrior_vigilance"] = "silver steel metal sword blade weapon shield armor defense protection blocking defending golden yellow",
	["ability_warrior_warbringer"] = "silver fire fiery burning hot heat flames flaming blazing scorching searing sword blade weapon forging dagger knife steel metal blacksmithing forged enchanted enchantment power magic talent:warbringer(warrior)",
	["ability_warrior_warcry"] = "orcish screaming crying shouting yelling tusks teeth mouth head bearded red green earring spell:demoralizing_shout(warrior) racial_passive:command(orc)",
	["ability_warrior_weaponmastery"] = "silver steel metal sword blade weapon bloody motion moving swinging striking strike dagger knife red",
	["ability_warstomp"] = "brown black smash impacting hitting ground powerful strong strength horse clefthoof centaur racial:war_stomp(tauren) spell:demonic_leap(warlock)",
	["ability_whirlwind"] = "silver blue steel metal sword blade weapon dagger knife golden yellow ring swinging striking motion moving circle path spell:auto_shot(hunter) spell:deterrence(hunter) spell:whirlwind(warrior)",
	["ability_wintergrasp_rank1"] = "medal metal honor place number ribbon steel platinum silver blue one first 1st",
	["ability_wintergrasp_rank2"] = "medal metal honor place number ribbon steel platinum silver purple two second 2nd",
	["ability_wintergrasp_rank3"] = "medal metal honor place number ribbon steel platinum silver red pink three third 3rd",
	["ability_worgen_darkflight"] = "worgen darkflight foot paw claws white wolf canine dog animal furry fuzzy creature beast stepping",
	["inv_alchemy_crystalvial"] = "crystal clear glass vial potion bottle jar drink restore empty",
	["inv_alchemy_elixir_01"] = "yellow glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_elixir_02"] = "blue glass vial flask potion elixir bottle jar drink mana",
	["inv_alchemy_elixir_03"] = "green glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_elixir_04"] = "orange glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_elixir_05"] = "red glass vial flask potion elixir bottle jar drink health healing",
	["inv_alchemy_elixir_06"] = "purple glass vial flask potion elixir bottle jar drink rejuvenation",
	["inv_alchemy_elixir_empty"] = "black glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_enchantedvial"] = "black glass %elixirempty",
	["inv_alchemy_endlessflask_01"] = "black clear water glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_endlessflask_02"] = "yellow glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_endlessflask_03"] = "green glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_endlessflask_04"] = "purple blue glass vial flask potion elixir bottle jar drink mana",
	["inv_alchemy_endlessflask_05"] = "pink glass vial flask potion elixir bottle jar drink rejuvenation",
	["inv_alchemy_endlessflask_06"] = "orange glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_imbuedvial"] = "clear glass vial potion bottle jar drink restore empty",
	["inv_alchemy_leadedvial"] = "clear glass vial potion bottle jar drink restore empty",
	["inv_alchemy_potion_01"] = "yellow glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_potion_02"] = "red glass vial flask potion elixir bottle jar drink health healing",
	["inv_alchemy_potion_03"] = "blue glass vial flask potion elixir bottle jar drink mana",
	["inv_alchemy_potion_04"] = "purple glass vial flask potion elixir bottle jar drink rejuvenation",
	["inv_alchemy_potion_05"] = "green glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_potion_06"] = "orange glass vial flask potion elixir bottle jar drink",
	["inv_alchemy_potion_empty"] = "clear glass vial flask potion elixir bottle jar drink empty",
	["inv_ammo_arrow_01"] = "silver steel metal arrow weapon ammunition ammo shot shooting",
	["inv_ammo_arrow_02"] = "silver steel metal arrow weapon ammunition ammo shot shooting companion:peddlefeet",
	["inv_ammo_arrow_03"] = "silver steel metal arrow weapon ammunition ammo shot shooting",
	["inv_ammo_arrow_04"] = "brown wooden copper arrow weapon ammunition ammo shot shooting",
	["inv_ammo_arrow_05"] = "brown copper arrow weapon ammunition ammo shot shooting",
	["inv_ammo_arrow_06"] = "purple obsidian stone arrow weapon ammunition ammo shot shooting",
	["inv_ammo_bullet_01"] = "silver copper bullet weapon ammunition ammo steel metal shot shooting",
	["inv_ammo_bullet_02"] = "silver gray stone bullet weapon ammunition ammo shot shooting throwing tossing tossed boulder rock",
	["inv_ammo_bullet_03"] = "green copper bullet weapon ammunition ammo bomb explosives shot shooting firing launching rocket missile angry face steel metal",
	["inv_ammo_bullet_04"] = "red bullet weapon ammunition ammo bomb explosives smiley face happy shot shooting spiked spikes steel metal",
	["inv_ammo_bullet_05"] = "blue bullet weapon ammunition ammo bomb explosives shot shooting spiked spikes steel metal",
	["inv_ammo_bullet_06"] = "silver steel metal bullet weapon ammunition ammo bomb explosives",
	["inv_ammo_bullet_07"] = "green bombs explosives weapons ammunition ammo steel metal belt chain grenades",
	["inv_ammo_bullet_08"] = "white bombs explosives weapons ammunition ammo steel metal belt chain grenades",
	["inv_axe_01"] = "",
	["inv_axe_02"] = "racial_passive:axe_specialization(orc)",
	["inv_axe_06"] = "spell:mastery:_executioner(rogue)",
	["inv_axe_09"] = "spell:seasoned_soldier(warrior) spell:auto_attack(warrior)",
	["inv_axe_10"] = "passive:weapon_skills(death_knight)",
	["inv_axe_66"] = "spell:heroic_throw(warrior)",
	["inv_axe_96"] = "spell:necrotic_strike(death_knight)",
	["inv_bijou_blue"] = "blue troll bijou zul'gurub",
	["inv_bijou_bronze"] = "silver bronze troll bijou zul'gurub",
	["inv_bijou_gold"] = "orange gold yellow troll bijou zul'gurub",
	["inv_bijou_green"] = "green troll bijou zul'gurub",
	["inv_bijou_orange"] = "orange troll bijou zul'gurub",
	["inv_bijou_purple"] = "purple troll bijou zul'gurub",
	["inv_bijou_red"] = "red troll bijou zul'gurub",
	["inv_bijou_silver"] = "white silver troll bijou zul'gurub",
	["inv_bijou_yellow"] = "yellow troll bijou zul'gurub",
	["inv_bone_skull_04"] = "white red cat kitty feline furry fuzzy beast animal creature skull sabretooth tiger monster",
	["inv_box_01"] = "brown crate lockbox locked vault bag wooden chest container storage",
	["inv_box_02"] = "red crate lockbox locked vault bag wooden chest container storage",
	["inv_box_03"] = "green crate lockbox locked vault bag wooden chest container storage",
	["inv_box_04"] = "purple crate lockbox locked vault bag wooden chest container storage",
	["inv_box_birdcage_01"] = "bird cage bronze silver steel metal storage companion:miniwing",
	["inv_box_petcarrier_01"] = "silver steel metal cage cat kitty dog pet animal creature beast storage companion:stinker companion:worg_pup",
	["inv_cask_01"] = "brown corked wooden keg cask barrel root beer mead booze ale alcohol drink liquid",
	["inv_cask_02"] = "brown wooden keg cask barrel root beer mead booze ale alcohol drink liquid",
	["inv_cask_03"] = "brown wooden keg cask barrel root beer mead booze ale alcohol drink liquid",
	["inv_cask_04"] = "brown tapped wooden keg cask barrel root beer mead booze ale alcohol drink liquid",
	["inv_celestialserpentmount"] = "blue reins of the astral cloud serpent mount flying",
	["inv_chaos_orb"] = "chaos orb red glowing pulsing sphere spherical marble bloody spell:mastery:_shadowy_recall(priest)",
	["inv_chest_chain_06"] = "passive:armor_skills(hunter)",
	["inv_chest_chain_11"] = "passive:mail_specialization(shaman)",
	["inv_chest_chain_12"] = "passive:armor_skills(shaman)",
	["inv_chest_chain_15"] = "passive:armor_skills(death_knight)",
	["inv_chest_cloth_05"] = "spell:leather_specialization(druid)",
	["inv_chest_cloth_08"] = "spell:leather_specialization(druid)",
	["inv_chest_cloth_23"] = "passive:armor_skills(mage)",
	["inv_chest_cloth_24"] = "passive:armor_skills(warlock)",
	["inv_chest_cloth_50"] = "passive:armor_skills(priest)",
	["inv_chest_leather_03"] = "passive:leather_specialization(rogue)",
	["inv_chest_leather_06"] = "spell:leather_specialization(druid)",
	["inv_chest_leather_07"] = "spell:armor_skills(druid)",
	["inv_chest_leather_09"] = "passive:armor_skills(rogue)",
	["inv_chest_plate04"] = "spell:armor_skills(warrior)",
	["inv_chest_plate12"] = "passive:plate_specialization(death_knight)",
	["inv_chest_plate13"] = "spell:plate_specialization(warrior)",
	["inv_chest_plate15"] = "spell:armor_skills(paladin) spell:plate_specialization(warrior)",
	["inv_cloudserpent_egg_black"] = "onyx egg black cloud serpent",
	["inv_cloudserpent_egg_blue"] = "blue cloud serpent egg",
	["inv_cloudserpent_egg_green"] = "green cloud serpent egg",
	["inv_cloudserpent_egg_red"] = "red cloud serpent egg",
	["inv_cloudserpent_egg_yellow"] = "yellow cloud serpent egg",
	["inv_crate_01"] = "brown crate box boarded wooden planks bag container chest storage",
	["inv_crate_02"] = "brown box boarded wooden planks bag container chest storage companion:creepy_crate",
	["inv_crate_03"] = "brown crate box boarded wooden planks bag container chest storage",
	["inv_crate_04"] = "brown crate box boarded wooden planks bag container chest storage",
	["inv_crate_05"] = "brown crate box boarded wooden planks bag container chest storage nails",
	["inv_crate_06"] = "brown crate box boarded wooden planks bag container chest storage nails",
	["inv_crate_07"] = "brown crate box boarded wooden planks bag container chest storage",
	["inv_crate_08"] = "brown crate box boarded wooden planks bag container chest storage red apples cherries",
	["inv_crate_09"] = "brown crate box boarded wooden planks bag container chest storage green glasses bottles wine beer",
	["inv_crystallized_earth"] = "silver blue gray grey crystallized earth elemental stone",
	["inv_crystallized_life"] = "green crystallized life elemental growth regeneration restoration",
	["inv_crystallized_water"] = "blue crystallized water elemental ice frost",
	["inv_datacrystal01"] = "purple blue glowing crystal shard disenchanting mats materials",
	["inv_datacrystal02"] = "purple blue glowing crystal shard disenchanting mats materials",
	["inv_datacrystal03"] = "purple yellow orange glowing crystal shard disenchanting mats materials fire fiery burning hot heat flames flaming blazing scorching searing",
	["inv_datacrystal04"] = "purple blue glowing crystal shard disenchanting mats materials",
	["inv_datacrystal05"] = "blue glowing crystal shard disenchanting mats materials mana",
	["inv_datacrystal06"] = "yellow orange glowing crystal shard disenchanting mats materials",
	["inv_datacrystal07"] = "purple glowing crystal shard disenchanting mats materials",
	["inv_datacrystal08"] = "purple blue glowing crystal shard disenchanting mats materials",
	["inv_datacrystal09"] = "blue glowing crystal shard disenchanting mats materials ice icy cold frozen freezing frosty mana",
	["inv_datacrystal10"] = "purple pink red glowing crystal shard disenchanting mats materials",
	["inv_datacrystal11"] = "red orange pink glowing crystal shard disenchanting mats materials",
	["inv_datacrystal12"] = "purple blue red pink glowing crystal shard disenchanting mats materials",
	["inv_diablostone"] = "red crystal shard dragon scale spike stone rock",
	["inv_dragonchromaticmount"] = "chromatic dragon yellow golden glowing helmet head drake mount",
	["inv_dragonwhelpcataclysm"] = "mini deathwing steel metal glowing yellow silver evil pet companion",
	["inv_drink_01"] = "silver flask potion elixir bottle jar drink steel metal canteen jug",
	["inv_drink_02"] = "purple golden potion jug drink wine juice mana berries chalice cup horn tusk ivory mammoth alcohol moonberry",
	["inv_drink_03"] = "black steel metal flask cup mug pint mead root beer booze ale alcohol drink liquid",
	["inv_drink_04"] = "purple wooden flask cup mug pint mead root beer booze ale alcohol drink liquid",
	["inv_drink_05"] = "brown wooden flask cup mug pint mead root beer booze ale alcohol drink liquid overflowing foamy foaming frothy bubbling",
	["inv_drink_06"] = "crystal glass vial potion bottle jar drink restore water jug clear",
	["inv_drink_07"] = "canteen brown drinking water liquid pouch flask container",
	["inv_drink_08"] = "brown wooden flask cup mug pint mead root beer booze ale alcohol drink liquid foamy foaming frothy bubbling",
	["inv_drink_09"] = "potion bottle jar drink restore jug glass purple moonberry juice wine alcohol corked",
	["inv_drink_10"] = "purple corked wine drink alcohol bottle",
	["inv_drink_11"] = "green corked wine drink alcohol bottle",
	["inv_drink_12"] = "white wine drink alcohol bottle red ribbon container jug jar milk",
	["inv_drink_13"] = "silver steel metal flask cup mug pint mead root beer booze ale alcohol drink liquid foamy foaming frothy bubbling overflowing companion:wolpertinger",
	["inv_drink_14"] = "drinking water liquid bamboo container jug",
	["inv_drink_15"] = "hot chocolate coffee steaming cup mug white brown liquid drink",
	["inv_drink_16"] = "white drinking water liquid jug jar bottle purple ribbon",
	["inv_drink_17"] = "black kettle cooking pot hot steaming bowl soup cup liquid drink",
	["inv_drink_18"] = "canteen jar jug drinking water liquid brown leather",
	["inv_drink_19"] = "water bucket liquid wooden",
	["inv_drink_20"] = "blue golden drinking water liquid cup glass ice icy cold frozen freezing frosty elegant cool",
	["inv_drink_21"] = "drinking water liquid goo ectoplasm poison green glass",
	["inv_drink_22"] = "blue glass of wine drink alcohol cup",
	["inv_drink_23"] = "hot apple cider steaming glass cup mug cinnamon sticks juice",
	["inv_drink_24_sealwhey"] = "pungeant seal whey straw smelly stinky drink carton pouch",
	["inv_drink_25_honeytea"] = "orange brown leaf cup drink steaming hot coffee",
	["inv_drink_26_goatsmilk"] = "jug goat's milk drink",
	["inv_drink_27_bluesoup"] = "blue soup cup bowl steaming hot drink",
	["inv_drink_28_gilneanfortifiedbrandy"] = "red bottle drink wine alcohol beer brandy ale",
	["inv_drink_29_sunkissedwine"] = "yellow golden rays clear glass bottle sunlight wine alcohol drinking water liquid",
	["inv_drink_30_blackheartgrog"] = "silver steel metal jeweled ruby cup mug decorated",
	["inv_drink_31_embalmingfluid"] = "green skull glass bottle wine drink alcohol death poison",
	["inv_drink_32_disgustingrotgut"] = "wooden mug cup bubbling green liquid drink goo poison slime",
	["inv_drink_33_bloodredale"] = "red wine drink alcohol glass bottle",
	["inv_drink_milk_01"] = "milk clear glass drink cold",
	["inv_drink_milk_02"] = "brown canteen drinking pouch",
	["inv_drink_milk_03"] = "white silver steel metal tin boiling hot milk bucket drink",
	["inv_drink_milk_04"] = "wooden bucket milk drink liquid",
	["inv_drink_milk_05"] = "green drink liquid mug cup",
	["inv_drink_waterskin_01"] = "blue drinking water liquid pouch canteen",
	["inv_drink_waterskin_02"] = "blue drinking water liquid pouch canteen broken ripped torn",
	["inv_drink_waterskin_03"] = "green drinking water liquid pouch canteen",
	["inv_drink_waterskin_04"] = "green drinking water liquid pouch canteen broken ripped torn",
	["inv_drink_waterskin_05"] = "orange brown drinking water liquid pouch canteen",
	["inv_drink_waterskin_06"] = "orange brown drinking water liquid pouch canteen broken ripped torn",
	["inv_drink_waterskin_07"] = "purple drinking water liquid pouch canteen",
	["inv_drink_waterskin_08"] = "purple drinking water liquid pouch canteen broken ripped torn",
	["inv_drink_waterskin_09"] = "red drinking water liquid pouch canteen",
	["inv_drink_waterskin_10"] = "red drinking water liquid pouch canteen broken ripped torn",
	["inv_drink_waterskin_11"] = "brown drinking water liquid pouch canteen",
	["inv_drink_waterskin_12"] = "brown drinking water liquid pouch canteen",
	["inv_egg_01"] = "eggs white",
	["inv_egg_02"] = "yellow green spotted egg",
	["inv_egg_03"] = "red brown spotted egg",
	["inv_egg_04"] = "hatched cracked egg yellow brown spotted",
	["inv_egg_05"] = "blue silver steel metal mechanical egg lightning thunder",
	["inv_egg_06"] = "blue spotted egg",
	["inv_egg_07"] = "red pink spotted egg",
	["inv_egg_08"] = "green spotted egg",
	["inv_egg_09"] = "purple spotted egg",
	["inv_elemental_crystal_air"] = "blue air crystallized elemental stone rock shard",
	["inv_elemental_crystal_earth"] = "yellow silver gray grey earth stone rock crystallized elemental shard",
	["inv_elemental_crystal_fire"] = "red fire fiery burning hot heat flames flaming blazing scorching searing crystallized elemental stone rock shard",
	["inv_elemental_crystal_life"] = "life nature restoration regrowth regeneration green crystallized elemental stone rock shard",
	["inv_elemental_crystal_shadow"] = "blue black shadow crystallized elemental stone rock shard",
	["inv_elemental_crystal_water"] = "blue water crystallized elemental stone rock shard",
	["inv_elemental_eternal_air"] = "blue water ice icy cold frozen freezing frosty crystallized elemental stone rock shard",
	["inv_elemental_eternal_earth"] = "silver gray grey crystallized elemental stone rock shard",
	["inv_elemental_eternal_fire"] = "yellow red orange fire fiery burning hot heat flames flaming blazing scorching searing crystallized elemental stone rock shard",
	["inv_elemental_eternal_life"] = "green life nature restoration regrowth regeneration crystallized elemental stone rock shard",
	["inv_elemental_eternal_shadow"] = "black shadow obsidian crystallized elemental stone rock shard",
	["inv_elemental_eternal_water"] = "blue water ice icy cold frozen freezing frosty crystallized elemental stone rock shard",
	["inv_elemental_mote_air01"] = "blue primal mote of air wind",
	["inv_elemental_mote_earth01"] = "gray grey blue primal mote of earth rocks stones pebbles",
	["inv_elemental_mote_fire01"] = "red orange primal mote of fire fiery burning hot heat flames flaming blazing scorching searing",
	["inv_elemental_mote_life01"] = "green primal mote of life nature restoration regrowth regeneration glowing light",
	["inv_elemental_mote_mana"] = "blue primal mote of mana glowing light passive:nether_attunement(mage)",
	["inv_elemental_mote_nether"] = "yellow primal mote of nether light energy glowing",
	["inv_elemental_mote_shadow01"] = "blue purple primal mote of shadow glowing light soul",
	["inv_elemental_mote_water01"] = "blue primal mote of water liquid glowing light",
	["inv_elemental_primal_air"] = "blue primal mote of air wind",
	["inv_elemental_primal_earth"] = "gray grey rocks stones primal mote of earth",
	["inv_elemental_primal_fire"] = "yellow orange primal mote of fire fiery burning hot heat flames flaming blazing scorching searing",
	["inv_elemental_primal_life"] = "green primal mote of life nature restoration regrowth regeneration glowing light",
	["inv_elemental_primal_mana"] = "blue primal mote of mana glowing light",
	["inv_elemental_primal_nether"] = "yellow primal mote of nether light energy glowing",
	["inv_elemental_primal_shadow"] = "blue purple primal mote of glowing light soul passive:voidwalker:_shadow_shield(warlock)",
	["inv_elemental_primal_water"] = "blue primal mote of water liquid glowing light",
	["inv_elemental_spiritofharmony_1"] = "mote of harmony bluegreen cyan turquoise water droplet",
	["inv_elemental_spiritofharmony_2"] = "spirit of harmony blue green cyan turquoise water droplets drops swirling",
	["inv_ember"] = "red orange seething cinder fiery burning hot heat flames flaming blazing scorching searing fireball spell:ember_tap(warlock)",
	["inv_emberweavebandage1"] = "dense embersilk bandage bluegreen cloth",
	["inv_emberweavebandage2"] = "heavy embersilk bandage bluegreen cloth",
	["inv_enchant_abysscrystal"] = "abyss crystal blue shard fragment disenchanting glowing magic professions:enchanting stone sharp",
	["inv_enchant_disenchant"] = "disenchanting destroying steel metal sword blade weapon breaking apart up professions:disenchant",
	["inv_enchant_dreamshard_01"] = "small dream shard crystal fragment disenchanting glowing magic professions:enchanting yellow bluegreen cyan",
	["inv_enchant_dreamshard_02"] = "dream shard crystal fragment disenchanting glowing magic professions:enchanting yellow bluegreen cyan",
	["inv_enchant_dust"] = "hypnotic dust powder sand disenchanting glowing pile magic professions:enchanting bluegreen cyan",
	["inv_enchant_dustarcane"] = "arcane dust powder sand disenchanting glowing pile magic professions:enchanting pink",
	["inv_enchant_dustdream"] = "dream dust powder sand disenchanting glowing pile magic professions:enchanting pink purple",
	["inv_enchant_dustillusion"] = "illusion dust powder sand disenchanting glowing pile magic professions:enchanting pink purple",
	["inv_enchant_dustsoul"] = "infinite dust powder sand disenchanting glowing pile magic professions:enchanting blue purple",
	["inv_enchant_duststrange"] = "strange dust powder sand disenchanting glowing pile magic professions:enchanting green",
	["inv_enchant_dustvision"] = "vision dust powder sand disenchanting glowing pile magic professions:enchanting turquoise bluegreen cyan",
	["inv_enchant_essencearcanelarge"] = "greater planar essence disenchanting glowing light radiant radiance magic purple nether vortex",
	["inv_enchant_essencearcanesmall"] = "lesser planar essence disenchanting glowing light radiant radiance magic purple nether",
	["inv_enchant_essenceastrallarge"] = "greater astral essence disenchanting glowing radiant radiance magic sunburst starburst spark lightning yellow orange",
	["inv_enchant_essenceastralsmall"] = "lesser astral essence disenchanting glowing radiant radiance magic lightning spark yellow orange",
	["inv_enchant_essencecosmicgreater"] = "greater cosmic essence disenchanting glowing light radiant radiance magic purple sunburst starburst vortex",
	["inv_enchant_essencecosmiclesser"] = "lesser cosmic essence disenchanting glowing light radiant radiance magic purple sunburst starburst vortex",
	["inv_enchant_essenceeternallarge"] = "greater eternal essence disenchanting glowing radiant radiance magic sunburst starburst spark lightning blue frost racial_passive:expansive_mind(gnome) spell:spirit_of_redemption(priest)",
	["inv_enchant_essenceeternalsmall"] = "lesser eternal essence disenchanting glowing radiant radiance magic lightning spark blue frost",
	["inv_enchant_essencemagiclarge"] = "greater magic essence disenchanting glowing radiant radiance sunburst starburst spark lightning blue passive:wizardry(mage)",
	["inv_enchant_essencemagicsmall"] = "lesser magic essence disenchanting glowing radiant radiance lightning spark blue",
	["inv_enchant_essencemysticallarge"] = "greater mystic essence disenchanting glowing radiant radiance magic sunburst starburst spark lightning purple pink",
	["inv_enchant_essencemysticalsmall"] = "lesser mystic essence disenchanting glowing radiant radiance magic lightning spark purple pink",
	["inv_enchant_essencenetherlarge"] = "greater essence disenchanting glowing radiant radiance magic sunburst starburst spark lightning green passive:nethermancy(warlock)",
	["inv_enchant_essencenethersmall"] = "lesser nether essence disenchanting glowing radiant radiance magic lightning spark green",
	["inv_enchant_formulaepic_01"] = "pattern paper parchment text spell scroll enchanting plans recipe purple",
	["inv_enchant_formulagood_01"] = "pattern paper parchment text spell scroll enchanting plans recipe green",
	["inv_enchant_formulasuperior_01"] = "pattern paper parchment text spell scroll enchanting plans recipe blue",
	["inv_enchant_prismaticsphere"] = "prismatic sphere ball marble pearl white purple shiny round spherical",
	["inv_enchant_shardbrilliantlarge"] = "large brilliant shard crystal fragment disenchanting glowing magic professions:enchanting yellow",
	["inv_enchant_shardbrilliantsmall"] = "small brilliant shard crystal fragment disenchanting glowing magic professions:enchanting yellow racial_passive:the_human_spirit(human)",
	["inv_enchant_shardgleamingsmall"] = "small gleaming shard crystal fragment disenchanting glowing magic professions:enchanting blue",
	["inv_enchant_shardglimmeringlarge"] = "large glimmering shard crystal fragment disenchanting glowing magic professions:enchanting bluegreen cyan purple racial_passive:arcane_affinity(blood_elf)",
	["inv_enchant_shardglimmeringsmall"] = "small glimmering shard crystal fragment disenchanting glowing magic professions:enchanting bluegreen cyan",
	["inv_enchant_shardglowinglarge"] = "large glowing shard crystal fragment disenchanting magic professions:enchanting yellow purple",
	["inv_enchant_shardglowingsmall"] = "small glowing shard crystal fragment disenchanting magic professions:enchanting purple",
	["inv_enchant_shardnexuslarge"] = "nexus crystal shard fragment disenchanting glowing magic professions:enchanting purple red",
	["inv_enchant_shardprismaticlarge"] = "large prismatic shard crystal fragment disenchanting glowing magic professions:enchanting blue purple",
	["inv_enchant_shardprismaticsmall"] = "small prismatic shard crystal fragment disenchanting glowing magic professions:enchanting blue purple",
	["inv_enchant_shardradientlarge"] = "large radiant shard crystal fragment disenchanting glowing magic professions:enchanting yellow orange red",
	["inv_enchant_shardradientsmall"] = "small radiant shard crystal fragment disenchanting glowing magic professions:enchanting yellow orange red",
	["inv_enchant_shardshadowfrostlarge"] = "shadowfrost shard stone crystal fragment blue rock",
	["inv_enchant_voidcrystal"] = "void crystal shard fragment disenchanting glowing magic professions:enchanting purple",
	["inv_enchant_voidsphere"] = "void sphere nether vortex purple glowing magic",
	["inv_epicguildtabard"] = "renowned guild tabard steel metal chestplate armor defense protection golden",
	["inv_fabric_ebonweave"] = "ebonweave cloth rags tailoring magic rune tattered torn demonic fel black bluegreen cyan",
	["inv_fabric_felcloth_ebon"] = "shadowcloth rags tailoring magic rune tattered torn demonic fel green",
	["inv_fabric_felrag"] = "felcloth rags tailoring magic rune tattered torn demonic purple",
	["inv_fabric_frostweave_bolt"] = "bolt of frostweave cloth tailoring fabric roll blue",
	["inv_fabric_frostweave_imbuedbolt"] = "bolt of imbued frostweave cloth tailoring fabric roll blue",
	["inv_fabric_imperialsilk"] = "imperial silk cloth fabric tailoring red",
	["inv_fabric_linen_01"] = "linen cloth fabric tailoring brown tan",
	["inv_fabric_linen_02"] = "linen bolts cloth tailoring fabric yarn thread string spools brown tan",
	["inv_fabric_linen_03"] = "coarse thread ball of yarn string white",
	["inv_fabric_mageweave_01"] = "mageweave cloth fabric tailoring sparkling white",
	["inv_fabric_mageweave_02"] = "ball of yarn thread string white sparkling",
	["inv_fabric_mageweave_03"] = "bolt of mageweave cloth tailoring fabric white sparkling",
	["inv_fabric_moonrag_01"] = "mooncloth shining fabric tailoring light purple white glowing rune",
	["inv_fabric_moonrag_02"] = "bolt of mooncloth roll tailoring fabric light white purple glowing",
	["inv_fabric_moonrag_primal"] = "primal mooncloth fabric tailoring light white purple glowing",
	["inv_fabric_moonshroud"] = "moonshroud bolt tailoring fabric mooncloth light white purple glowing",
	["inv_fabric_netherweave"] = "netherweave cloth fabric tailoring purple",
	["inv_fabric_netherweave_bolt"] = "bolt of netherweave cloth tailoring fabric purple roll",
	["inv_fabric_netherweave_bolt_imbued"] = "bolt of imbued netherweave cloth tailoring fabric purple roll sparkling",
	["inv_fabric_purple_01"] = "purple white cloth fabric tailoring pillow glowing",
	["inv_fabric_purple_02"] = "purple white bolt cloth tailoring fabric glowing roll",
	["inv_fabric_purplefire_01"] = "runecloth fabric tailoring glowing purple red",
	["inv_fabric_purplefire_02"] = "runecloth bolt tailoring fabric glowing purple red roll",
	["inv_fabric_silk_01"] = "silk cloth fabric tailoring purple pink",
	["inv_fabric_silk_02"] = "heavy silken thread spool yarn string purple pink",
	["inv_fabric_silk_03"] = "bolt of silk cloth tailoring fabric purple pink",
	["inv_fabric_soulcloth"] = "frostweave cloth fabric tailoring blue tattered torn",
	["inv_fabric_soulcloth_bolt"] = "bolt of soulcloth fabric tailoring blue glowing",
	["inv_fabric_spellfire"] = "spellcloth fabric tailoring fire fiery burning hot heat red yellow orange",
	["inv_fabric_spellweave"] = "spellweave bolt cloth tailoring fabric fire fiery burning hot heat flames flaming blazing scorching searing red yellow orange rune",
	["inv_fabric_windwool"] = "windwool cloth fabric tailoring bluegreen cyan turquoise",
	["inv_fabric_windwool_bolt"] = "bolt of windwool cloth tailoring fabric bluegreen cyan turquoise",
	["inv_fabric_windwool_bolt_imbued"] = "bolt of imbued windwool cloth tailoring fabric bluegreen cyan turquoise enchanted enchantment",
	["inv_fabric_wool_01"] = "woolen cloth fabric tailoring white fleece",
	["inv_fabric_wool_02"] = "fine thread woolen ball of yarn string white",
	["inv_fabric_wool_03"] = "bolt of woolen cloth tailoring fabric white",
	["inv_farm_enchantedseed"] = "magebulb seed green cyan turquoise shard crystal rock glowing",
	["inv_farm_goldenseed"] = "songbell seedling yellow golden growing life sprouting",
	["inv_farm_herbseed"] = "enigma seedling yellow life sprouting",
	["inv_farm_kypariteseed"] = "snakeroot seedling crystal shards brown blue cyan turquoise",
	["inv_farm_leatherseed"] = "raptorleaf seed brown leather",
	["inv_feather_01"] = "feather quill black white spell:feline_grace(druid) passive:safe_fall(rogue)",
	["inv_feather_02"] = "feather quill black",
	["inv_feather_03"] = "feather quill black white",
	["inv_feather_04"] = "feather quill black white",
	["inv_feather_05"] = "feather quill brown",
	["inv_feather_06"] = "feather quill red",
	["inv_feather_07"] = "feather quill purple",
	["inv_feather_08"] = "feather quill blue",
	["inv_feather_09"] = "feather quill brown",
	["inv_feather_10"] = "feather quill brown",
	["inv_feather_11"] = "feather quill green",
	["inv_feather_12"] = "feather quill green",
	["inv_feather_13"] = "feather quill purple",
	["inv_feather_14"] = "feather quill brown",
	["inv_feather_15"] = "feather quill red black",
	["inv_feather_16"] = "feather quill red black",
	["inv_firearm_2h_rifle_430future_c_01"] = "dragon's head blue orange steel metal silver dragonsmaw blaster",
	["inv_fishingpole_01"] = "fishing pole hook line silver steel metal",
	["inv_fishingpole_02"] = "wooden stick fishing pole hook line branch",
	["inv_fishingpole_03"] = "wooden stick fishing pole reel line",
	["inv_fishingpole_05"] = "jeweled fishing pole golden",
	["inv_fishingpole_06"] = "wooden branch tree stick fishing pole line rope knot tied",
	["inv_fishingpole_07"] = "gnomish bait-o-matic fishing pole silver steel metal",
	["inv_food_christmasfruitcake_01"] = "graccu's mince meat fruitcake cherries brown",
	["inv_gauntlets_02"] = "furry fuzzy claws paw bear beast animal creature snowy white blue",
	["inv_gauntlets_04"] = "spell:pummel(warrior)",
	["inv_gizmo_01"] = "steel metal silver gizmo gadget professions:engineering machine device",
	["inv_gizmo_02"] = "gears cogs steel metal silver gizmo gadget professions:engineering machine device",
	["inv_gizmo_03"] = "steel metal silver gizmo gadget professions:engineering",
	["inv_gizmo_04"] = "blue silver steel metal barbell gizmo gadget professions:engineering cylinders",
	["inv_gizmo_05"] = "golden metal barbell gizmo gadget professions:engineering cylinders",
	["inv_gizmo_06"] = "blue orange lightning thunder silver steel metal gizmo gadget professions:engineering machine device",
	["inv_gizmo_07"] = "blue red lightning thunder silver steel metal gizmo gadget professions:engineering machine device",
	["inv_gizmo_08"] = "purple silver steel metal gizmo gadget professions:engineering machine device",
	["inv_gizmo_09"] = "blue silver steel metal gizmo gadget professions:engineering machine device",
	["inv_gizmo_adamantiteframe"] = "silver steel metal adamantite picture frame border edge bolted gizmo gadget professions:engineering",
	["inv_gizmo_adamantiteshells"] = "adamantite shells bullet gizmo gadget professions:engineering",
	["inv_gizmo_bronzeframework_01"] = "bronze picture frame border edge bolted gizmo gadget professions:engineering",
	["inv_gizmo_electrifiedether"] = "electrified ether yellow golden glass container vial sand charge bomb explosives gizmo gadget professions:engineering cylinder lightning thunder device",
	["inv_gizmo_elementalblastingpowder"] = "purple drinking horn flask elemental blasting powder steel metal",
	["inv_gizmo_felironbolts"] = "green fel iron bolts metal screws parts pieces engineering trade skill profession",
	["inv_gizmo_felironbomb"] = "green fel iron bomb explosives metal spiked spiky ball and chain gizmo gadget professions:engineering device",
	["inv_gizmo_felironcasing"] = "green fel iron metal casing pipe gizmo gadget professions:engineering cylinder",
	["inv_gizmo_felironshell"] = "green fel iron shells casing bullet",
	["inv_gizmo_felstabilizer"] = "felsteel stabilizer green iron metal gizmo gadget professions:engineering",
	["inv_gizmo_fuelcell"] = "red steel metal gas tank fuel cell canister container goblin barbecue gizmo gadget professions:engineering machine device",
	["inv_gizmo_gnomishflameturret"] = "silver steel metal gnomish flame turret gizmo gadget professions:engineering machine device",
	["inv_gizmo_goblinboombox_01"] = "black goblin boom box stereo noise machine sonic booster gizmo gadget professions:engineering device",
	["inv_gizmo_goblingtonkcontroller"] = "machine remote control steel metal gizmo gadget device",
	["inv_gizmo_hardenedadamantitetube"] = "hardened adamantite tube pipe cylinder silver steel metal gizmo gadget professions:engineering",
	["inv_gizmo_healthpotionpack"] = "health healing potion package bundle gizmo gadget box bag container",
	["inv_gizmo_khoriumpowercore"] = "khorium power core gizmo gadget professions:engineering cylinder charge bomb explosives glass container electrified blue lightning thunder device",
	["inv_gizmo_manapotionpack"] = "blue mana potion package bundle gizmo gadget box bag container",
	["inv_gizmo_manasyphon"] = "glass cup vial of blue steaming mana potion powder",
	["inv_gizmo_mithrilcasing_01"] = "silver steel metal mithril casing plate cover case gizmo gadget professions:engineering",
	["inv_gizmo_mithrilcasing_02"] = "silver steel metal plate cover case casing gizmo gadget",
	["inv_gizmo_newgoggles"] = "blue goggles spectracles glasses gizmo gadget device",
	["inv_gizmo_pipe_01"] = "bronze tube pipeline gizmo gadget professions:engineering",
	["inv_gizmo_pipe_02"] = "bronze tube pipeline gizmo gadget",
	["inv_gizmo_pipe_03"] = "golden bronze tube pipeline gizmo gadget",
	["inv_gizmo_pipe_04"] = "thorium tube pipeline gizmo gadget professions:engineering",
	["inv_gizmo_poltryiser_01"] = "polymorph chicken poltryiser poltryizer transformation human animal bird",
	["inv_gizmo_rocketboot_01"] = "rocket boots shoes flaming fire flames flying flight transportation engineering trade skill profession",
	["inv_gizmo_rocketboot_destroyed_02"] = "rocket boots shoes failure failed junk garbage engineering trade skill profession",
	["inv_gizmo_rocketbootextreme"] = "rocket boots shoes flaming fire flames flying flight transportation silver steel metal guns cannons engineering trade skill profession",
	["inv_gizmo_rocketlauncher"] = "red bomb explosives missile rpg launcherengineering trade skill profession racial:rocket_barrage(goblin)",
	["inv_gizmo_runichealthinjector"] = "red health healing runic injector potion injection shot needle engineering trade skill profession",
	["inv_gizmo_runicmanainjector"] = "blue mana runic injector potion injection shot needle engineering trade skill profession",
	["inv_gizmo_supersappercharge"] = "red orange atomic bomb explosives explosion fire fiery burning hot heat flames flaming blazing scorching searing destruction destroyed blowing blown super sapper charge engineering trade skill profession",
	["inv_gizmo_thebiggerone"] = "green shark face missile rocket rpg bomb explosives painted steel metal the bigger one engineering trade skill profession",
	["inv_gizmo_zapthrottlegascollector"] = "zapthrottle mote extractor plunger vacuum collector gas clouds engineering trade skill profession",
	["inv_glyph_majordeathknight"] = "red death knight major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majordruid"] = "orange druid major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorhunter"] = "green hunter major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majormage"] = "blue mage major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majormonk"] = "bluegreen cyan turquoise monk major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorpaladin"] = "purple pink paladin major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorpriest"] = "white priest major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorrogue"] = "yellow rogue major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorshaman"] = "blue shaman major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorwarlock"] = "purple warlock major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_majorwarrior"] = "orange brown warrior major glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minordeathknight"] = "red death knight minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minordruid"] = "orange druid minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorhunter"] = "green hunter minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minormage"] = "blue mage minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minormonk"] = "bluegreen cyan turquoise monk minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorpaladin"] = "purple pink paladin minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorpriest"] = "white priest minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorrogue"] = "yellow rogue minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorshaman"] = "blue shaman minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorwarlock"] = "purple warlock minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_minorwarrior"] = "orange brown warrior minor glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primedeathknight"] = "red death knight prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primedruid"] = "orange druid prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primehunter"] = "green hunter prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primemage"] = "blue mage prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primepaladin"] = "purple pink paladin prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primepriest"] = "white priest prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primerogue"] = "yellow rogue prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primeshaman"] = "blue shaman prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primewarlock"] = "purple warlock prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_glyph_primewarrior"] = "orange brown warrior prime glyph parchment paper rune scroll inscription trade skill profession",
	["inv_guild_cauldron_a"] = "small cauldron of battle pot flasks black steel metal",
	["inv_guild_cauldron_b"] = "big cauldron of battle pot flasks black steel metal glowing",
	["inv_hammer_03"] = "spell:weapon_skills(paladin)",
	["inv_hammer_05"] = "racial_passive:mace_specialization(human) racial_passive:mace_specialization(dwarf)",
	["inv_hammer_15"] = "brown wooden hammer mace weapon mallet spell:auto_attack(paladin)",
	["inv_hammer_16"] = "brown wooden hammer mace weapon mallet",
	["inv_helmet_08"] = "spell:unholy_aura(death_knight)",
	["inv_helmet_21"] = "spell:unwavering_sentinel(warrior) racial_passive:heroic_presence(draenei)",
	["inv_helmet_23"] = "racial_passive:hardiness(orc)",
	["inv_helmet_74"] = "spell:grand_crusader(paladin)",
	["inv_helmet_96"] = "spell:word_of_glory(paladin)",
	["inv_holiday_beerfestpretzel01"] = "brown salted salty pretzel food brewfest knot tied",
	["inv_holiday_beerfestsausage01"] = "food red brewfest sausage meat",
	["inv_holiday_beerfestsausage02"] = "food brown brewfest sausage meat",
	["inv_holiday_beerfestsausage03"] = "food pink brewfest sausage meat",
	["inv_holiday_beerfestsausage04"] = "food brown brewfest sausage meat",
	["inv_holiday_brewfestbuff_01"] = "yellow white flask cup mug pint mead root beer booze ale alcohol drink liquid foamy foaming frothy bubbling",
	["inv_holiday_christmas_present_01"] = "red green package wrapped gift present winter veil christmas holiday tied knot bow",
	["inv_holiday_christmas_present_02"] = "purple package wrapped gift present winter veil christmas holiday tied knot bow",
	["inv_holiday_christmas_present_03"] = "blue package wrapped gift present veil christmas holiday tied knot bow companion:father_winter's_helper",
	["inv_holiday_christmas_wrapping_01"] = "red white striped gift present package wrapping paper winter veil christmas holiday",
	["inv_holiday_christmas_wrapping_02"] = "blue white striped gift present package wrapping paper winter veil christmas holiday",
	["inv_holiday_christmas_wrapping_03"] = "green white striped gift present package wrapping paper winter veil christmas holiday",
	["inv_holiday_summerfest_petals"] = "leaves flower petals blowing windy falling fallen green red midsummer festival holiday",
	["inv_holiday_thanksgiving_cornucopia"] = "foods feast banquet cornucopia thanksgiving holiday pumpkin grapes apples radishes",
	["inv_holiday_thanksgiving_empty_cornucopia"] = "food feast banquet cornucopia thanksgiving holiday empty",
	["inv_holiday_tow_spicebandage"] = "brown bandage wrap cloth first aid powdered cinnamon spices food",
	["inv_holiday_tow_spicebowl"] = "wooden bowl cinnamon brown sugar spices powder liquid drink soup stew pouring dumping applying food",
	["inv_holiday_tow_spicebrownie"] = "brownie square treat snack food chocolatey sugary sweet cake",
	["inv_holiday_tow_spicepotion01"] = "brown cinnamon sugar powder liquid vial potion spices drink",
	["inv_holiday_tow_spicepotion02"] = "brown cinnamon sugar powder liquid vial potion spices drink",
	["inv_holiday_tow_spicepotion03"] = "brown cinnamon sugar powder liquid vial potion spices drink",
	["inv_ingot_01"] = "gray grey iron silver bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_02"] = "brown copper bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_03"] = "yellow golden bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_04"] = "gray grey stone iron marble bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_05"] = "yellow golden bronze bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_06"] = "green lime white mithril bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_07"] = "green lime silver steel thorium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_08"] = "gray grey truesilver bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_09"] = "brown copper khorium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_10"] = "blue silver gray grey adamantite bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_11"] = "purple eternium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_adamantite"] = "silver white gray grey hardened adamantite bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_bronze"] = "golden bronze bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_cobalt"] = "blue cobalt bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_eternium"] = "rainbow red yellow green blue enchanted thorium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_feliron"] = "green fel iron bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_felsteel"] = "green felsteel bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_ghostiron"] = "ghost iron bar ingot metal smelting smelted professions:mining shiny gray grey silver",
	["inv_ingot_iron"] = "gray grey silver iron bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_livingsteel"] = "living steel bar ingot metal smelting smelted professions:mining shiny green jade turquoise",
	["inv_ingot_manticyte"] = "manticyte bar ingot metal smelting smelted professions:mining shiny orange",
	["inv_ingot_mithril"] = "gray grey silver steel dark iron bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_platinum"] = "gray grey silver steel titanium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_steel"] = "gray grey silver iron steel bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_thorium"] = "purple enchanted elementium hardened khorium bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_titansteel_blue"] = "blue glowing cracked titansteel bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_titansteel_dark"] = "white glowing cracked bar ingot metal smelting smelted professions:mining shiny",
	["inv_ingot_titansteel_red"] = "red orange glowing cracked bar ingot metal smelting smelted professions:mining shiny fragment val'anyr",
	["inv_ingot_trillium"] = "trillium bar ingot metal smelting smelted professions:mining shiny silver gray grey",
	["inv_ingot_yoggthorite"] = "green saronite bar ingot metal smelting smelted professions:mining",
	["inv_inscription_armorscroll01"] = "scroll parchment paper buff enchanting vellium inscription trade skill profession",
	["inv_inscription_armorscroll02"] = "yellow golden shield scroll parchment paper buff enchanting vellium inscription trade skill profession",
	["inv_inscription_armorscroll03"] = "yellow golden shield scroll parchment paper buff enchanting vellium inscription trade skill profession",
	["inv_inscription_certificate"] = "resilient parchment paper blank plain empty inscription trade skill profession",
	["inv_inscription_crane"] = "white paper origami crane",
	["inv_inscription_deck_blackox"] = "blue cyan oxen deck darkmoon card inscription trade skill profession",
	["inv_inscription_deck_jadeserpent"] = "green serpents deck darkmoon card inscription trade skill profession",
	["inv_inscription_deck_redcrane"] = "red cranes deck darkmoon card inscription trade skill profession",
	["inv_inscription_deck_whitetiger"] = "white tigers deck darkmoon card inscription trade skill profession",
	["inv_inscription_frog"] = "white paper origami frog",
	["inv_inscription_ink_dreams"] = "ink dreams cyan bluegreen turquoise bottle jar inscription trade skill profession",
	["inv_inscription_ink_starlight"] = "starlight ink bottle jar inscription trade skill profession blue",
	["inv_inscription_inkblack01"] = "black midnight ink bottle jar inscription trade skill profession",
	["inv_inscription_inkblack02"] = "black ethereal ink bottle jar inscription trade skill profession",
	["inv_inscription_inkblack03"] = "blackfallow ink bottle jar inscription trade skill profession",
	["inv_inscription_inkblack04"] = "black darkflame ink bottle jar inscription trade skill profession",
	["inv_inscription_inkbluewhite01"] = "blue snowfall ink bottle jar inscription trade skill profession",
	["inv_inscription_inkbluewhite02"] = "blue ink bottle jar inscription trade skill profession",
	["inv_inscription_inkbluewhite03"] = "blue ink bottle jar inscription trade skill profession",
	["inv_inscription_inkbluewhite04"] = "blue ink bottle jar inscription trade skill profession",
	["inv_inscription_inkgreen01"] = "green ink bottle jar inscription trade skill profession",
	["inv_inscription_inkgreen02"] = "green hunter's ink bottle jar inscription trade skill profession",
	["inv_inscription_inkgreen03"] = "green jadefire ink bottle jar inscription trade skill profession",
	["inv_inscription_inkgreen04"] = "green ink bottle jar inscription trade skill profession",
	["inv_inscription_inkorange01"] = "orange ink bottle jar inscription trade skill profession",
	["inv_inscription_inkorange02"] = "orange ink bottle jar inscription trade skill profession",
	["inv_inscription_inkorange03"] = "orange ink bottle jar inscription trade skill profession",
	["inv_inscription_inkorange04"] = "orange ink bottle jar inscription trade skill profession",
	["inv_inscription_inkpurple01"] = "ink the sea bottle jar inscription trade skill profession",
	["inv_inscription_inkpurple02"] = "purple ink bottle jar inscription trade skill profession",
	["inv_inscription_inkpurple03"] = "purple celestial ink bottle jar inscription trade skill profession",
	["inv_inscription_inkpurple04"] = "purple ink bottle jar inscription trade skill profession",
	["inv_inscription_inkred01"] = "red fiery ink bottle jar inscription trade skill profession",
	["inv_inscription_inkred02"] = "red ink bottle jar inscription trade skill profession",
	["inv_inscription_inkred03"] = "inferno ink bottle jar inscription trade skill profession",
	["inv_inscription_inkred04"] = "red pink bottle jar inscription trade skill profession",
	["inv_inscription_inksilver01"] = "white silver gray grey shimmering ink bottle jar inscription trade skill profession",
	["inv_inscription_inksilver02"] = "white silver gray grey ink bottle jar inscription trade skill profession",
	["inv_inscription_inksilver03"] = "white silver gray grey ink bottle jar inscription trade skill profession",
	["inv_inscription_inksilver04"] = "white silver gray grey ink bottle jar inscription trade skill profession",
	["inv_inscription_inkwhite01"] = "white ink bottle jar inscription trade skill profession",
	["inv_inscription_inkwhite02"] = "white moonglow ink bottle jar inscription trade skill profession",
	["inv_inscription_inkwhite03"] = "white ivory ink bottle jar inscription trade skill profession",
	["inv_inscription_inkwhite04"] = "white ink bottle jar inscription trade skill profession",
	["inv_inscription_inkyellow01"] = "yellow ink bottle jar inscription trade skill profession",
	["inv_inscription_inkyellow02"] = "yellow lion's ink bottle jar inscription trade skill profession",
	["inv_inscription_inkyellow03"] = "yellow ink bottle jar inscription trade skill profession",
	["inv_inscription_inkyellow04"] = "yellow dawnstar ink bottle jar inscription trade skill profession",
	["inv_inscription_majorglyph00"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph01"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph02"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph03"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph04"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph05"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph06"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph07"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph08"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph09"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph10"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph11"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph12"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph13"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph14"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph15"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph16"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph17"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph18"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph19"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_majorglyph20"] = "orange glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph00"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph01"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph02"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph03"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph04"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph05"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph06"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph07"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph08"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph09"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph10"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph11"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph12"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph13"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph14"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph15"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph16"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph17"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph18"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph19"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_minorglyph20"] = "blue glyph parchment paper rune scroll inscription trade skill profession",
	["inv_inscription_papyrus"] = "brown paper parchment papyrus empty blank inscription trade skill profession bark",
	["inv_inscription_parchment"] = "brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar01"] = "red brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar02"] = "green brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar03"] = "purple brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar04"] = "blue brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar05"] = "yellow orange brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_parchmentvar06"] = "white brown paper parchment papyrus empty blank inscription trade skill profession",
	["inv_inscription_pigment_azure"] = "blue azure pigment inscription trade skill profession milling",
	["inv_inscription_pigment_bug01"] = "white blue moth bug winged wings flying insect glowing",
	["inv_inscription_pigment_bug03"] = "blue firefly bug insect glowing flying",
	["inv_inscription_pigment_bug04"] = "ladybug red insect winged wings",
	["inv_inscription_pigment_bug05"] = "purple moth bug winged wings flying insect glowing",
	["inv_inscription_pigment_bug06"] = "yellow firefly bug insect glowing flying companion:tiny_flamefly",
	["inv_inscription_pigment_bug07"] = "glowing bug insect glowworm",
	["inv_inscription_pigment_burnt"] = "orange pigment inscription trade skill profession milling",
	["inv_inscription_pigment_ebon"] = "black ebon pigment inscription trade skill profession milling",
	["inv_inscription_pigment_emerald"] = "green emerald pigment inscription trade skill profession milling",
	["inv_inscription_pigment_golden"] = "yellow golden pigment inscription trade skill profession milling",
	["inv_inscription_pigment_grey"] = "white gray grey dusky pigment inscription trade skill profession milling",
	["inv_inscription_pigment_icy"] = "blue cyan icy pigment inscription trade skill profession milling",
	["inv_inscription_pigment_indigo"] = "purple indigo pigment inscription trade skill profession milling",
	["inv_inscription_pigment_misty"] = "misty pigment gray grey silver inscription trade skill profession milling",
	["inv_inscription_pigment_nether"] = "pink purple nether pigment inscription trade skill profession milling",
	["inv_inscription_pigment_ruby"] = "red ruby pigment inscription trade skill profession milling",
	["inv_inscription_pigment_sapphire"] = "blue sapphire pigment inscription trade skill profession milling",
	["inv_inscription_pigment_shadow"] = "shadow pigment inscription trade skill profession milling purple violet",
	["inv_inscription_pigment_silvery"] = "white gray grey silvery pigment inscription trade skill profession milling",
	["inv_inscription_pigment_verdant"] = "green verdant pigment inscription trade skill profession milling",
	["inv_inscription_pigment_violet"] = "purple violet pigment inscription trade skill profession milling",
	["inv_inscription_pigment_white"] = "white alabaster pigment inscription trade skill profession milling",
	["inv_inscription_runescrolloffortitude_blue"] = "blue inscription scroll parchment paper papyrus",
	["inv_inscription_runescrolloffortitude_green"] = "green inscription scroll parchment paper papyrus",
	["inv_inscription_runescrolloffortitude_red"] = "red inscription scroll parchment paper papyrus",
	["inv_inscription_runescrolloffortitude_yellow"] = "yellow inscription scroll parchment paper papyrus",
	["inv_inscription_scroll"] = "white parchment papyrus paper scroll empty blank inscription trade skill profession",
	["inv_inscription_scrollofwisdom_01"] = "red scroll paper parchment papyrus",
	["inv_inscription_scrollofwisdom_02"] = "red scroll paper parchment papyrus",
	["inv_inscription_tarot_earthquakeace"] = "darkmoon card inscription trade skill profession brown yellow stones earthquake deck",
	["inv_inscription_tarot_earthquakecard"] = "brown yellow stones earthquake deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_hurricaneace"] = "blue winds hurricane deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_hurricanecard"] = "blue winds hurricane deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_tsunamiace"] = "blue waves tsunami deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_tsunamicard"] = "blue waves tsunami deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_volcanoace"] = "red orange fire flames embers volcano deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarot_volcanocard"] = "red orange fire flames embers volcano deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarotberserker"] = "darkmoon card inscription trade skill profession red orange berserker chaos deck",
	["inv_inscription_tarotchaos"] = "darkmoon card inscription trade skill profession red orange berserker chaos deck",
	["inv_inscription_tarotdeath"] = "darkmoon card inscription trade skill profession red orange undeath deck",
	["inv_inscription_tarotgreatness"] = "darkmoon card inscription trade skill profession red greatness nobles deck",
	["inv_inscription_tarotillusion"] = "blue turquoise cyan illusion prisms deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarotlords"] = "purple golden yellow nobles greatness deck darkmoon card inscription trade skill profession crown",
	["inv_inscription_tarotprism"] = "purple illusion prisms deck darkmoon card inscription trade skill profession",
	["inv_inscription_tarotundeath"] = "green undeath deck darkmoon card inscription trade skill profession",
	["inv_inscription_tomeoftheclearmind"] = "brown tome of clear mind book libram leather bound eyeball",
	["inv_inscription_tooltip_darkmooncard_mop"] = "darkmoon card mists inscription trade skill profession",
	["inv_inscription_tradeskill01"] = "feather quill inkwell scroll parchment paper papyrus rolled scribe writing professions:inscription",
	["inv_inscription_trinket_crane"] = "gray grey relic of chi ji crane statue stone bird winged feathered feathers animal creature",
	["inv_inscription_trinket_dragon"] = "blue gray grey relic of yu'lon stone statue lizard cloud serpent dragon beast animal creature scaled scales scaly reptile",
	["inv_inscription_trinket_ox"] = "green gray grey relic of niuzao stone ox tauren bull horned horns furry fuzzy beast animal creature",
	["inv_inscription_trinket_tiger"] = "purple gray grey relic of xuen stone tiger striped cat kitty feline furry fuzzy beast animal creature",
	["inv_inscription_weaponscroll01"] = "yellow hammer mace weapon scroll buff inscription trade skill profession",
	["inv_inscription_weaponscroll02"] = "yellow battleaxe blade weapon scroll buff inscription trade skill profession",
	["inv_inscription_weaponscroll03"] = "yellow sword blade weapon scroll buff inscription trade skill profession",
	["inv_jewelcrafting_azureboar"] = "azure blue sapphire boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_azurecrab"] = "azure blue sapphire crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_azurehare"] = "azure blue sapphire rabbit gopher prarie dog critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_azureowl"] = "azure blue sapphire owl bird winged feathered feathers animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_azureserpent"] = "azure blue sapphire snake beast animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_blackpearlpanther"] = "azure blue obsidian stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_bronzesetting"] = "copper bronze setting frame metal jewelcrafting trade skill profession",
	["inv_jewelcrafting_crimsonboar"] = "crimson red ruby boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_crimsoncrab"] = "crimson red ruby crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_crimsonhare"] = "crimson red ruby rabbit gopher prarie dog critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_crimsonowl"] = "crimson ruby owl bird winged feathered feathers animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_crimsonspinel_01"] = "crimson spinel red gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_crimsonspinel_02"] = "crimson spinel red gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_dawnstone_01"] = "yellow golden gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_dawnstone_02"] = "yellow golden dawnstone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_dawnstone_03"] = "yellow golden dawnstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_delicatecopperwire"] = "brown delicate copper wire jewelcrafting trade skill profession",
	["inv_jewelcrafting_dragonseye01"] = "dragon's eye prismatic gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_dragonseye02"] = "dragon's eye prismatic gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_dragonseye03"] = "yellow golden dragon's eye chimera's gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_dragonseye04"] = "blue dragon's eye chimera's gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_dragonseye05"] = "red dragon's eye chimera's gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_emeraldboar"] = "emerald green jade boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_emeraldcrab"] = "emerald green jade crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_emeraldhare"] = "emerald green jade rabbit gopher prarie dog critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_empyreansapphire_01"] = "blue empyrean sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_empyreansapphire_02"] = "blue empyrean sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_01"] = "green forest emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_02"] = "orange monarch topaz gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_03"] = "yellow golden autumn's glow gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_04"] = "red scarlet ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_05"] = "blue purple sky sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_06"] = "purple twilight opal gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_gem_07"] = "green dark jade gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_08"] = "yellow golden sun crystal gemstone shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_09"] = "orange huge citrine gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_10"] = "blue chalcedony gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_11"] = "purple shadow crystal gemstone shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_12"] = "red bloodstone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_13"] = "green dark jade gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_14"] = "orange huge citrine gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_15"] = "yellow golden sun crystal gemstone shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_16"] = "red bloodstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_17"] = "blue chalcedony gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_18"] = "purple shadow crystal gemstone shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_jewelcrafting_gem_19"] = "green dark jade gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_20"] = "orange huge citrine gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_21"] = "yellow golden sun crystal gemstone shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_22"] = "red bloodstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_23"] = "purple shadow crystal gemstone shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_24"] = "blue chalcedony gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_25"] = "green forest emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_26"] = "yellow golden autumn's glow gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_27"] = "blue purple sky sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_28"] = "red scarlet ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_29"] = "purple twilight opal gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_30"] = "orange monarch topaz gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_31"] = "purple dreadstone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_32"] = "red cardinal ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_33"] = "orange ametrine gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_34"] = "green eye zul gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_35"] = "blue majestic zircon gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_36"] = "yellow golden king's amber gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_gem_37"] = "red cardinal ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_38"] = "yellow golden king's amber gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_39"] = "orange ametrine gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_40"] = "purple dreadstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_41"] = "green eye zul gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_gem_42"] = "blue majestic zircon gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_goldenboar"] = "yellow golden boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_goldencrab"] = "yellow golden crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_goldenhare"] = "yellow golden rabbit gopher prarie dog critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_goldenowl"] = "yellow golden owl bird winged feathered feathers animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_goldenserpent"] = "yellow golden snake beast animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_icediamond_01"] = "blue cyan skyflare diamond gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_icediamond_02"] = "blue cyan skyflare diamond shard fragment gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_jadeowl"] = "emerald green jade owl bird winged feathered feathers animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_jadeserpent"] = "emerald green jade snake beast animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_lionseye_01"] = "yellow golden lionseye gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_lionseye_02"] = "yellow golden lionseye gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_livingruby_01"] = "red ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_livingruby_02"] = "red living ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_livingruby_03"] = "red living ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_mithrilfiligree"] = "mithril filigree white designs decorations jewelcrafting trade skill profession",
	["inv_jewelcrafting_nightseye_01"] = "purple gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_nightseye_02"] = "purple nightseye gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_nightseye_03"] = "purple tanzanite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_nobletopaz_01"] = "orange gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_nobletopaz_02"] = "orange noble topaz gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_nobletopaz_03"] = "orange noble topaz fire opal gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_purpleboar"] = "purple amythyst boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_purplecrab"] = "purple amythyst crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_purplehare"] = "purple amythyst rabbit gopher prarie dog critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_purpleowl"] = "purple amythyst owl bird winged feathered feathers animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_purpleserpent"] = "purple amythyst snake beast animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_pyrestone_01"] = "orange pyrestone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_pyrestone_02"] = "orange pyrestone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_rubyserpent"] = "crimson red ruby snake beast animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelcrafting_seasprayemerald_01"] = "green seaspray emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_seasprayemerald_02"] = "green seaspray emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_shadowsongamethyst_01"] = "purple shadowsong amethyst gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_shadowsongamethyst_02"] = "purple shadowsong amethyst gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_shadowspirit_01"] = "purple earthsiege diamond gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_shadowspirit_02"] = "purple earthsiege diamond gemstone crystal shiny sparkling jewelcrafting trade skill profession cut passive:metamorphosis:_touch_of_chaos(warlock)",
	["inv_jewelcrafting_starofelune_01"] = "blue eye the sea gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_starofelune_02"] = "blue star elune gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_starofelune_03"] = "blue star elune gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_talasite_01"] = "green gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_jewelcrafting_talasite_02"] = "green talasite gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_jewelcrafting_talasite_03"] = "green talasite chrysoprase gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_jewelcrafting_thoriumsetting"] = "blue cyan frame thorium setting jewelcrafting trade skill profession",
	["inv_jewelcrafting_truesilverboar"] = "azure pearl blue cyan truesilver boar pig hog beast animal creature stone figurine statuette jewelcrafting trade skill profession",
	["inv_jewelcrafting_truesilvercrab"] = "azure pearl blue cyan truesilver crab sea critter animal creature stone figurine jewelcrafting trade skill profession",
	["inv_jewelry_necklace_22"] = "pet_command:spiked_collar(hunter)",
	["inv_knife_1h_cataclysm_c_05"] = "spell:crimson_tempest(rogue)",
	["inv_knife_1h_deathwingraid_d_02"] = "spell:auto_attack(mage)",
	["inv_knife_1h_grimbatolraid_d_03"] = "spell:shadow_blades(rogue)",
	["inv_legendary_axe"] = "battleaxe blade weapon yellow orange black silhouette",
	["inv_legendary_fistweapon"] = "fist weapon yellow orange black silhouette",
	["inv_legendary_gun"] = "rifle gun yellow orange black silhouette",
	["inv_legendary_knife"] = "dagger blade weapon knife yellow orange black silhouette",
	["inv_legendary_mace"] = "mace weapon yellow orange black silhouette",
	["inv_legendary_shield"] = "shield armor defense protection blocking yellow orange black silhouette",
	["inv_legendary_sigilofpower"] = "sigil of power bronze copper coin lightning emblem symbol",
	["inv_legendary_sigilofwisdom"] = "sigil of wisdom bronze copper coin radiant radiance light beams sun emblem symbol",
	["inv_legendary_staff"] = "staff weapon yellow orange black silhouette",
	["inv_legendary_sword"] = "sword blade weapon yellow orange black silhouette",
	["inv_letter_01"] = "white mail package letter envelope paper",
	["inv_letter_02"] = "white mail package letter envelope paper",
	["inv_letter_03"] = "white mail stack of letters papers envelopes packages",
	["inv_letter_04"] = "brown mail package letter envelope paper",
	["inv_letter_05"] = "brown mail package letter envelope paper",
	["inv_letter_06"] = "brown mail stack of letters papers envelopes packages",
	["inv_letter_07"] = "brown mail package letter envelope paper",
	["inv_letter_08"] = "brown mail package letter envelope paper",
	["inv_letter_09"] = "white mail letters envelopes paper",
	["inv_letter_10"] = "white mail package letter envelope paper",
	["inv_letter_11"] = "white mail letters envelopes paper",
	["inv_letter_12"] = "white mail package letter envelope paper",
	["inv_letter_13"] = "white mail package letter envelope paper",
	["inv_letter_14"] = "brown mail stack of letters papers envelopes packages",
	["inv_letter_15"] = "brown mail package letter envelope paper",
	["inv_letter_16"] = "green skull mail package letter envelope paper",
	["inv_letter_17"] = "purple skull mail package letter envelope paper",
	["inv_letter_18"] = "yellow note letter card paper greetings address tag mail",
	["inv_letter_19"] = "green mail package letter envelope paper",
	["inv_letter_20"] = "brown mail package letter envelope paper",
	["inv_letter_21"] = "blue mail package letter envelope paper",
	["inv_letter_22"] = "purple mail package letter envelope paper",
	["inv_mace_03"] = "passive:weapon_skills(priest) spell:auto_attack(shaman)",
	["inv_mace_11"] = "club cudgel mace wooden brown silver steel metal",
	["inv_mace_110"] = "",
	["inv_mace_12"] = "passive:weapon_skills(shaman)",
	["inv_mace_46"] = "",
	["inv_mace_62"] = "",
	["inv_mace_82"] = "",
	["inv_mask_01"] = "white mask face hallow's end holiday halloween",
	["inv_mask_02"] = "white green skull mask face undead undeath demonic glowing tusks ghoul skeleton skeletal hallow's end holiday halloween",
	["inv_mask_03"] = "blood elf female mask face hallow's end holiday halloween",
	["inv_mask_04"] = "blood elf male mask face hallow's end holiday halloween",
	["inv_mask_05"] = "blue draenei male mask face hallow's end holiday halloween",
	["inv_mask_06"] = "blue draeni female mask face hallow's end holiday halloween",
	["inv_mask_07"] = "green goblin female mask face hallow's end holiday halloween",
	["inv_mask_08"] = "gray grey worgen male mask face hallow's end holiday halloween",
	["inv_mask_09"] = "gray grey worgen female mask face hallow's end holiday halloween",
	["inv_mask_10"] = "green goblin male mask face hallow's end holiday halloween",
	["inv_misc_2h_bamboo_fishing"] = "pandaren fishing pole green reel line",
	["inv_misc_2h_panda_fishing"] = "blue dragon fishing pole cloud serpent beast animal creature scaled scales scaly reptile",
	["inv_misc_abyssalclam"] = "clam pearl sea creature animal shell",
	["inv_misc_ammo_bullet_01"] = "bullets weapon ammunition ammo bag brown sack pouch flash bomb engineering trade skill profession",
	["inv_misc_ammo_bullet_02"] = "brown bullet weapon ammunition ammo copper metal",
	["inv_misc_ammo_bullet_03"] = "silver gray grey bullet weapon ammunition ammo steel metal",
	["inv_misc_ammo_bullet_04"] = "silver gray grey bullet weapon ammunition ammo steel metal",
	["inv_misc_ammo_bullet_05"] = "silver gray grey bullet weapon ammunition ammo steel metal",
	["inv_misc_ammo_bullet_06"] = "silver gray grey bullet weapon ammunition ammo steel metal",
	["inv_misc_ammo_gunpowder_01"] = "brown gray grey blasting gunpowder dust sand flux engineering trade skill profession",
	["inv_misc_ammo_gunpowder_02"] = "brown gray grey blasting gunpowder dust sand flux engineering trade skill profession",
	["inv_misc_ammo_gunpowder_03"] = "purple gray grey blasting gunpowder dust sand flux",
	["inv_misc_ammo_gunpowder_04"] = "green gray grey blasting gunpowder dust sand flux",
	["inv_misc_ammo_gunpowder_05"] = "blue blasting gunpowder dust sand flux northern spices",
	["inv_misc_ammo_gunpowder_06"] = "silver gray grey blasting gunpowder dust sand flux",
	["inv_misc_ammo_gunpowder_07"] = "green horn elemental blasting powder",
	["inv_misc_apexis_crystal"] = "green glowing apexis crystal shard gemstone",
	["inv_misc_apexis_shard"] = "green crystal shard gemstone",
	["inv_misc_archaeology_amburfly"] = "amber insect mosquito stone trapped caught stuck inside bug hardened flying",
	["inv_misc_archaeology_babypterrodax"] = "green raptor pterrodax pterrodactyl dinosaur prehistoric cute baby infant hatching hatched eggshell",
	["inv_misc_archaeology_trolldrum"] = "drumsticks instrument music sound beat haunted war",
	["inv_misc_archaeology_trollgolem"] = "silver gray grey stone golem troll statue rock glowing eyes",
	["inv_misc_archaeology_vrykuldrinkinghorn"] = "brown wooden flask cup drinking horn foamy foaming liquid frothy bubbling overflowing root beer mead booze ale alcohol vrykul",
	["inv_misc_archstone_01"] = "blue cyan stone rock shard fragment rune",
	["inv_misc_armorkit_01"] = "brown iron shield spike blacksmithing trade skill profession",
	["inv_misc_armorkit_02"] = "green mithril shield spike blacksmithing trade skill profession",
	["inv_misc_armorkit_03"] = "black insignia up arrow",
	["inv_misc_armorkit_04"] = "brown insignia up arrow",
	["inv_misc_armorkit_05"] = "brown core armor kit leatherworking trade skill profession",
	["inv_misc_armorkit_06"] = "purple blue armor kit leather",
	["inv_misc_armorkit_07"] = "brown hammer thick armor kit leatherworking trade skill profession",
	["inv_misc_armorkit_08"] = "purple hammer armor kit",
	["inv_misc_armorkit_09"] = "black rugged armor kit leatherworking trade skill profession",
	["inv_misc_armorkit_10"] = "purple armor kit",
	["inv_misc_armorkit_11"] = "red silver gray grey zulian coin zul'gurub gurubashi zandalari troll",
	["inv_misc_armorkit_12"] = "blue silver gray grey iron belt buckle blacksmithing trade skill profession",
	["inv_misc_armorkit_14"] = "green dragonscale",
	["inv_misc_armorkit_15"] = "brown medium armor kit leatherworking trade skill profession silver spike",
	["inv_misc_armorkit_16"] = "brown heavy armor kit studded leatherworking trade skill profession",
	["inv_misc_armorkit_17"] = "brown light armor kit trade skill professions:leatherworking",
	["inv_misc_armorkit_18"] = "red armor kit leatherworking trade skill profession",
	["inv_misc_armorkit_19"] = "blue studded leather armor kit",
	["inv_misc_armorkit_20"] = "silver gray grey steel metal thorium shield blacksmithing trade skill profession spikes",
	["inv_misc_ashenpigment"] = "ashen pigment inscription trade skill profession milling black",
	["inv_misc_babyarmadillopet"] = "critter animal creature baby companion:armadillo_pup",
	["inv_misc_bag_01"] = "brown leather bag container sack satchel pouch",
	["inv_misc_bag_02"] = "pink leather bag container sack satchel pouch",
	["inv_misc_bag_03"] = "green leather bag container sack satchel pouch",
	["inv_misc_bag_04"] = "purple leather bag container sack satchel pouch",
	["inv_misc_bag_05"] = "blue leather bag container sack satchel pouch",
	["inv_misc_bag_06"] = "bluegreen turquoise cyan leather bag container sack satchel pouch",
	["inv_misc_bag_07"] = "brown leather bag backpack container sack",
	["inv_misc_bag_07_black"] = "silver gray grey leather bag container sack satchel pouch",
	["inv_misc_bag_07_blue"] = "blue leather bag container sack satchel pouch",
	["inv_misc_bag_07_green"] = "green leather bag container sack satchel pouch",
	["inv_misc_bag_07_red"] = "red leather bag container sack satchel pouch",
	["inv_misc_bag_08"] = "brown leather bag container sack satchel pouch",
	["inv_misc_bag_09"] = "brown leather bag knapsack container satchel pouch belt",
	["inv_misc_bag_09_black"] = "black leather bag knapsack container satchel pouch belt",
	["inv_misc_bag_09_blue"] = "blue leather bag knapsack container satchel pouch belt",
	["inv_misc_bag_09_green"] = "green leather bag knapsack container satchel pouch belt",
	["inv_misc_bag_09_red"] = "red leather bag knapsack container satchel pouch belt",
	["inv_misc_bag_10"] = "brown leather bag container sack",
	["inv_misc_bag_10_black"] = "black leather bag container sack",
	["inv_misc_bag_10_blue"] = "blue leather bag container sack",
	["inv_misc_bag_10_green"] = "green leather bag container sack",
	["inv_misc_bag_10_red"] = "red leather bag container sack",
	["inv_misc_bag_11"] = "red leather bag container sack spell:pick_pocket(rogue)",
	["inv_misc_bag_12"] = "green leather bag container sack",
	["inv_misc_bag_13"] = "brown leather backpack knapsack bag container",
	["inv_misc_bag_14"] = "blue mooncloth leather backpack knapsack bag container",
	["inv_misc_bag_15"] = "purple sack bag container locked padlock",
	["inv_misc_bag_16"] = "blue sack bag container locked padlock",
	["inv_misc_bag_17"] = "blue white sack bag container locked padlock",
	["inv_misc_bag_18"] = "green brown leaf herbalist herbalism herbs nature leather backpack knapsack bag container",
	["inv_misc_bag_19"] = "brown leather backpack knapsack bag container",
	["inv_misc_bag_20"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_21"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_22"] = "red leather backpack knapsack bag container",
	["inv_misc_bag_23_netherweave"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_24_netherweave_imbued"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_25_mooncloth"] = "purple white primal mooncloth leather backpack knapsack bag container",
	["inv_misc_bag_26_spellfire"] = "red leather dragon hide bag backpack knapsack container",
	["inv_misc_bag_27"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_28_halloween"] = "orange pumpkin face carved carving jack-o'-lantern hallow's end holiday halloween scary",
	["inv_misc_bag_29"] = "brown leather backpack knapsack bag container",
	["inv_misc_bag_30"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_31"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_32"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_33"] = "white leather backpack knapsack bag container",
	["inv_misc_bag_34"] = "blue leather backpack knapsack bag container",
	["inv_misc_bag_35"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_36"] = "brown leather backpack knapsack bag container",
	["inv_misc_bag_37"] = "blue leather backpack knapsack bag container",
	["inv_misc_bag_bigbagofenchantments"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_cenarionherbbag"] = "green leather backpack knapsack bag container",
	["inv_misc_bag_corefelclothbag"] = "purple leather backpack knapsack bag container",
	["inv_misc_bag_enchantedmageweave"] = "blue leather backpack knapsack bag container",
	["inv_misc_bag_enchantedrunecloth"] = "blue cloth sack bag container",
	["inv_misc_bag_felclothbag"] = "blue purple cloth sack bag container",
	["inv_misc_bag_herbpouch"] = "green leather satchel pouch bag container",
	["inv_misc_bag_satchelofcenarius"] = "green sack bag container satchel pouch",
	["inv_misc_bag_soulbag"] = "black red leather satchel pouch bag container",
	["inv_misc_balloon_01"] = "yellow balloon rubber floating flying",
	["inv_misc_balloon_02"] = "red rubber floating flying companion:horde_balloon",
	["inv_misc_balloon_03"] = "green balloon rubber floating flying",
	["inv_misc_balloon_04"] = "blue yellow golden alliance balloon rubber floating flying",
	["inv_misc_bandage_01"] = "brown silk bandage cloth wrap rolled first aid trade skill profession",
	["inv_misc_bandage_02"] = "white heavy silk bandage cloth wrap rolled first aid trade skill profession",
	["inv_misc_bandage_03"] = "brown bandage cloth wrap rolled paper",
	["inv_misc_bandage_04"] = "blue bandage cloth wrap rolled paper",
	["inv_misc_bandage_05"] = "blue bandage cloth wrap torn tattered ripped",
	["inv_misc_bandage_06"] = "brown bandana headband bandage cloth wrap tied knotted",
	["inv_misc_bandage_07"] = "white paper papyrus stacked sheets",
	["inv_misc_bandage_08"] = "yellow brown bandage cloth wrap torn tattered ripped",
	["inv_misc_bandage_09"] = "bandana headband bandage cloth wrap tied knotted",
	["inv_misc_bandage_10"] = "brown paper papyrus stacked sheets",
	["inv_misc_bandage_11"] = "brown runecloth bandage wrap rolled first aid trade skill profession",
	["inv_misc_bandage_12"] = "brown heavy runecloth bandage wrap rolled first aid trade skill profession",
	["inv_misc_bandage_13"] = "white cloth bandage wrap rolled",
	["inv_misc_bandage_14"] = "white cloth bandage wrap strip",
	["inv_misc_bandage_15"] = "brown cloth bandage wrap strip",
	["inv_misc_bandage_16"] = "red orange embersilk bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_17"] = "white heavy wool bandage cloth paper parchment papyrus wrap rolled first aid trade skill profession",
	["inv_misc_bandage_18"] = "brown heavy linen bandage cloth paper parchment papyrus wrap rolled first aid trade skill profession",
	["inv_misc_bandage_19"] = "brown mageweave bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_20"] = "green heavy mageweave bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_frostweave"] = "blue frostweave bandage ice icy cold frozen freezing frosty cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_frostweave_heavy"] = "blue heavy frostweave bandage ice icy cold frozen freezing frosty cloth wrap rolled first aid trade skill profession",
	["inv_misc_bandage_netherweave"] = "purple netherweave bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_netherweave_heavy"] = "purple heavy netherweave bandage cloth wrap rolled first aid trade skill profession",
	["inv_misc_bandage_windwool"] = "bluegreen turquoise cyan windwool bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandage_windwool_heavy"] = "bluegreen turquoise cyan heavy windwool bandage cloth wrap strip first aid trade skill profession",
	["inv_misc_bandana_01"] = "blue cloth bandana headband bandage wrap tied knotted",
	["inv_misc_bandana_03"] = "red cloth bandana headband bandage wrap tied knotted",
	["inv_misc_basket_01"] = "brown pink romantic picnic basket food lunch",
	["inv_misc_basket_02"] = "brown basket of apples food",
	["inv_misc_basket_03"] = "brown basket of radishes herbs cherries mistletoe",
	["inv_misc_basket_04"] = "brown basket of fish trout salmon",
	["inv_misc_basket_05"] = "brown basket empty container",
	["inv_misc_bearcubbrown"] = "brown furry beast animal creature cute companion:hyjal_bear_cub",
	["inv_misc_beer_01"] = "yellow tankard cup mug root beer mead booze ale alcohol drink liquid empty open",
	["inv_misc_beer_02"] = "yellow tankard cup mug root beer mead booze ale alcohol drink liquid closed",
	["inv_misc_beer_03"] = "blue tankard cup mug root beer mead booze ale alcohol drink liquid empty open",
	["inv_misc_beer_04"] = "blue tankard cup mug root beer mead booze ale alcohol drink liquid closed",
	["inv_misc_beer_05"] = "green tankard cup mug root beer mead booze ale alcohol drink liquid empty open",
	["inv_misc_beer_06"] = "green tankard cup mug root beer mead booze ale alcohol drink liquid closed",
	["inv_misc_beer_07"] = "purple tankard cup mug root beer mead booze ale alcohol drink liquid empty open",
	["inv_misc_beer_08"] = "purple tankard cup mug root beer mead booze ale alcohol drink liquid closed",
	["inv_misc_beer_09"] = "red tankard cup mug root beer mead booze ale alcohol drink liquid empty open",
	["inv_misc_beer_10"] = "red tankard cup mug root beer mead booze ale alcohol drink liquid closed",
	["inv_misc_beerhat_a_01"] = "yellow brown brewhelm beer hat root mead booze ale alcohol drink liquid keg barrel cap",
	["inv_misc_bell_01"] = "silver gray grey steel metal bell ringing sound noise instrument music",
	["inv_misc_birdbeck_01"] = "beak bird winged feathered feathers animal creature",
	["inv_misc_birdbeck_02"] = "crab claw sea critter animal creature",
	["inv_misc_blackfallowink"] = "black jar jug container inkwell",
	["inv_misc_blackfollowing"] = "black jar jug container inkwell",
	["inv_misc_blackironbomb"] = "black bomb explosives fuse lit steel metal",
	["inv_misc_blackironore"] = "black metal obsidian obsidium ore smooth rock fragments shards pieces broken shattered scattered mining trade skill profession",
	["inv_misc_blizzcon09_graphicscard"] = "gizmo gadget machine device viewer spyglass smoke gear video camera surveillance",
	["inv_misc_blizzcon09_poster"] = "blizzcon blizzard convention poster advertisement flier murloc",
	["inv_misc_boarshead"] = "boar pig hog beast animal creature head face tusks snout derpy",
	["inv_misc_bomb_01"] = "black silver gray grey steel metal bomb explosives sphere ball",
	["inv_misc_bomb_02"] = "silver gray grey steel metal spiky spikes spiked ball and chain sphere",
	["inv_misc_bomb_03"] = "red stick of dynamite tnt bomb explosives firework firecracker lit fuse",
	["inv_misc_bomb_04"] = "yellow smiley face bomb explosives sphere ball",
	["inv_misc_bomb_05"] = "red bundle sticks of dynamite tnt fireworks firecrackers explosives bombs lit fuse",
	["inv_misc_bomb_06"] = "red stick of dynamite tnt firework firecracker explosives bomb fuse unlit",
	["inv_misc_bomb_07"] = "red bundle sticks of dynamite tnt fireworks firecrackers explosives bombs lit fuse string tied",
	["inv_misc_bomb_08"] = "brown grenade stick bomb explosives",
	["inv_misc_bomb_09"] = "red bomb explosives lit fuse smoking ball",
	["inv_misc_bone_01"] = "white bone skeletal femur companion:perky_pug",
	["inv_misc_bone_02"] = "white bone skeletal ribcage",
	["inv_misc_bone_03"] = "white skeletal spine tail bones",
	["inv_misc_bone_04"] = "white tusk tooth ivory mammoth curved bone skeletal",
	["inv_misc_bone_05"] = "white tusk tooth ivory mammoth curved bone skeletal",
	["inv_misc_bone_06"] = "white fang tusk tooth bone skeletal sharp",
	["inv_misc_bone_07"] = "white tusks teeth bones skeletal fragments pieces sharp",
	["inv_misc_bone_08"] = "white tooth bone skeletal sharp",
	["inv_misc_bone_09"] = "white teeth bones skeletal jaw mouth grin sharp",
	["inv_misc_bone_10"] = "white bone skeletal",
	["inv_misc_bone_dwarfskull_01"] = "white skull bone skeletal dead death",
	["inv_misc_bone_elfskull_01"] = "white skull bone skeletal dead death",
	["inv_misc_bone_humanskull_01"] = "white skull bone skeletal dead death",
	["inv_misc_bone_humanskull_02"] = "white yellow human skull bone skeletal dead death",
	["inv_misc_bone_orcskull_01"] = "white skull bone skeletal dead death",
	["inv_misc_bone_skull_01"] = "green skull bone skeletal demon troll spell:control_undead(death_knight)",
	["inv_misc_bone_skull_02"] = "blue skull bone skeletal dead death demon troll",
	["inv_misc_bone_skull_03"] = "yellow skull bone skeletal dead death demon troll",
	["inv_misc_bone_taurenskull_01"] = "white tauren bone skeletal dead death spell:skull_bash(druid)",
	["inv_misc_bonenecklace"] = "white bone skeletal necklace charm",
	["inv_misc_book_01"] = "purple red tome libram spellbook bookmark skull",
	["inv_misc_book_02"] = "green pandaren tome libram spellbook",
	["inv_misc_book_03"] = "purple tome libram spellbook",
	["inv_misc_book_04"] = "purple blue crest emblem tome libram spellbook",
	["inv_misc_book_05"] = "blue purple red tome libram spellbook bookmark skull",
	["inv_misc_book_06"] = "black orange yellow golden red tome libram spellbook bookmark skull",
	["inv_misc_book_07"] = "green orange yellow golden tome libram spellbook",
	["inv_misc_book_08"] = "blue tome libram spellbook",
	["inv_misc_book_09"] = "brown leather steel tome libram spellbook",
	["inv_misc_book_10"] = "yellow red golden tome libram spellbook",
	["inv_misc_book_11"] = "brown tome libram spellbook",
	["inv_misc_book_12"] = "yellow blue golden alliance emblem symbol tome libram spellbook",
	["inv_misc_book_13"] = "yellow golden white tome libram spellbook",
	["inv_misc_book_14"] = "black yellow golden horde emblem symbol tome libram spellbook",
	["inv_misc_book_15"] = "red white horde emblem symbol tome libram spellbook",
	["inv_misc_book_16"] = "purple brown leather tome libram spellbook",
	["inv_misc_book_17"] = "golden yellow orange blue tome libram spellbook",
	["inv_misc_book_18"] = "blue brown leather bound tome libram spellbook gemmed sapphire",
	["inv_misc_bowl_01"] = "gray grey silver brown wooden tin bowl",
	["inv_misc_branch_01"] = "brown green tree branch",
	["inv_misc_breadofthedead"] = "brown bread of the food zombie undead demon stitched mouth silenced",
	["inv_misc_candle_01"] = "white candle melting melted wick wax",
	["inv_misc_candle_02"] = "white candles melting melted wax",
	["inv_misc_candle_03"] = "white candle melting melted wick wax",
	["inv_misc_candyskull"] = "colorful decorative festive skull mask hearts face of the dead holiday",
	["inv_misc_cauldron_arcane"] = "white bubbling black cauldron brew liquid drink goo potion hot heated cooking food stew",
	["inv_misc_cauldron_fire"] = "red bubbling black cauldron brew liquid drink goo potion hot heated cooking food stew health healing",
	["inv_misc_cauldron_frost"] = "blue bubbling black cauldron brew liquid drink goo potion hot heated cooking food stew mana",
	["inv_misc_cauldron_nature"] = "green bubbling black cauldron brew liquid drink goo potion hot heated cooking food stew nature",
	["inv_misc_cauldron_shadow"] = "purple bubbling black cauldron brew liquid drink goo potion hot heated cooking food stew",
	["inv_misc_celebrationcake_01"] = "cake star cherries yellow green pink baked delicious chocolate birthday cooking trade skill profession",
	["inv_misc_clothscrap_01"] = "yellow tattered torn ripped cape drape robe banner flag curtain clothes",
	["inv_misc_clothscrap_02"] = "tattered torn ripped cape drape robe banner flag curtain clothes",
	["inv_misc_clothscrap_03"] = "blue tattered torn ripped cape drape robe banner flag curtain clothes",
	["inv_misc_clothscrap_04"] = "purple tattered torn ripped cape drape robe banner flag curtain clothes",
	["inv_misc_clothscrap_05"] = "green tattered torn ripped cape drape robe banner flag curtain clothes",
	["inv_misc_coin_01"] = "pieces golden money coins change currency cash",
	["inv_misc_coin_02"] = "pieces golden money coins change currency cash stacks stacked",
	["inv_misc_coin_03"] = "silver pieces money coins change currency cash",
	["inv_misc_coin_04"] = "silver pieces money coins change currency cash stacks stacked",
	["inv_misc_coin_05"] = "copper pieces money coins change currency cash",
	["inv_misc_coin_06"] = "copper pieces money coins change currency cash stacks stacked",
	["inv_misc_coin_07"] = "green gurubashi coin troll zul'gurub fortune card ticket slip paper",
	["inv_misc_coin_08"] = "zandalar honor token purple troll zul'gurub",
	["inv_misc_coin_09"] = "red white cunning of the cruel skull hakkari coin troll zul'gurub",
	["inv_misc_coin_10"] = "blue silver steel metal gray grey razzashi coin troll zul'gurub blade",
	["inv_misc_coin_11"] = "red bloodscalp coin troll zul'gurub diamond square shape",
	["inv_misc_coin_12"] = "silver gray grey white green skullsplitter coin troll zul'gurub bone",
	["inv_misc_crystalepic"] = "purple blue maelstrom crystal shard fragment disenchanting glowing magic professions:enchanting",
	["inv_misc_crystalepic2"] = "green crystal shard fragment disenchanting glowing magic professions:enchanting",
	["inv_misc_cutgemnormal"] = "pink purple gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal2"] = "purple gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal2a"] = "pink purple nightstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal3"] = "yellow orange gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal3a"] = "orange hessonite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal4"] = "light blue cyan turquoise gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal4a"] = "blue zephyrite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal5"] = "orange gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal5a"] = "green jasper gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormal6a"] = "yellow alicite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemnormala"] = "red carnelian gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemperfect"] = "orange hessonite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemperfect2"] = "purple pink nightstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemperfect3"] = "blue zephyrite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemperfect4"] = "red carnelian gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemperfect5"] = "green jasper gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemperfect6"] = "orange yellow alicite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut perfect",
	["inv_misc_cutgemsuperior"] = "yellow amberjewel gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemsuperior2"] = "blue ocean sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemsuperior3"] = "purple demonseye gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemsuperior4"] = "orange ember topaz gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemsuperior5"] = "green dream emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_cutgemsuperior6"] = "red inferno ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_darkphoenixpet_01"] = "purple dark phoenix hatchling pet bird winged feathered feathers animal creature wings",
	["inv_misc_deepseascales"] = "passive:nether_plating(warlock)",
	["inv_misc_dragonkite_01"] = "blue red dragon kite",
	["inv_misc_dragonkite_02"] = "green blue dragon kite",
	["inv_misc_dragonkite_03"] = "purple black blue dragon kite",
	["inv_misc_dragonkite_04"] = "blue dragon kite",
	["inv_misc_driedneedlemushrooms"] = "brown needle mushrooms food fungi fungus cooking trade skill profession",
	["inv_misc_driedpeaches"] = "dried peaches orange yellow cooking trade skill profession",
	["inv_misc_dust_01"] = "yellow golden arcane powder dust sand birdseed piled mound",
	["inv_misc_emberweavecloth_01"] = "brown bolt of embersilk cloth tailoring fabric rolled",
	["inv_misc_emberweavecloth_02"] = "red glowing brown bolt of embersilk cloth tailoring fabric rolled",
	["inv_misc_emberweaveclothbolt_01"] = "brown embersilk cloth fabric tailoring wavy",
	["inv_misc_emberweaveclothbolt_02"] = "red glowing brown embersilk cloth fabric tailoring wavy",
	["inv_misc_endlesspotion"] = "purple endless unlimited flask of enhancement potion elixir bottle jar drink spilling spilt leaking dropped glass jug liquid",
	["inv_misc_epicgem_01"] = "red queens queen's garnet gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_02"] = "blue deepholm iolite gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_03"] = "yellow lightstone gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_04"] = "orange lava coral gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_05"] = "purple shadow spinel gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_06"] = "green elven peridot gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_epicgem_uncut_01"] = "red queens queen's garnet gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_epicgem_uncut_02"] = "blue deepholm iolite gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_epicgem_uncut_03"] = "yellow lightstone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_epicgem_uncut_04"] = "orange lava coral gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_epicgem_uncut_05"] = "purple shadow spinel gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_epicgem_uncut_06"] = "green elven peridot gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_fish_14"] = "companion:magical_crawdad",
	["inv_misc_fish_15"] = "orange red pink shrimp fish seafood crustacean critter animal creature",
	["inv_misc_fish_78"] = "red orange blue lavascale catfish ocean seafood cooking trade skill profession head",
	["inv_misc_fish_79"] = "blue deepsea sagefish ocean seafood cooking trade skill profession head",
	["inv_misc_fish_80"] = "purple blue fathom eel fish ocean seafood cooking trade skill profession teeth snake head",
	["inv_misc_fish_81"] = "blue albino cavefish ocean seafood cooking trade skill profession head",
	["inv_misc_fish_82"] = "green fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_83"] = "pink sharptooth fish ocean seafood cooking trade skill profession head",
	["inv_misc_fish_84"] = "blue algaefin rockfish head ocean seafood cooking trade skill profession",
	["inv_misc_fish_85"] = "orange blue blackbelly mudfish ocean seafood cooking trade skill profession",
	["inv_misc_fish_89"] = "red orange blue murglesnout fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_90"] = "mountain trout green fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_91"] = "giant furious pike orange fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_92"] = "jewel danio orange fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_93"] = "krasarang paddlefish green blue turquoise ocean seafood cooking trade skill profession",
	["inv_misc_fish_94"] = "redbelly mandarin blue fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_95"] = "jade lungfish green ocean seafood cooking trade skill profession",
	["inv_misc_fish_96"] = "white spinefish skeletal bones skeleton ocean seafood cooking trade skill profession",
	["inv_misc_fish_98"] = "emperor salmon gray grey slick fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_99"] = "tiger gourami orange black stripes striped fish ocean seafood cooking trade skill profession",
	["inv_misc_fish_turtle_01"] = "green turtle tortoise head animal creature",
	["inv_misc_fish_turtle_02"] = "blue tortoise head animal creature mount:sea_turtle",
	["inv_misc_fish_turtle_03"] = "green turtle tortoise head animal creature",
	["inv_misc_flaskofvolatility"] = "silver filigree flask purple potion elixir bottle jar drink glass alchemy trade skill profession",
	["inv_misc_flower_01"] = "racial_passive:cultivation(tauren)",
	["inv_misc_flower_04"] = "companion:crimson_lasher",
	["inv_misc_food_15"] = "professions:cooking",
	["inv_misc_food_157_fish_80"] = "brown lightly fried lurker fish ocean seafood plated",
	["inv_misc_food_158_fish_81"] = "blue delicious sagefish tail ocean seafood sushi",
	["inv_misc_food_158_fish_redish"] = "red fish ocean seafood sushi",
	["inv_misc_food_159_fish_82"] = "red green broiled mountain trout soup stew fish ocean seafood",
	["inv_misc_food_160_fish_87"] = "blue severed sagefish head ocean seafood",
	["inv_misc_food_161_fish_89"] = "red blackbelly sushi fish ocean seafood rice",
	["inv_misc_food_161_fish_white"] = "white sushi fish ocean seafood rice",
	["inv_misc_food_165_bambooshoot01"] = "small bamboo shoot food",
	["inv_misc_food_167_sugarcane"] = "small sugarcane stalk bamboo food",
	["inv_misc_food_168_ricecake01"] = "pounded rice cakes green red cooking trade skill profession",
	["inv_misc_food_26"] = "red aquadynamic attractor lollipop candy lure bait tackle fishing trade skill profession",
	["inv_misc_food_54"] = "companion:spring_rabbit",
	["inv_misc_food_59"] = "companion:sinister_squashling",
	["inv_misc_food_cooked_greatpabanquet_brew"] = "great banquet the brew food cooking trade skill profession",
	["inv_misc_food_cooked_greatpabanquet_general"] = "great pandaren banquet food cooking trade skill profession sauce",
	["inv_misc_food_cooked_greatpabanquet_grill"] = "banquet the food cooking trade skill profession grilled meats fish",
	["inv_misc_food_cooked_greatpabanquet_oven"] = "great banquet the oven roast turkey food cooking trade skill profession",
	["inv_misc_food_cooked_greatpabanquet_pot"] = "great banquet the pot food cooking trade skill profession",
	["inv_misc_food_cooked_greatpabanquet_steamer"] = "great banquet the steamer food cooking trade skill profession baskets containers",
	["inv_misc_food_cooked_greatpabanquet_wok"] = "great banquet the wok food cooking trade skill profession",
	["inv_misc_food_cooked_jadewitchbrew"] = "serpent brew root beer mead booze ale alcohol drink liquid food cooking trade skill profession jug jar spilling pouring",
	["inv_misc_food_cooked_madbrewbreakfast"] = "breakfast brew root beer mead booze ale alcohol drink liquid food cooking trade skill profession mug cup",
	["inv_misc_food_cooked_mogufishstew"] = "mogu fish stew bowl food cooking trade skill profession",
	["inv_misc_food_cooked_pabanquet_brew"] = "banquet the brew food cooking trade skill profession",
	["inv_misc_food_cooked_pabanquet_general"] = "pandaren banquet food cooking trade skill profession sushi rolls sauce",
	["inv_misc_food_cooked_pabanquet_grill"] = "great banquet the food cooking trade skill profession grilled meats fish",
	["inv_misc_food_cooked_pabanquet_oven"] = "banquet the oven roast turkey food cooking trade skill profession",
	["inv_misc_food_cooked_pabanquet_pot"] = "banquet the pot food cooking trade skill profession",
	["inv_misc_food_cooked_pabanquet_steamer"] = "banquet the steamer food cooking trade skill profession baskets containers",
	["inv_misc_food_cooked_pabanquet_wok"] = "banquet the wok food cooking trade skill profession",
	["inv_misc_food_meat_raw_02"] = "gray grey red blood shrimp fish seafood crustacean critter animal creature",
	["inv_misc_gear_01"] = "racial_passive:engineering_specialization(gnome)",
	["inv_misc_gem_02"] = "professions:jewelcrafting",
	["inv_misc_gem_bloodgem_01"] = "professions:prospecting",
	["inv_misc_gem_pearl_01"] = "black pearl marble shiny clam sea ocean sphere ball",
	["inv_misc_gem_pearl_02"] = "white pearl shiny clam sea ocean sphere ball light radiant radiance",
	["inv_misc_gem_pearl_03"] = "white pearl shiny clam sea ocean sphere ball light radiant radiance",
	["inv_misc_gem_pearl_04"] = "golden yellow pearl radiant radiance ball of light shiny clam sea ocean sphere",
	["inv_misc_gem_pearl_05"] = "pink purple red pearl radiant radiance ball of light shiny clam sea ocean sphere",
	["inv_misc_gem_pearl_06"] = "light blue cyan turquoise pearl radiant radiance ball of shiny clam sea ocean sphere",
	["inv_misc_gem_pearl_07"] = "black pearl radiant radiance blue ball of light planet marble shiny clam sea ocean sphere",
	["inv_misc_gem_sapphire_01"] = "spell:mastery:_frozen_heart(death_knight)",
	["inv_misc_gem_sapphire_02"] = "spell:conjure_mana_gem(mage)",
	["inv_misc_gem_x4_metagem_cut"] = "primal diamond blue clear white gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_metagem_uncut"] = "primal diamond blue clear white gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_cut_blue"] = "river's heart blue gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_cut_green"] = "wild jade green gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_cut_orange"] = "vermilion onyx orange gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_cut_purple"] = "imperial amethyst purple violet gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_cut_red"] = "primordial ruby red gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_cut_yellow"] = "sun's radiance yellow gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_rare_uncut_blue"] = "river's heart blue gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_uncut_green"] = "wild jade green gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_uncut_orange"] = "vermilion onyx orange gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_uncut_purple"] = "imperial amethyst purple violet gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_uncut_red"] = "primordial ruby red gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_rare_uncut_yellow"] = "sun's radiance yellow gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_gem_x4_uncommon_cut_blue"] = "lapis lazuli blue gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_cut_green"] = "adventurine green gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_cut_orange"] = "sardonyx orange gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_cut_purple"] = "zyanite purple violet gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_cut_red"] = "pandarian garnet red gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_cut_yellow"] = "sunstone yellow gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_blue"] = "lapis lazuli blue gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_green"] = "adventurine green gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_orange"] = "sardonyx orange gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_purple"] = "zyanite purple violet gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_red"] = "pandarian garnet red gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_gem_x4_uncommon_perfectcut_yellow"] = "sunstone yellow gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_greateressence"] = "pink purple greater celestial essence disenchanting glowing light radiant radiance magic sparks",
	["inv_misc_head_clockworkgnome_01"] = "companion:clockwork_gnome",
	["inv_misc_head_dragon_01"] = "spell:dragon's_breath(mage)",
	["inv_misc_head_tiger_01"] = "tiger animal feline cat kitty furry fuzzy beast creature head mount striped stripes white fangs teeth",
	["inv_misc_herb_16"] = "spell:wound_poison(rogue)",
	["inv_misc_herb_azsharasveil"] = "azshara's veil",
	["inv_misc_herb_felblossom"] = "spell:lifebloom(druid)",
	["inv_misc_horn_02"] = "spell:horn_of_winter(death_knight)",
	["inv_misc_horn_04"] = "talent:double_time(warrior)",
	["inv_misc_idol_02"] = "racial_passive:da_voodoo_shuffle(troll)",
	["inv_misc_key_03"] = "mount:goblin_trike",
	["inv_misc_key_04"] = "mount:goblin_turbo-trike",
	["inv_misc_lesseressence"] = "light blue cyan lesser celestial essence disenchanting glowing radiant radiance magic sparks",
	["inv_misc_map08"] = "racial_passive:explorer(dwarf)",
	["inv_misc_metagem_b"] = "light blue white radiant radiance shadowspirit diamond gemstone crystal shiny sparkling jewelcrafting trade skill profession cut",
	["inv_misc_metagemuncut_b"] = "light blue white radiant radiance shadowspirit diamond gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_monsterclaw_03"] = "spell:swipe(druid) spell:frenzy(hunter)",
	["inv_misc_monsterclaw_05"] = "spell:mastery:_razor_claws(druid)",
	["inv_misc_note_02"] = "racial_passive:diplomacy(human)",
	["inv_misc_note_05"] = "racial_passive:languages(blood_elf) racial_passive:languages(tauren) racial_passive:languages(worgen) racial_passive:languages(human) racial_passive:languages(goblin) racial_passive:languages(night_elf) racial_passive:languages(dwarf) racial_passive:languages(gnome) racial_passive:languages(draenei) racial_passive:languages(troll) racial_passive:languages(orc) racial_passive:languages(undead)",
	["inv_misc_orb_04"] = "mount:corrupted_fire_hawk",
	["inv_misc_organ_01"] = "professions:master_of_anatomy",
	["inv_misc_pelt_bear_03"] = "spell:thick_hide(druid)",
	["inv_misc_pelt_bear_ruin_02"] = "racial_passive:beast_slaying(troll)",
	["inv_misc_pelt_wolf_01"] = "professions:skinning",
	["inv_misc_penguinpet"] = "companion:mr._chilly",
	["inv_misc_petdragonwhelptarecgosa"] = "companion:lil'_tarecgosa",
	["inv_misc_pheonixpet_01"] = "pet_command:heart_of_the_phoenix(hunter)",
	["inv_misc_pyriumore"] = "silver gray grey elementium ore rock nugget mining trade skill profession stone boulder",
	["inv_misc_qirajicrystal_01"] = "mount:yellow_qiraji_battle_tank",
	["inv_misc_qirajicrystal_03"] = "mount:green_qiraji_battle_tank",
	["inv_misc_qirajicrystal_04"] = "mount:blue_qiraji_battle_tank",
	["inv_misc_questionmark"] = "question mark unknown",
	["inv_misc_rune_01"] = "gray grey silver blue rock hearthstone rune swirl teleportation home magic",
	["inv_misc_rune_06"] = "gray grey golden yellow stone silver rune of teleportation portals tablet magic",
	["inv_misc_rune_08"] = "brown green rune of portals teleportation earth rock magic",
	["inv_misc_sandbox_spectraltiger_01"] = "tiger animal cat kitty feline furry fuzzy beast creature spectral ghost spirit apparition undead mount striped stripes",
	["inv_misc_seagullpet_01"] = "companion:rustberg_gull",
	["inv_misc_shadowegg"] = "talent:purgatory(death_knight)",
	["inv_misc_shovel_01"] = "professions:survey",
	["inv_misc_slime_01"] = "companion:toxic_wasteling",
	["inv_misc_stonedragonorange"] = "mount:sandstone_drake",
	["inv_misc_stormdragongreen"] = "mount:drake_of_the_west_wind",
	["inv_misc_uncutgemnormal"] = "orange ember topaz gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemnormal1"] = "green dream emerald gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemnormal2"] = "blue ocean sapphire gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemnormal3"] = "red inferno ruby gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemnormal4"] = "purple demonseye gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemnormal5"] = "yellow amberjewel gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut",
	["inv_misc_uncutgemsuperior"] = "yellow alicite gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_uncutgemsuperior2"] = "green jasper gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_uncutgemsuperior3"] = "purple nightstone gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_uncutgemsuperior4"] = "red carnelian gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_uncutgemsuperior5"] = "orange hessonite gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_uncutgemsuperior6"] = "blue zephyrite gemstone crystal shiny sparkling jewelcrafting trade skill profession uncut shard fragment",
	["inv_misc_volatilefire"] = "passive:imp:_cauterize_master(warlock)",
	["inv_misc_web_01"] = "talent:narrow_escape(hunter)",
	["inv_mount_darkphoenixa"] = "mount:dark_phoenix",
	["inv_mount_hordescorpiong"] = "mount:kor'kron_annihilator",
	["inv_mount_spectralwolf"] = "mount:spectral_wolf",
	["inv_musket_03"] = "racial_passive:crack_shot(dwarf)",
	["inv_offhand_1h_firelandsraid_d_01"] = "passive:pyroclasm(warlock)",
	["inv_ore_arcanite_01"] = "rainbow red yellow green blue pyrium pyrite ore rock mining trade skill profession stone boulder nugget",
	["inv_pandarenserpentgodmount_black"] = "reins of the heavenly onyx cloud serpent black purple blue mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentgodmount_red"] = "reins of the heavenly crimson cloud serpent red mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount"] = "red blue mount cloud serpent dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_blue"] = "reins of the azure cloud serpent blue purple yellow mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_green"] = "reins of the jade cloud serpent green red mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_lightning"] = "reins of the thundering ruby cloud serpent red mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_lightning_green"] = "reins of the thundering jade cloud serpent green mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_lightning_yellow"] = "reins of the thundering august cloud serpent yellow mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentmount_yellow"] = "reins of the golden cloud serpent yellow mount dragon beast animal creature scaled scales scaly reptile head",
	["inv_pandarenserpentpet"] = "pet cloud serpent dragon beast animal creature scaled scales scaly reptile young baby horns horned",
	["inv_pet_babyblizzardbear"] = "companion:baby_blizzard_bear",
	["inv_pet_babycrocodile_snarly"] = "companion:snarly",
	["inv_pet_babypengu"] = "companion:pengu",
	["inv_pet_battlepettraining"] = "passive:battle_pet_training(mage) passive:battle_pet_training(death_knight) passive:battle_pet_training(druid) passive:battle_pet_training(warlock) passive:battle_pet_training(rogue) passive:battle_pet_training(hunter) passive:battle_pet_training(paladin) passive:battle_pet_training(warrior) passive:battle_pet_training(priest) passive:battle_pet_training(shaman)",
	["inv_pet_cats_calicocat"] = "companion:calico_cat",
	["inv_pet_cockroach"] = "companion:undercity_cockroach",
	["inv_pet_egbert"] = "companion:egbert",
	["inv_pet_scorchedstone"] = "companion:searing_scorchling",
	["inv_pet_speedy"] = "companion:speedy",
	["inv_potion_33"] = "companion:spirit_of_summer",
	["inv_potiond_1"] = "red yellow golemblood potion golden flask elixir bottle jar drink glass poison",
	["inv_potiond_2"] = "blue elixir of the master yellow golden flask potion bottle jar drink glass poison mana",
	["inv_potiond_3"] = "red volcanic potion yellow golden flask elixir bottle jar drink glass poison health healing",
	["inv_potiond_4"] = "green potion of the tol'vir yellow golden flask elixir bottle jar drink glass poison",
	["inv_potiond_5"] = "purple ghost elixir yellow golden flask potion bottle jar drink glass poison rejuvenation",
	["inv_potiond_6"] = "pale green cyan turquoise mysterious potion yellow golden flask elixir bottle jar drink glass poison",
	["inv_potione_1"] = "purple flask of steelskin rejuvenation yellow golden potion elixir bottle jar drink glass sphere globe",
	["inv_potione_2"] = "green flask of the winds yellow golden potion elixir bottle jar drink glass sphere globe poison",
	["inv_potione_3"] = "pink purple yellow golden flask potion elixir bottle jar drink glass sphere globe rejuvenation",
	["inv_potione_4"] = "light blue cyan flask of flowing water yellow golden potion elixir bottle jar drink glass sphere globe mana",
	["inv_potione_5"] = "green flask of the draconic mind yellow golden potion elixir bottle jar drink glass sphere globe poison",
	["inv_potione_6"] = "red orange flask of titanic strength yellow golden potion elixir bottle jar drink glass sphere globe health healing",
	["inv_potionf_1"] = "red orange deepstone oil flask potion elixir bottle jar drink glass poison health healing",
	["inv_potionf_2"] = "purple pink elixir of mighty speed flask potion bottle jar drink glass poison",
	["inv_potionf_3"] = "green deathblood venom flask potion elixir bottle jar drink glass poison",
	["inv_potionf_4"] = "light blue cyan prismatic elixir flask potion bottle jar drink glass poison mana",
	["inv_potionf_5"] = "purple pink elixir of deep earth flask potion bottle jar drink glass poison rejuvenation",
	["inv_potionf_6"] = "yellow elixir of impossible accuracy flask potion bottle jar drink glass poison",
	["inv_qirajidol_sun"] = "spell:daybreak(paladin)",
	["inv_relics_idolofrejuvenation"] = "spell:swiftmend(druid)",
	["inv_scroll_11"] = "passive:flight_master's_license(mage) passive:flight_master's_license(death_knight) passive:flight_master's_license(druid) passive:flight_master's_license(warlock) passive:flight_master's_license(rogue) passive:flight_master's_license(hunter) passive:flight_master's_license(paladin) passive:flight_master's_license(warrior) passive:flight_master's_license(priest) passive:flight_master's_license(shaman)",
	["inv_shield_05"] = "spell:shield_slam(warrior)",
	["inv_shield_07"] = "spell:shield_barrier(warrior)",
	["inv_shield_26"] = "companion:festival_lantern",
	["inv_spear_02"] = "talent:wyvern_sting(hunter)",
	["inv_spear_04"] = "spell:healing_stream_totem(shaman)",
	["inv_spear_07"] = "spell:aimed_shot(hunter)",
	["inv_staff_08"] = "spell:auto_attack(mage)",
	["inv_staff_10"] = "passive:weapon_skills(mage)",
	["inv_staff_21"] = "spell:weapon_skills(druid) spell:auto_attack(hunter) spell:auto_attack(priest)",
	["inv_staff_90"] = "spell:nature's_focus(druid)",
	["inv_sword_06"] = "spell:auto_attack(warrior)",
	["inv_sword_07"] = "spell:dancing_rune_weapon(death_knight)",
	["inv_sword_11"] = "spell:devastate(warrior)",
	["inv_sword_120"] = "spell:might_of_the_frozen_wastes(death_knight)",
	["inv_sword_122"] = "spell:killing_machine(death_knight)",
	["inv_sword_136"] = "spell:sanctity_of_battle(paladin)",
	["inv_sword_17"] = "spell:auto_attack(rogue)",
	["inv_sword_27"] = "spell:weapon_skills(warrior)",
	["inv_sword_2h_cataclysm_b_02"] = "spell:auto_attack(death_knight)",
	["inv_sword_48"] = "spell:execute(warrior)",
	["inv_sword_61"] = "spell:festering_strike(death_knight)",
	["inv_sword_62"] = "spell:empower_rune_weapon(death_knight)",
	["inv_sword_97"] = "spell:revealing_strike(rogue)",
	["inv_thanksgiving_turkey"] = "companion:plump_turkey",
	["inv_throwingknife_02"] = "passive:weapon_skills(rogue)",
	["inv_throwingknife_04"] = "spell:shiv(rogue)",
	["inv_throwingknife_06"] = "talent:deadly_throw(rogue)",
	["inv_wand_11"] = "passive:weapon_skills(warlock)",
	["inv_wand_1h_firelandsraid_d_02"] = "passive:decimation(warlock)",
	["inv_weapon_bow_02"] = "passive:weapon_skills(hunter)",
	["inv_weapon_bow_12"] = "racial_passive:dead_eye(troll)",
	["inv_weapon_halberd_23"] = "spell:auto_attack(druid)",
	["inv_weapon_shortblade_05"] = "racial_passive:shortblade_specialization(gnome)",
	["inv_weapon_shortblade_15"] = "spell:mastery:_main_gauche(rogue)",
	["inv_weapon_shortblade_38"] = "spell:combat_potency(rogue)",
	["inv_weapon_shortblade_40"] = "spell:blood_strike(death_knight) spell:heart_strike(death_knight)",
	["inv_weapon_shortblade_79"] = "spell:blood_of_the_north(death_knight)",
	["spell_arcane_arcane01"] = "whirlwind power magic swirly winds windy gusts gusty snowflakes icy ice cold frozen freezing frosty shiny sparkles light blue wizard spell:arcane_charge(mage) spell:celestial_focus(druid) spell:soulshatter(warlock)",
	["spell_arcane_arcane02"] = "magic power arcane flows flowing river stream shiny sparkles light blue wizard spell:spellsteal(mage)",
	["spell_arcane_arcane03"] = "purple shiny sparkles magic power lightbeams arcane blast burst ray wizard mage spell:starsurge(druid)",
	["spell_arcane_arcane04"] = "blast burst power magic arcane purple explosion point spot star implosion supernova wizard mage spell",
	["spell_arcane_arcanepotency"] = "wizard mage power conjuring arcane ball blue purple hands holding magic spell",
	["spell_arcane_arcaneresilience"] = "blue purple magical shield barrier armor defense protection spell ward riveted rivets ethereal wizard's mage's",
	["spell_arcane_arcanetactics"] = "purple spears people glowing magical enchanted enchantment",
	["spell_arcane_arcanetorrent"] = "wizard mage arcane magic power blast burst falling stars shooting comet meteors cosmos cosmic purple spell sky",
	["spell_arcane_blast"] = "wizard burst explosion outward circle magic purple blue spell:arcane_blast(mage)",
	["spell_arcane_blink"] = "shiny eye magic wizard purple shining tear teleport displacement head spell:blink(mage) spell:distracting_shot(hunter)",
	["spell_arcane_focusedpower"] = "mind controlled telepathy telepathic anguish anger angry migraine headache tormented blue monk bald hands magic spell power",
	["spell_arcane_invocation"] = "arcane blast burst powerful beam blue purple circles light mage magic spell",
	["spell_arcane_manatap"] = "hand touch light blue cyan power magic wizard absorption absorb spell:mastery:_mana_adept(mage)",
	["spell_arcane_massdispel"] = "circles swirly swirls magic sparkly sparkles purple wizard spell:conjure_refreshment_table(mage) spell:mass_dispel(priest)",
	["spell_arcane_mindmastery"] = "pink mind control telepathy telepathic power spell magic anguish pain migraine headache torment",
	["spell_arcane_portaldalaran"] = "gateway major city town circle twisting nether swirly swirls magic teleportation purple blue northrend spell:portal:_dalaran(mage) spell:ancient_portal:_dalaran(mage)",
	["spell_arcane_portaldarnassus"] = "portal gateway major city town circle twisting nether swirly swirls mage spell magic teleportation purple kalimdor",
	["spell_arcane_portalexodar"] = "portal gateway major city town circle twisting nether swirly swirls mage spell magic teleportation purple azuremyst isle kalimdor",
	["spell_arcane_portalironforge"] = "portal gateway major city town circle twisting nether swirly swirls mage spell magic teleportation light blue cyan dun morogh eastern kingdoms",
	["spell_arcane_portalorgrimmar"] = "gateway major city town circle twisting nether swirly swirls magic teleportation orange durotar kalimdor spell:portal:_orgrimmar(mage)",
	["spell_arcane_portalshattrath"] = "gateway major city town circle twisting nether swirly swirls magic teleportation blue-green cyan terokkar forest outland spell:portal:_shattrath(mage)",
	["spell_arcane_portalsilvermoon"] = "gateway major city town circle twisting nether swirly swirls magic teleportation red eversong woods eastern kingdoms spell:portal:_silvermoon(mage)",
	["spell_arcane_portalstonard"] = "gateway major city town circle twisting nether swirly swirls magic teleportation light green swamp of sorrows eastern kingdoms spell:portal:_stonard(mage)",
	["spell_arcane_portalstormwind"] = "portal gateway major city town circle twisting nether swirly swirls mage spell magic teleportation blue elwynn forest eastern kingdoms",
	["spell_arcane_portaltheramore"] = "portal gateway major city town circle twisting nether swirly swirls mage spell magic teleportation yellow dustwallow marsh kalimdor",
	["spell_arcane_portalthunderbluff"] = "gateway major city town circle twisting nether swirly swirls magic teleportation cyan light blue mulgore kalimdor spell:portal:_thunder_bluff(mage)",
	["spell_arcane_portaltolbarad"] = "gateway major city town circle twisting nether swirly swirls magic teleportation red eastern kingdoms spell:portal:_tol_barad(mage)",
	["spell_arcane_portalundercity"] = "gateway major town circle twisting nether swirly swirls magic teleportation green ruins of lordaeron tirisfal glades eastern kingdoms spell:portal:_undercity(mage)",
	["spell_arcane_prismaticcloak"] = "shadowy unholy cloaked hooded figure purple outlined silhouette glowing eyes evil magic spell power spirit phantom ghost apparition",
	["spell_arcane_rune"] = "purple arcane magic rune of runic power symbol mage blast burst explosion letter",
	["spell_arcane_starfire"] = "blue purple rain falling stars blazing burning fiery hot heat comets meteors sky bolts missiles nukes magic spells power spell:starfire(druid)",
	["spell_arcane_studentofmagic"] = "magic focus blue red hands holding mage wizard power spell stars arcane",
	["spell_arcane_teleportdalaran"] = "portal gateway major city town twisting nether magic teleportation purple blue northrend spell:teleport:_dalaran(mage) spell:ancient_teleport:_dalaran(mage)",
	["spell_arcane_teleportdalarancrater"] = "purple dalaran crater portal gateway major city town twisting nether mage spell magic teleportation",
	["spell_arcane_teleportdarnassus"] = "portal gateway major city town twisting nether mage spell magic teleportation purple kalimdor",
	["spell_arcane_teleportexodar"] = "portal gateway major city town twisting nether mage spell magic teleportation purple blue azuremyst isle kalimdor",
	["spell_arcane_teleportironforge"] = "portal gateway major city town twisting nether mage spell magic teleportation blue icy snowy frozen ice cold freezing frosty dun morogh eastern kingdoms",
	["spell_arcane_teleportmoonglade"] = "portal gateway twisting nether magic teleportation purple structure moonwell kalimdor spell:teleport:_moonglade(druid)",
	["spell_arcane_teleportorgrimmar"] = "portal gateway major city town twisting nether magic teleportation orange durotar kalimdor spell:teleport:_orgrimmar(mage)",
	["spell_arcane_teleportshattrath"] = "portal gateway major city town twisting nether magic teleportation light blue green cyan terokkar forest outland spell:teleport:_shattrath(mage)",
	["spell_arcane_teleportsilvermoon"] = "portal gateway major city town twisting nether magic teleportation red eversong woods eastern kingdoms spell:teleport:_silvermoon(mage)",
	["spell_arcane_teleportstonard"] = "portal gateway major city town twisting nether magic teleportation green forest swamp of sorrows eastern kingdoms spell:teleport:_stonard(mage)",
	["spell_arcane_teleportstormwind"] = "portal gateway major city town twisting nether mage spell magic teleportation blue green elwynn forest eastern kingdoms",
	["spell_arcane_teleporttheramore"] = "portal gateway major city town twisting nether mage spell magic teleportation orange blue dustwallow marsh kalimdor",
	["spell_arcane_teleportthunderbluff"] = "portal gateway major city town twisting nether magic teleportation light blue cyan mulgore kalimdor spell:teleport:_thunder_bluff(mage)",
	["spell_arcane_teleporttolbarad"] = "portal gateway major city town twisting nether magic teleportation red eastern kingdoms spell:teleport:_tol_barad(mage)",
	["spell_arcane_teleportundercity"] = "portal gateway major town twisting nether magic teleportation green purple ruins of lordaeron tirisfal glades eastern kingdoms spell:teleport:_undercity(mage) spell:death_gate(death_knight)",
	["spell_brew_bolt_dark"] = "brown chocolate rust poison toxic disease liquid blob goo snot mucus spit saliva",
	["spell_brew_bolt_medium"] = "orange poison toxic disease liquid blob goo snot mucus spit saliva",
	["spell_brew_bolt_wheat"] = "yellow wheat root beer mead booze ale alcohol drink liquid poison toxic disease blob goo snot mucus spit saliva",
	["spell_brew_dark"] = "brown chocolate bubbling brew cauldron pool liquid water bubbles boiling",
	["spell_brew_medium"] = "orange bubbling brew cauldron pool liquid water bubbles boiling",
	["spell_brew_wheat"] = "yellow wheat root beer mead booze ale alcohol drink liquid bubbling brew cauldron pool water bubbles boiling",
	["spell_brokenheart"] = "broken heartbreak ripped torn apart tear cracked destroyed red yellow pink",
	["spell_burningsoul"] = "yellow orange man on fire fiery hot heat flames flaming blazing scorching searing outline silhouette dust magic spell passive:burning_soul(mage)",
	["spell_chargenegative"] = "red pink minus - negative charged lightning thunder zap electricity thaddius crackle crackling",
	["spell_chargepositive"] = "blue cyan plus + positive charged lightning thunder zap electricity thaddius crackle crackling",
	["spell_deathknight_antimagiczone"] = "lightning thunder zapping deflection reflection spell bubble shield defense defend protection barrier blue crackle crackling striking hitting electricity natural power talent:anti-magic_zone(death_knight)",
	["spell_deathknight_armyofthedead"] = "purple zombies undead demons shadow unholy magic necromancy ghouls reanimation raise raising evil scourge corpses spell:army_of_the_dead(death_knight)",
	["spell_deathknight_bladedarmor"] = "glowing enchanted enchantment helmet armor defense protection head gear plate steel metal red orange blades",
	["spell_deathknight_bloodboil"] = "liquid lava boiling steaming hot heated boiled bubbles red magic surface spell:blood_boil(death_knight) spell:crimson_scourge(death_knight)",
	["spell_deathknight_bloodplague"] = "pestilence purple bloody bleeding shadow unholy magic hand arm passive:blood_plague(death_knight)",
	["spell_deathknight_bloodpresence"] = "skull teeth undead undeath demonic creature evil glowing eyes necromancy magic power head red bloody scourge corpse tab:blood(death_knight) spell:blood_presence(death_knight) spell:improved_blood_presence(death_knight)",
	["spell_deathknight_bloodtap"] = "hand holding clutching heart red draining magic power spell:blood_rites(death_knight)",
	["spell_deathknight_butcher"] = "silver battleaxe blade weapon steel metal bloody wounding striking moving motion",
	["spell_deathknight_butcher2"] = "silver battleaxe blade weapon steel metal bloody wounding striking moving motion spell:death_strike(death_knight)",
	["spell_deathknight_classicon"] = "silver steel metal battleaxe blade weapon elegant skull emblem glowing blue spell:obliterate(death_knight)",
	["spell_deathknight_darkconviction"] = "swirly circles enchanted enchantment sword blade weapon dagger knife white aura spell:rune_strike(death_knight)",
	["spell_deathknight_deathstrike"] = "hand holding swinging steel metal sword blade weapon glowing enchanted enchantment gemmed plated armor glove attacking red spell:mastery:_blood_shield(death_knight)",
	["spell_deathknight_empowerruneblade"] = "glowing magical battleaxe blade weapon skull emblem edge lightning thunder powerful enchanted enchantment green spell:plague_strike(death_knight)",
	["spell_deathknight_empowerruneblade2"] = "glowing magical battleaxe blade weapon skull emblem edge lightning thunder powerful enchanted enchantment blue spell:frost_strike(death_knight)",
	["spell_deathknight_explode_ghoul"] = "exploding ghoul flying eyeball thrown exploded green goo liquid bursting gross",
	["spell_deathknight_frostfever"] = "snowflake forehead man ice icy cold frozen freezing frosty blue lightning thunder power magic passive:frost_fever(death_knight)",
	["spell_deathknight_frostpresence"] = "skull teeth undead undeath demonic creature evil glowing eyes necromancy magic power head blue icy ice cold frozen freezing frosty scourge corpse spell:frost_presence(death_knight) tab:frost(death_knight) spell:improved_frost_presence(death_knight)",
	["spell_deathknight_frozenruneweapon"] = "silver glowing blue sword blade weapon elegant dagger knife ice icy cold frozen freezing frosty aura enchanted enchantment steel metal spell:runeforging(death_knight)",
	["spell_deathknight_gnaw_ghoul"] = "red white bloody teeth mouth gross ghoul undead undeath necromancy gnawing gnashing scourge corpse",
	["spell_deathknight_iceboundfortitude"] = "naked man blue power aura glowing muscles bodybuilding icy cold frozen freezing frosty magic enchanted enchantment spell:icebound_fortitude(death_knight)",
	["spell_deathknight_icetouch"] = "power clenched fist punching ice cold frozen freezing frosty sparkles sparkly magic enchanted enchantment blue knuckles spell:icy_touch(death_knight)",
	["spell_deathknight_icytalons"] = "ice cold frozen freezing frosty icicles bird winged feathered feathers animal creature claws foot blue white spell:icy_talons(death_knight)",
	["spell_deathknight_mindfreeze"] = "blue ice icy cold frozen freezing frosty brain icicles magic power spell:mind_freeze(death_knight)",
	["spell_deathknight_pathoffrost"] = "ice icy cold frozen freezing frosty cracked footprint walking on water blue magic power spell:path_of_frost(death_knight)",
	["spell_deathknight_plaguestrike"] = "hand holding swinging steel metal sword blade weapon glowing enchanted enchantment gemmed plated armor glove attacking green",
	["spell_deathknight_runetap"] = "red steel metal x marks the spot map of power symbol cross spell:rune_tap(death_knight)",
	["spell_deathknight_scourgestrike"] = "hand holding swinging steel metal sword blade weapon glowing enchanted enchantment gemmed plated armor glove attacking purple spell:scourge_strike(death_knight)",
	["spell_deathknight_spelldeflection"] = "silver sword blade weapon steel metal reflecting reflection deflecting deflection magic spell protection defense defensive bounced bouncing",
	["spell_deathknight_strangulate"] = "purple shadowy ghostly apparition hands reaching grabbing spirit demonic creature evil outline silhouette magic power spell:death_grip(death_knight)",
	["spell_deathknight_subversion"] = "silver skull undead undeath demonic creature evil sword blade weapon steel metal scourge corpse face",
	["spell_deathknight_summondeathcharger"] = "horse steed swift glowing blue steel mount:acherus_deathcharger",
	["spell_deathknight_thrash_ghoul"] = "green hand scratching claws fingernails glowing diseased ghoul zombie clawing",
	["spell_deathknight_unholypresence"] = "skull teeth undead undeath demonic creature evil glowing eyes necromancy magic power head green diseased poison toxic scourge corpse spell:unholy_presence(death_knight) tab:unholy(death_knight) spell:improved_unholy_presence(death_knight)",
	["spell_deathknight_vendetta"] = "red silver plate armor steel metal blood liquid",
	["spell_deathvortex"] = "green skull undeath undead demonic creature evil plague poison toxic deadly glowing eyes spiral swirly scourge corpse head spell:outbreak(death_knight)",
	["spell_druid_astralstorm"] = "blue tornado cyclone windy gusty gusts whirlwind swirling spiraling twister twisting vortex",
	["spell_druid_bearhug"] = "brown furry beast animal creature roaring teeth claws anger angry ferocity ferocious fierce spell:bear_hug(druid)",
	["spell_druid_displacement"] = "blue lion cat kitty feline furry fuzzy animal creature talent:displacer_beast(druid)",
	["spell_druid_feralchargecat"] = "red orange brown cat kitty feline furry fuzzy beast animal creature horns glowing eyes anger angry fury furious rage raging aggression roaring charging running snarling head",
	["spell_druid_incarnation"] = "blue enchanted enchantment ghostly ghastly spirit silhouettes ethereal cloud demon posession posessed exorcism",
	["spell_druid_ironbark"] = "brown wooden tree form skin hand fingers spell:ironbark(druid)",
	["spell_druid_massentanglement"] = "green vines thorns weeds entangled entanglement strangulation strangling strangled",
	["spell_druid_mightofursoc"] = "black green brown bear furry beast animal creature silhouette anger angry fierce ferocious ferocity outline spell:might_of_ursoc(druid)",
	["spell_druid_stamedingroar"] = "yellow brown bear furry beast animal creature roaring anger angry fury furious rage raging aggression noisy noise snarling growling head",
	["spell_druid_stampedingroar_cat"] = "wolf canine dog worg animal furry fuzzy creature beast cat kitty feline gray grey yellow red orange snarling growling anger angry fury furious rage raging aggression intimidating intimidation roaring baring teeth head spell:stampeding_roar(druid)",
	["spell_druid_swarm"] = "blue purple violet lights sparkling swarming spirits will-o'-the-wisps",
	["spell_druid_symbiosis"] = "green link connection connected tied silhouettes spell:symbiosis(druid)",
	["spell_druid_thrash"] = "bear furry beast animal creature claws slashing scratching mauling brown paw spell:thrash(druid)",
	["spell_druid_tirelesspursuit"] = "bluegreen cyan turquoise brown wolf canine dog worg animal furry fuzzy creature beast running chasing talent:feline_swiftness(druid)",
	["spell_druid_ursolsvortex"] = "blue circle sphere orb vortex maelstrom",
	["spell_druid_wildburst"] = "gnome leaves breeze windy breezy nature wild",
	["spell_druid_wildcharge"] = "purple blue green yellow stampede beasts animals creatures cat bear moonkin talent:wild_charge(druid)",
	["spell_druid_wildmushroom"] = "mushroom toadstool green white fungus fungi poisonous toxic glowing",
	["spell_druid_wildmushroom_bloom"] = "green mushrooms toadstools fungi fungus glowing",
	["spell_druid_wildmushroom_frenzy"] = "red mushrooms toadstools fungi fungus glowing",
	["spell_festergutgas"] = "green yellow festergut poisonous toxic gas cloud fumes pestilence",
	["spell_fire_bluecano"] = "blue ice icy cold freezing frosty snowy volcano magma lava flow mountain fire fiery burning hot heat flames flaming blazing scorching searing magic spell talent:frozen_power(shaman)",
	["spell_fire_bluefire"] = "blue ice icy cold frozen freezing frosty fire fiery burning hot heat flames flaming blazing scorching searing magic spell conjured candlelight ethereal spirit ghost wisp apparition",
	["spell_fire_bluefirenova"] = "blue ice icy cold frozen freezing frosty circle ball sphere light bubbles magic spell power",
	["spell_fire_bluefireward"] = "blue ice icy cold frozen freezing frosty fire fiery burning hot heat flames flaming blazing scorching searing water magic spell barrier shield armor defense protection light elemental",
	["spell_fire_blueflamebolt"] = "blue ice icy cold frozen freezing frosty fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell power fireball water light",
	["spell_fire_blueflamebreath"] = "blue ice icy cold frozen freezing frosty breath teeth beam light magic spell blast burst fire fiery burning hot heat flames flaming blazing scorching searing",
	["spell_fire_blueflamering"] = "blue ice icy cold frozen freezing frosty nova explosion blast burst circle spell magic wave area fire fiery burning hot heat flames flaming blazing scorching searing",
	["spell_fire_blueflamestrike"] = "blue ice icy cold frozen freezing frosty fire fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell power ball impact blast burst explosion wave area",
	["spell_fire_bluehellfire"] = "blue ice icy cold frozen freezing frosty crack tear rift chasm winds sundered spell magic",
	["spell_fire_blueimmolation"] = "ice icy cold frozen freezing frosty hand undeath undead fire fiery burning hot heat flames flaming blazing scorching searing spell magic ashen charred fingers claws corpse",
	["spell_fire_bluepyroblast"] = "blue ice icy cold frozen freezing frosty fiery burning hot heat flames flaming blazing scorching searing pyroblast fireball bolt missile nuke magic spell power comet meteor falling sky",
	["spell_fire_bluerainoffire"] = "blue ice icy cold frozen freezing frosty fiery burning hot heat flames flaming blazing scorching searing rain falling meteors comets fireballs firebolts sky spell magic missiles nukes",
	["spell_fire_burningspeed"] = "red yellow orange running boots fire fiery burning hot heat flames flaming scorching searing increase magic spell talent:blazing_speed(mage)",
	["spell_fire_burnout"] = "red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing magic spell:pyromaniac(mage) spell:incinerate(warlock)",
	["spell_fire_elemental_totem"] = "red yellow orange fiery burning hot heat flames flaming blazing scorching searing power magic summoned creature demon spell:fire_elemental_totem(shaman)",
	["spell_fire_elementaldevastation"] = "white light holy power destruction bomb explosion blowing up passive:imp:_singe_magic(warlock) spell:ascendance(shaman)",
	["spell_fire_enchantweapon"] = "orange glowing battleaxe blade weapon enchanted enchantment",
	["spell_fire_felcano"] = "green felfire demonic power volcano magma lava flow mountain fiery burning hot heat flames flaming blazing scorching searing magic spell",
	["spell_fire_felfire"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing magic spell conjured candlelight ethereal spirit ghost wisp apparition",
	["spell_fire_felfirenova"] = "green felfire demonic power circle ball sphere light bubbles magic spell:fel_flame(warlock)",
	["spell_fire_felfireward"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing magic spell barrier shield armor defense protection light elemental",
	["spell_fire_felflamebolt"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell fireball water light",
	["spell_fire_felflamebreath"] = "green felfire demonic power breath teeth beam light magic spell blast burst fiery burning hot heat flames flaming blazing scorching searing",
	["spell_fire_felflamering"] = "green felfire nova demonic power explosion blast burst circle magic wave area fiery burning hot heat flames flaming blazing scorching searing spell:chaotic_energy(warlock)",
	["spell_fire_felflamestrike"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell ball impact blast burst explosion wave area",
	["spell_fire_felhellfire"] = "green felfire demonic power crack tear rift chasm winds sundered spell magic",
	["spell_fire_felimmolation"] = "green felfire demonic power hand undeath undead fiery burning hot heat flames flaming blazing scorching searing spell magic ashen charred fingers claws corpse",
	["spell_fire_felpyroblast"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing pyroblast fireball bolt missile nuke magic spell comet meteor falling sky",
	["spell_fire_felrainoffire"] = "green felfire demonic power fiery burning hot heat flames flaming blazing scorching searing rain falling meteors comets fireballs firebolts sky spell magic missiles nukes",
	["spell_fire_fire"] = "yellow orange fiery burning hot heat flames flaming blazing scorching searing magic spell conjured candlelight ethereal spirit ghost wisp apparition professions:cooking_fire passive:aftermath(warlock)",
	["spell_fire_firearmor"] = "golden yellow steel metal shield armor defense protection blocking barrier magic spell fiery fire burning hot heat red orange",
	["spell_fire_fireball"] = "red orange yellow fireball fiery burning hot heat flames flaming blazing scorching searing bolt magic nuke missile spell:conflagrate(warlock) spell:fire_blast(mage)",
	["spell_fire_fireball02"] = "red orange yellow fireball fiery burning hot heat flames flaming blazing scorching searing bolt magic nuke missile spell:pyroblast(mage) spell:soul_fire(warlock)",
	["spell_fire_firebolt"] = "red orange yellow fireball fiery burning hot heat flames flaming blazing scorching searing bolt magic spell nuke missile",
	["spell_fire_firebolt02"] = "red orange yellow fireball fiery burning hot heat flames flaming blazing scorching searing bolt magic spell nuke missile tab:fire(mage)",
	["spell_fire_flameblades"] = "fire fiery burning hot heat flames flaming blazing scorching searing forging forged dagger blade weapon knife steel metal red yellow orange magic spell power professions:smelting",
	["spell_fire_flamebolt"] = "red orange yellow fiery burning hot heat flames flaming blazing scorching searing bolt magic nuke missile spell:fireball(mage)",
	["spell_fire_flameshock"] = "silhouette outline yellow red orange fire fiery burning hot heat flames flaming blazing scorching searing magic power spell:flame_shock(shaman)",
	["spell_fire_flametounge"] = "red orange yellow silver mace hammer weapon steel metal fire fiery burning hot heat flames flaming blazing scorching searing enchanted enchantment glowing",
	["spell_fire_flare"] = "red orange yellow fiery burning hot heat flames flaming blazing scorching searing bolts missiles nukes magic spells power fireball firebolt comet meteor sky stars starry night spell:flare(hunter)",
	["spell_fire_frostresistancetotem"] = "blue ice icy cold frozen freezing frosty scepter weapon staff wand ranged snowflake magic power radiance radiant",
	["spell_fire_immolation"] = "yellow orange hand undeath undead fire fiery burning hot heat flames flaming blazing scorching searing magic ashen charred fingers claws corpse spell:immolate(warlock)",
	["spell_fire_incinerate"] = "yellow orange fiery burning hot heat flames flaming blazing scorching searing man soul anger angry fury furious rage raging aggression shouting yelling tortured magic power spell:mastery:_ignite(mage) spell:hellfire(warlock)",
	["spell_fire_lavaspawn"] = "yellow red orange fire fiery burning hot heat flames flaming blazing scorching searing lava magma face crooked twisted smile demonic elemental magic spell power evil",
	["spell_fire_masterofelements"] = "blue red yellow purple orange fiery burning hot heat flames flaming blazing scorching searing missiles nukes bolts fireballs explosion bursting blowing up magic spells elements elemental",
	["spell_fire_meteorstorm"] = "yellow orange red fiery burning hot heat flames flaming blazing scorching searing rain falling meteors comets fireballs firebolts sky spell magic missiles nukes",
	["spell_fire_moltenblood"] = "yellow orange red molten lava magma cracks cracked ground earth fire fiery burning hot heat magic power spell:mastery:_emberstorm(warlock)",
	["spell_fire_playingwithfire"] = "yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing hand magic spell wizard mage passive:backlash(warlock)",
	["spell_fire_ragnaros_lavabolt"] = "yellow orange magma lava fiery burning hot heat flames flaming blazing scorching searing bolts missiles nukes magic spells power fireballs earth ground",
	["spell_fire_ragnaros_molteninferno"] = "yellow orange magma lava fiery burning hot heat flames flaming blazing scorching searing fireball sphere of earth ground magic spell power",
	["spell_fire_ragnaros_splittingblow"] = "sulfuras hand of ragnaros hammer mace weapon fiery burning hot heat flames flaming blazing scorching searing yellow orange red firelord",
	["spell_fire_ragnaros_supernova"] = "yellow orange ragnaros supernova firelord bursting exploding blasting light power magic spell fiery burning hot heat flames flaming blazing scorching searing",
	["spell_fire_rune"] = "red yellow orange fire fiery burning hot heat rune of power symbol magic spell passive:cauterize(mage)",
	["spell_fire_sealoffire"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing rune of power symbols emblem sigil seal magic spell:combustion(mage)",
	["spell_fire_searingtotem"] = "yellow orange red fire fiery burning hot heat flaming blazing scorching wand weapon ranged arrowhead tip point shoot spell:searing_totem(shaman) spell:searing_flames(shaman)",
	["spell_fire_selfdestruct"] = "yellow orange fire fiery burning hot heat flaming blazing scorching searing powerful explosion bomb blast burst ashen charred death dying magic spell:flamestrike(mage) spell:explosive_trap(hunter) spell:magma_totem(shaman)",
	["spell_fire_soulburn"] = "yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing talons claws hand beam of light magic spell power talent:scorch(mage)",
	["spell_fire_sunkey"] = "yellow key unlocks",
	["spell_fire_totemofwrath"] = "red yellow orange totem mask face crooked twisted smile glowing eyes fire fiery hot heat flames flaming blazing scorching searing magic power spell:burning_wrath(shaman)",
	["spell_fire_twilightcano"] = "purple twilight volcano magma lava flow mountain fire fiery burning hot heat flames flaming blazing scorching searing magic spell power",
	["spell_fire_twilightfire"] = "purple twilight fire fiery burning hot heat flames flaming blazing scorching searing magic spell conjured candlelight ethereal spirit ghost wisp apparition",
	["spell_fire_twilightfireward"] = "purple fire fiery burning hot heat flames flaming blazing scorching searing magic barrier shield armor defense protection elemental spell:twilight_ward(warlock)",
	["spell_fire_twilightflamebolt"] = "purple twilight fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell power fireball water",
	["spell_fire_twilightflamebreath"] = "purple twilight breath teeth beam magic spell blast burst fire fiery burning hot heat flames flaming blazing scorching searing",
	["spell_fire_twilightflamestrike"] = "purple twilight fire fiery burning hot heat flames flaming blazing scorching searing bolt missile nuke magic spell power ball impact blast burst explosion wave area",
	["spell_fire_twilighthellfire"] = "purple fire crack tear rift chasm winds sundered spell magic",
	["spell_fire_twilightimmolation"] = "purple fire hand undeath undead fiery burning hot heat flames flaming blazing scorching searing spell magic ashen charred fingers claws corpse",
	["spell_fire_twilightnova"] = "purple twilight circle ball sphere bubbles magic spell power passive:metamorphosis:_void_ray(warlock)",
	["spell_fire_twilightpyroblast"] = "purple twilight fiery burning hot heat flames flaming blazing scorching searing pyroblast fireball bolt missile nuke magic spell power comet meteor falling sky",
	["spell_fire_twilightrainoffire"] = "purple twilight fiery burning hot heat flames flaming blazing scorching searing rain falling meteors comets fireballs firebolts sky spell magic missiles nukes",
	["spell_fire_volcano"] = "red yellow orange volcano mountain molten lava magma flowing liquid ash falling fire fiery burning hot heat flames flaming blazing scorching searing magic spell power passive:elemental_fury(shaman)",
	["spell_fire_windsofwoe"] = "sky dark clouds windy winds gusty gusts orange sunlight sunset pain suffering magic spell power",
	["spell_firefrost orb"] = "fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty conflicting elements red yellow orange blue magic spell power",
	["spell_fireresistancetotem_01"] = "blue yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty conflicting elements totem wand weapon ranged magic spell power",
	["spell_frost_arcticwinds"] = "blue ice icy frozen freezing frosty windy gusts gusty winds magic power burst passive:cold_weather_flying(mage) passive:cold_weather_flying(death_knight) spell:howling_blast(death_knight) passive:cold_weather_flying(druid) passive:cold_weather_flying(warlock) passive:cold_weather_flying(rogue) passive:cold_weather_flying(hunter) spell:mastery:_essence_of_the_viper(hunter) passive:cold_weather_flying(paladin) passive:cold_weather_flying(warrior) passive:cold_weather_flying(priest) passive:cold_weather_flying(shaman)",
	["spell_frost_chainsofice"] = "purple icy cold frozen frosty glowing enchanted enchantment links magic power spell:chains_of_ice(death_knight) spell:freezing_trap(hunter)",
	["spell_frost_chillingarmor"] = "blue ice icy cold frozen freezing frosty barrier shield armor defense protection magic spell lightbeams radiant radiance elemental",
	["spell_frost_chillingblast"] = "blue snowy ice icy cold frozen freezing frosty lightbeam radiance radiant magic power blast burst",
	["spell_frost_chillingbolt"] = "purple bolt missile nuke magic spell power",
	["spell_frost_coldhearted"] = "blue ice cold frozen freezing frosty pillar magic power spell:icy_veins(mage)",
	["spell_frost_fireresistancetotem"] = "red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing wand weapon ranged totem",
	["spell_frost_freezingbreath"] = "blue ice icy cold frozen freezing frosty nova blast burst breath shards icicles magic power spell:rime(death_knight)",
	["spell_frost_frost"] = "blue icy cold frozen freezing frosty magic power spell:ice_block(mage)",
	["spell_frost_frostarmor"] = "light blue cyan ice icy cold frozen freezing frosty skull skeleton skeletal evil demonic creature undead undeath necromancy magic power scourge bones head corpse spell:mastery:_frostburn(mage)",
	["spell_frost_frostarmor02"] = "blue ice icy cold frozen freezing frosty lightning thunder barrier shield defense protection magic radiant radiance elemental spell:frost_armor(mage)",
	["spell_frost_frostblast"] = "blue icy cold frozen freezing frosty lightbeam radiance radiant magic power blast burst spell:ice_lance(mage)",
	["spell_frost_frostbolt"] = "blue ice icy cold frozen freezing frosty bolt missile nuke magic spell power shards sharp flying",
	["spell_frost_frostbolt02"] = "blue ice icy cold frozen freezing frosty missile nuke magic power tab:frost(mage) spell:frostbolt(mage)",
	["spell_frost_frostbrand"] = "blue silver steel metal mace weapon hammer",
	["spell_frost_frostnova"] = "blue ice icy cold frozen freezing frosty blast burst breath shards icicles magic power spell:frost_nova(mage)",
	["spell_frost_frostshock"] = "blue ice icy cold frozen freezing frosty snowflake light radiant radiance magic power passive:shatter(mage) spell:frost_shock(shaman)",
	["spell_frost_frostward"] = "blue ice icy cold frozen freezing frosty snowflake lightbeams radiant radiance magic spell power",
	["spell_frost_frozencore"] = "blue ice icy cold frozen freezing frosty ball glowing pillar orb sphere magic spell power",
	["spell_frost_frozenorb"] = "spell:frozen_orb(mage)",
	["spell_frost_glacier"] = "blue ice icy frozen freezing frosty wave magic sharp shards freeze spell:cone_of_cold(mage)",
	["spell_frost_ice shards"] = "blue ice icy cold frozen freezing frosty magic spell shards sharp freeze bolt missile nuke power",
	["spell_frost_iceclaw"] = "light blue white polar bear furry beast animal creature ice icy cold frozen freezing frosty claws paw hand scratching mauling attacking",
	["spell_frost_icefloes"] = "blue icy cold frozen freezing frosty flowing water iceberg river sea ocean magic spell power",
	["spell_frost_iceshard"] = "blue purple ice icy cold frozen freezing frosty bolt missile nuke magic spell power shard point tip sharp dagger blade weapon knife enchanted enchantment icicle",
	["spell_frost_iceshock"] = "blue light ice icy cold frozen freezing frosty cloud dust powder magic fire fiery burning hot heat flames flaming blazing scorching searing ethereal spirit ghost wisp apparition spell:counterspell(mage) passive:improved_counterspell(mage)",
	["spell_frost_icestorm"] = "blue ice icy cold frozen freezing frosty shards bolts missiles nukes magic spells power flying falling spell:blizzard(mage) spell:blizzard(warlock)",
	["spell_frost_manaburn"] = "blue ice icy cold frozen freezing frosty fingers hand holding clutching light point spot magic spell mana fingernails claws",
	["spell_frost_manarecharge"] = "blue ice icy cold frozen freezing frosty water bubbles glowing light power arm hand mana river ocean sea magic spell",
	["spell_frost_piercing chill"] = "blue ice icy cold frozen freezing frosty bolt missile nuke magic spell power shards comet meteor impacting falling hitting",
	["spell_frost_ring of frost"] = "blue ice icy cold frozen freezing frosty ring circle snowflakes light magic spell power",
	["spell_frost_stun"] = "blue swirly vortex portal maelstrom magic power spiraling spell:concussive_shot(hunter)",
	["spell_frost_summonwaterelemental"] = "blue wave skull water elemental ocean sea liquid summoned creature magic power spell:mana_tide_totem(shaman)",
	["spell_frost_summonwaterelemental_2"] = "blue liquid summoned creature magic power spell:summon_water_elemental(mage)",
	["spell_frost_windwalkon"] = "white gray grey clouds windy gusty circle winds gusts cold aura magic power spell:inner_focus(priest) spell:water_walking(shaman)",
	["spell_frost_wisp"] = "blue ice icy cold frozen freezing frosty cloud dust powder ethereal wisp spirit ghost apparition magic spell power",
	["spell_frost_wizardmark"] = "blue rune of power symbol eyeball emblem mark vision sight magic spell sigil seal racial_passive:frost_resistance(dwarf) passive:purification(shaman)",
	["spell_frostfire orb"] = "blue red orange yellow frostfire orb fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty bolt missile nuke magic spell power",
	["spell_frostresistancetotem_01"] = "blue yellow orange fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty bolt missile nuke magic spell power flamethrower shattered shards totem wand weapon ranged",
	["spell_holiday_tow_spicecloud"] = "red toxic poison disease cloud dust powder magic spell power",
	["spell_holy_absolution"] = "runes of power symbols designs emblem brown winged wings restoration magic sigil seal spell:mastery:_illuminated_healing(paladin)",
	["spell_holy_arcaneintellect"] = "mage wizard's arcane brilliance intellect increase blue eyeball sight vision magic spell power",
	["spell_holy_ardentdefender"] = "golden yellow silver steel metal shield armor defense protection blocking helmet plate spell:ardent_defender(paladin)",
	["spell_holy_ashestoashes"] = "yellow brown crumbling clay golem man person ashen ashes dead death charred spell:last_stand(warrior) spell:strength_of_soul(priest)",
	["spell_holy_aspiration"] = "yellow golden holy power glowing radiant radiance magic healing restoration aura spell:divine_plea(paladin) spell:mastery:_echo_of_light(priest)",
	["spell_holy_auramastery"] = "yellow golden man person radiant radiance divine power magic barrier shield armor defense protection blocking enchanted enchantment healing restoration spell:devotion_aura(paladin)",
	["spell_holy_auraoflight"] = "purple hammer mace weapon glowing radiant radiance enchanted enchantment lightbeams",
	["spell_holy_avengersshield"] = "yellow golden steel metal armor defense protection blocking barrier magic radiant radiance enchanted enchantment glowing divine power healing restoration spell:avenger's_shield(paladin)",
	["spell_holy_avenginewrath"] = "yellow golden hammer mace weapon enchanted enchantment radiant radiance divine power magic spell:avenging_wrath(paladin)",
	["spell_holy_blessedlife"] = "yellow eyeball vision sight hands holding granting giving holy power magic spell",
	["spell_holy_blessedrecovery"] = "holy wings divine power glowing blue yellow man person naked bare chest magic spell healing restoration",
	["spell_holy_blessedresillience"] = "sword blade weapon seal emblem yellow red orange golden radiant radiance enchanted enchantment divine power holy circle lightbeams aura healing restoration magic spell sigil",
	["spell_holy_blessingofagility"] = "cat kitty feline furry fuzzy beast animal creature pawprint yellow outline silhouette magic spell power",
	["spell_holy_blessingofprotection"] = "holy shield armor defense protection blocking barrier magic spell divine power yellow golden",
	["spell_holy_blessingofstamina"] = "yellow tauren bull horned horns furry fuzzy beast animal creature cow silhouette outline strong strength magic spell power head",
	["spell_holy_blessingofstrength"] = "purple outlines silhouettes swords blades weapons crossed conflicting conflicted attack battle magic spell power",
	["spell_holy_blindingheal"] = "yellow arm hand reaching grabbing holy divine power healing restoration head human magic spell:binding_heal(priest)",
	["spell_holy_borrowedtime"] = "golden yellow chronos watch clock speed tempo fast slowed stopped halted warped magic power spell:borrowed_time(priest)",
	["spell_holy_championsbond"] = "blue silver helmet armor defense protection plate steel metal horns horned radiant radiance enchanted enchantment magic spell power",
	["spell_holy_championsgrace"] = "silver yellow golden helmet armor defense protection plate steel metal horns horned enchanted enchantment magic spell power",
	["spell_holy_chastise"] = "divine power radiant radiance yellow human hand lightbeams healing restoration magic spell:holy_word:_chastise(priest)",
	["spell_holy_circleofrenewal"] = "area orange yellow glowing sparkles sparkly radiant radiance lightwell lightbeams divine holy power restoration magic aura spell:circle_of_healing(priest)",
	["spell_holy_consumemagic"] = "black purple absorption absorbing magic mana consuming wizard mage shadowy unholy figure hooded cloaked evil spell:dark_simulacrum(death_knight)",
	["spell_holy_crusade"] = "golden yellow holy divine power sunset sunlight dusty misty radiant radiance glowing noble cause nobility human people strength strong purpose resolve healing restoration magic spell:supplication(paladin)",
	["spell_holy_crusaderaura"] = "golden yellow aura protection barrier light divine holy power magic cross healing restoration spell:heart_of_the_crusader(paladin)",
	["spell_holy_crusaderstrike"] = "yellow golden sword blade weapon enchanted enchantment glowing radiant radiance sunlight sunbeams lightbeams shiny shining healing restoration magic power spell:crusader_strike(paladin)",
	["spell_holy_devineaegis"] = "yellow light holy power casting magic spell praying prayer barrier shield armor defense protection bubble healing restoration passive:divine_aegis(priest)",
	["spell_holy_devotion"] = "purple radiant radiance fist enchanted enchantment punching clenched clenching powerful magic spell glowing plate armor steel metal circles lightbeams knuckles aura",
	["spell_holy_devotionaura"] = "silver blue golden steel metal shield armor defense protection blocking barrier magic spell radiant radiance lightbeams power enchanted enchantment aura circles",
	["spell_holy_dispelmagic"] = "swirly purple bolts missiles nukes spells power arcane passive:felhunter:_devour_magic(warlock) spell:purify(priest)",
	["spell_holy_divinehymn"] = "song yellow golden holy power musical notes flats winged harp wings flying magical melody tune bard healing restoration spell:divine_hymn(priest)",
	["spell_holy_divineillumination"] = "yellow golden holy power lightbeams shooting from eyes sight vision clarity posession spiritual magic devotion healing restoration spell:divine_favor(paladin) passive:evangelism(priest)",
	["spell_holy_divineintervention"] = "glowing yellow golden hammer mace weapon hand holding swinging attacking enchanted enchantment radiant radiance divine holy power magic spell",
	["spell_holy_divineprotection"] = "yellow golden holy power barrier shield armor defense magic radiant radiance enchanted enchantment fist seal emblem sigil clenched clenching circles aura restoration healing bubble knuckles spell:divine_protection(paladin)",
	["spell_holy_divineprovidence"] = "yellow golden x crossed lightbeams sparkles sparkly healing restoration divine holy power magic spell",
	["spell_holy_divinepurpose"] = "divine holy power yellow golden pink sunbeams sunlight lightbeams noble cause nobility human lifting hammer mace weapon person radiant radiance magic spell viking vrykul strength strong purpose resolve healing restoration",
	["spell_holy_divineshield"] = "yellow golden pink barrier armor defense protection magic radiant radiance enchanted enchantment seal emblem sigil glowing lightbeams healing restoration spell:divine_shield(paladin)",
	["spell_holy_divinespirit"] = "blue yellow naked human bare chest muscles enchanted enchantment glowing lightbeams magic holy divine power radiant radiance healing restoration spell:spiritual_precision(priest)",
	["spell_holy_dizzy"] = "yellow confusion magic spell stars squiggles bird swirly knocked out unconscious dizzy confused crazy",
	["spell_holy_elunesgrace"] = "crescent sky stars forest mists nighttime black blue dark moonlight moonlit scenery landscape background",
	["spell_holy_empowerchampion"] = "silver red yellow enchanted enchantment plate armor steel metal helmet defense protection glowing radiant radiance lightbeams horns horned magic spell power",
	["spell_holy_excorcism"] = "golden yellow skeleton skeletal skull dust sands dissolving fading eroding magic holy divine power bones spell:fear_ward(priest)",
	["spell_holy_excorcism_02"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing melting skeleton skeletal skull ashen ashes charred charring magic undeath undead scourge bones corpse blast burst spell:exorcism(paladin)",
	["spell_holy_eyeforaneye"] = "silver steel metal hammer mace weapon sword blade crossed conflicting conflicted smashing breaking cracking intercepting",
	["spell_holy_fanaticism"] = "glowing yellow orange golden holy power magic spell divine lightbeams radiant radiance person human silhouette outline stylized healing restoration",
	["spell_holy_fistofjustice"] = "blue brown enchanted enchantment weapon knuckles clenched punching radiant radiance glowing light talent:fist_of_justice(paladin)",
	["spell_holy_flashheal"] = "golden yellow holy divine power magic casting hand healing restoration lightbeams radiant radiance bright spell:flash_of_light(paladin) spell:flash_heal(priest)",
	["spell_holy_greaterblessingofkings"] = "blue golden yellow enchanted enchantment fist weapon knuckles clenched punching radiant radiance glowing light spell:blessing_of_might(paladin)",
	["spell_holy_greaterblessingoflight"] = "glowing golden yellow holy divine power magic spell radiant radiance seal emblem sigil symbol barrier shield armor defense protection shiny sparkly sparkles bright lightbeams healing restoration",
	["spell_holy_greaterblessingofsalvation"] = "purple pink golden yellow hands praying seal emblem sigil prayer circle healing restoration aura holy magic spell power",
	["spell_holy_greaterblessingofsanctuary"] = "blue barrier shield armor defense protection magic spell lightbeams radiant radiance holy divine power enchanted enchantment glowing healing restoration",
	["spell_holy_greaterblessingofwisdom"] = "cross seal emblem sigil radiant radiance glowing lightbeams purple silver copper healing restoration holy magic spell power",
	["spell_holy_greaterheal"] = "blue healing restoration lightbeam hand arm casting magic holy divine power spell:greater_heal(priest)",
	["spell_holy_guardianspirit"] = "spiritual healer wings winged archangel hooded cloaked holy divine messenger blue healing magic power restoration spell:guardian_spirit(priest)",
	["spell_holy_harmundeadaura"] = "yellow golden skull glowing radiant radiance evil undead undeath divine holy power magic spell circles aura scourge corpse",
	["spell_holy_heal"] = "yellow golden holy healing restoration magic spell power divine hand arm casting glowing radiant radiance",
	["spell_holy_heal02"] = "yellow golden holy healing restoration magic spell power divine hand arm casting glowing radiant radiance",
	["spell_holy_healingaura"] = "yellow golden holy magic power lightbeams radiant radiance hand arm circles aura divine priest healing restoration spell:seal_of_insight(paladin)",
	["spell_holy_healingfocus"] = "yellow lightning bolt thunder eyeball magic powerful spell healing restoration passive:divine_focus(priest)",
	["spell_holy_heroism"] = "yellow golden pink hero steel metal armor helmet defense protection chestplate warrior divine holy noble cause nobility power strength strong purpose resolve spell:guardian_of_ancient_kings(paladin)",
	["spell_holy_holybolt"] = "power magic divine lightbeams healing restoration yellow golden radiant radiance spell:holy_light(paladin)",
	["spell_holy_holyguidance"] = "yellow golden plate armor steel metal helmet defense protection cross holy divine power restoration healing radiant radiance lightbeams glowing sunbeams sunlight prayer magic spell:emancipate(paladin)",
	["spell_holy_holynova"] = "yellow golden holy healing divine power magic spell blast burst explosion restoration staff human bright lightbeams radiant radiance enchanted enchantment",
	["spell_holy_holyprotection"] = "red yellow golden holy healing power magic seal emblem sigil symbol glowing lightbeams radiant radiance restoration spell:mastery:_divine_bulwark(paladin) racial:gift_of_the_naaru(draenei)",
	["spell_holy_holysmite"] = "yellow golden orange lightbeams radiant radiance magic divine power sunlight holy blast burst spell:smite(priest)",
	["spell_holy_hopeandgrace"] = "blue holy magic spell divine power winged seal emblem sigil radiant radiance lightbeams circle aura healing restoration passive:grace(priest)",
	["spell_holy_impholyconcentration"] = "golden yellow holy magic spell divine power lightbeams radiant radiance aura human outline silhouette healing restoration",
	["spell_holy_improvedresistanceauras"] = "blue cyan holy magic spell divine power radiant radiance lightbeams steel metal barrier shield armor defense protection healing restoration",
	["spell_holy_innerfire"] = "yellow golden fiery burning hot heat flames flaming blazing scorching searing holy magic divine power circle aura spell:consecration(paladin) spell:inner_fire(priest)",
	["spell_holy_lastingdefense"] = "yellow golden silver orange steel metal armor defense protection blocking shields clock watch time fast slowed stopped halted warped",
	["spell_holy_layonhands"] = "white yellow golden holy magic divine power light glowing radiant radiance healing restoration holding giving granting spell:lay_on_hands(paladin) spell:levitate(priest)",
	["spell_holy_lesserheal"] = "purple holy magic divine power lightbeam restoration healing hand giving granting spell:heal(priest)",
	["spell_holy_lesserheal02"] = "purple holy magic spell divine power lightbeam restoration healing hand giving granting",
	["spell_holy_lightsgrace"] = "yellow golden healing restoration magic spell divine power radiant radiance lightbeams altar prayer blast burst explosion enchanted enchantment",
	["spell_holy_magicalsentry"] = "blue eyeball wizard's intellect increase spell:arcane_brilliance(mage) tab:arcane(mage)",
	["spell_holy_mindsooth"] = "purple lightning thunder bolt eyeball magic spell arcane power",
	["spell_holy_mindvision"] = "yellow purple red holy magic divine power sunlight sunbeams lightbeams radiant radiance glowing sky spell:mind_vision(priest)",
	["spell_holy_nullifydisease"] = "blue cyan skull undead undeath lightning thunder destroyed vaporized nullified glowing magic spell power scourge corpse",
	["spell_holy_painsupression"] = "grey gray teeth mouth agony torture anguish face human spell:pain_suppression(priest)",
	["spell_holy_penance"] = "yellow blue bolt missile nuke magic power holy divine healing restoration spell:penance(priest)",
	["spell_holy_persecution"] = "confusion magic spell stars squiggles bird swirly knocked out unconscious dizzy confused crazy yellow holy power glowing",
	["spell_holy_persuitofjustice"] = "blue white staff weapon scepter ruler ruling glowing enchanted enchantment",
	["spell_holy_power"] = "silver helmet armor defense protection steel metal plate holy divine power magic spell glowing white eyes bearded dwarf paladin light",
	["spell_holy_powerinfusion"] = "yellow holy divine anger angry fury furious rage raging aggression powerful glowing enchanted enchantment magic spell",
	["spell_holy_powerwordbarrier"] = "yellow golden holy magic divine healing restoration glowing radiant radiance enchanted enchantment shield armor defense protection bubble lightbeams spell:power_word:_barrier(priest)",
	["spell_holy_powerwordshield"] = "golden yellow barrier armor defense protection magic blocking holy divine healing restoration lightbeams glowing enchanted enchantment radiant radiance spell:power_word:_shield(priest) spell:mastery:_shield_discipline(priest)",
	["spell_holy_prayeroffortitude"] = "green glowing seal emblem sigil symbol holy magic spell divine power healing restoration lightbeams radiant radiance diamond",
	["spell_holy_prayerofhealing"] = "blue sky soldiers silver plate armor steel metal spears weapons helmets defense protection plume glowing healing restoration enchanted enchantment radiant radiance talent:repentance(paladin)",
	["spell_holy_prayerofhealing02"] = "glowing golden yellow holy divine power magic radiant radiance seal emblem sigil symbol barrier shield armor defense protection shiny sparkly sparkles bright lightbeams restoration spell:prayer_of_healing(priest)",
	["spell_holy_prayerofmendingtga"] = "golden yellow holy magic divine power healing restoration hands praying emblem sigil seal spell:prayer_of_mending(priest)",
	["spell_holy_prayerofshadowprotection"] = "purple magic spell protection defense demons summoned demonic creatures seal emblem sigil shadowy unholy figures voidwalkers runes symbols evil",
	["spell_holy_prayerofspirit"] = "blue yellow holy magic spell divine power glowing radiant radiance men human naked bare chests muscles",
	["spell_holy_proclaimchampion"] = "silver blue yellow plate armor steel metal helmet defense protection holy divine power magic radiant radiance lightbeam glowing enchanted enchantment horned horns",
	["spell_holy_proclaimchampion_02"] = "silver purple yellow plate armor steel metal helmet defense protection holy divine power magic lightbeam glowing radiant radiance enchanted enchantment horned horns",
	["spell_holy_pureofheart"] = "golden yellow holy casting magic spell divine power healing restoration glowing radiant radiance lightbeams point holidng conjuring hands heart center core",
	["spell_holy_purify"] = "golden yellow holy magic divine power healing restoration hand glowing radiant radiance lightbeam casting bright spell:cleanse(paladin)",
	["spell_holy_purifyingpower"] = "golden yellow holy magic divine power healing restoration purification spell:denounce(paladin)",
	["spell_holy_rapture"] = "white holy magic divine power healing restoration bright light halo angel spirit night elf gray grey radiant radiance glowing head face spell:rapture(priest)",
	["spell_holy_rebuke"] = "yellow golden magic divine power holy restoration healing hand arm clenched closed fist punch glowing radiant radiance knuckles spell:rebuke(paladin)",
	["spell_holy_redemption"] = "red silver blood plate armor steel metal gloves wounded holy power divine magic spell restoration healing glowing radiant radiance light casting giving life bringing back restoring",
	["spell_holy_removecurse"] = "purple red cursed sword blade weapon steel metal magic decurse purify restore cleanse cleansing spell:remove_corruption(druid)",
	["spell_holy_renew"] = "green holy magic casting divine power restoration healing hand arm glowing radiant radiance spell:renew(priest) spell:rapid_renewal(priest)",
	["spell_holy_restoration"] = "yellow pink holy magic spell blast burst divine power healing restoration lightbeams glowing radiant radiance",
	["spell_holy_resurrection"] = "blue silver blood plate armor steel metal gloves wounded holy power divine magic restoration healing glowing radiant radiance light casting giving life bringing back restoring spell:redemption(paladin) spell:resurrection(priest)",
	["spell_holy_retribution"] = "yellow golden sands skulls skeletons skeletal dust erosion ashes fading away eroding exorcism magic spell power bones spirits ghosts apparitions",
	["spell_holy_retributionaura"] = "red silver steel metal sword blade weapon glowing enchanted enchantment radiant radiance aura circles power magic",
	["spell_holy_revivechampion"] = "yellow gray grey silver helmet armor defense protection plate steel metal horns horned divine power magic spell lightbeams radiant radiance enchanted enchantment glowing faded fallen dead",
	["spell_holy_righteousfury"] = "yellow golden shiny silver hammer mace weapon steel metal smashing striking attacking motion movement spell:judgment(paladin)",
	["spell_holy_righteousnessaura"] = "blue silver steel metal sword blade weapon glowing enchanted enchantment radiant radiance aura circles power magic spell:seal_of_righteousness(paladin)",
	["spell_holy_rune"] = "yellow golden holy magic spell divine power glowing radiant radiance rune enchanted enchantment symbol healing restoration",
	["spell_holy_sealofblood"] = "silver red orange yellow sigil emblem symbol swords blades weapons steel metal crossed glowing enchanted enchantment radiant radiance magic power spell:seal_of_command(paladin)",
	["spell_holy_sealoffury"] = "red yellow orange golden metal black winged sword blade weapon seal sigil emblem symbol glowing enchanted enchantment radiant radiance magic power spell:righteous_fury(paladin)",
	["spell_holy_sealofmight"] = "golden yellow thor's mace weapon lightning bolts thunder enchanted enchantment glowing seal sigil emblem symbol magic power spell:hammer_of_justice(paladin)",
	["spell_holy_sealofprotection"] = "silver grey gray seal sigil emblem symbol golden yellow crown feathers purple royalty magic power spell:hand_of_protection(paladin)",
	["spell_holy_sealofrighteousness"] = "green seal sigil emblem symbol winged wings skull undeath undead mouth glowing enchanted enchantment radiant radiance poison toxic disease scourge teeth magic spell power corpse",
	["spell_holy_sealofsacrifice"] = "blood drop red symbol seal emblem tear liquid circle professions:first_aid spell:hand_of_sacrifice(paladin)",
	["spell_holy_sealofsalvation"] = "golden yellow holy magic divine power prayer praying hands circle seal emblem sigil aura symbol healing restoration spell:hand_of_salvation(paladin)",
	["spell_holy_sealofvalor"] = "red orange symbol seal emblem sigil sword blade weapon glowing enchanted enchantment magic spell:hand_of_freedom(paladin)",
	["spell_holy_sealofvengeance"] = "yellow golden steel metal daggers blades weapons knives knife crossed conflicting conflicted battle attack emblem sigil symbol magic power spell:seal_of_truth(paladin)",
	["spell_holy_sealofwisdom"] = "cross circle gold emblem sigil seal symbol holy healing restoration magic spell divine power aura",
	["spell_holy_sealofwrath"] = "circle golden yellow plate armor steel metal fist clenched resolve purpose determination determined emblem sigil symbol holy healing restoration magic divine power knuckles aura spell:seal_of_justice(paladin)",
	["spell_holy_searinglight"] = "yellow golden light touch magic divine power glowing radiant radiance healing restoration spell:holy_shock(paladin) spell:holy_fire(priest)",
	["spell_holy_searinglightpriest"] = "yellow golden holy magic spell divine power radiant radiance lightbeams glowing reaching out touch grasping holding blocking searing burning cleansing purifying purging sunlight",
	["spell_holy_senseundead"] = "blue floating flying skulls undeath magic spell enchanted enchantment shadow demonic creatures fel power necromancy glowing eyes unholy evil undead scourge corpses spirits ghosts apparitions heads",
	["spell_holy_serendipity"] = "swirly swirls blue yellow lightbeams radiant radiance glowing lightwells holy magic divine power spell:serendipity(priest)",
	["spell_holy_silence"] = "pink red face lipstick kissing hushing hushed silencing silenced kept quiet duress human female",
	["spell_holy_spellwarding"] = "blue yellow shadowy unholy figure silhouette outline casting magic spell power evil divine meditiation healing restoration",
	["spell_holy_spiritualguidence"] = "yellow golden magic divine power healing restoration meditation monk third eyes glowing radiant radiance human face head meditating spell:holy_insight(paladin) passive:mysticism(priest)",
	["spell_holy_stoicism"] = "blue stone gray grey hand finger pointing magic spell:vampiric_touch(priest)",
	["spell_holy_summonchampion"] = "blue silver plate armor steel metal helmet defense protection horned horns radiant radiance glowing lightbeams emblem sigil seal symbol magic spell power strength",
	["spell_holy_summonlightwell"] = "yellow golden summoning healing restoration holy magic divine power glowing radiant radiance lightbeams sparks runes runic of circle burst blast point spot aura spell:lightwell(priest)",
	["spell_holy_surgeoflight"] = "yellow pink healing restoration holy magic power lightbeams sunlight sunrise sunset human person standing radiant radiance glowing burst blast spell:divine_light(paladin)",
	["spell_holy_symbolofhope"] = "gray grey red stone statue flowers roses petals symbol icon deity hero human person spell:hymn_of_hope(priest) talent:body_and_soul(priest)",
	["spell_holy_testoffaith"] = "yellow golden holy magic spell prayer divine power healing restoration lightbeams sunlight morning praying hands together head female human person",
	["spell_holy_turnundead"] = "green skeleton skeletal skull glowing evil demonic creature undeath undead eyes mouth corrupted fel magic spell unholy power scourge bones corpse",
	["spell_holy_unyieldingfaith"] = "hand marked cross symbol yellow divine power magic fingers lightbeams radiant radiance glowing healing restoration spell:reckoning(paladin)",
	["spell_holy_vindication"] = "golden red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing plate armor steel metal gloves gauntlets first knuckles glowing impact holy divine power magic strength strong",
	["spell_holy_weaponmastery"] = "yellow golden metal swords blades weapons crossed conflicting conflicted battle attack glowing enchanted enchantment radiant radiance light divine power magic spell:holy_wrath(paladin)",
	["spell_holy_wordfortitude"] = "glowing seal emblem sigil symbol holy magic divine healing restoration lightbeams radiant radiance diamond blue white spell:power_word:_fortitude(priest)",
	["spell_hunter_aspectofthehawk"] = "brown eagle hawk bird",
	["spell_hunter_aspectoftheironhawk"] = "black eagle hawk bird",
	["spell_hunter_blackicetrap"] = "blue purple ice icy cold frozen freezing frosty shattered shards broken",
	["spell_hunter_icetrap"] = "light blue cyan icy cold frozen freezing frosty shattered shards broken spell:ice_trap(hunter)",
	["spell_ice_lament"] = "blue-green cyan skull skeleton skeletal face glowing eyes shield armor defense protection magic spell icy cold frozen freezing frosty bones talent:ice_barrier(mage)",
	["spell_ice_magicdamage"] = "blue ice icy cold frozen freezing frosty bolt missile nuke magic spell power fire fiery burning hot heat flames flaming blazing scorching searing flying",
	["spell_ice_rune"] = "blue ice icy cold frozen freezing frosty rune of power symbol glowing radiant radiance magic spell lightbeams 1 letter snowflakes snowing snowy blast burst explosion",
	["spell_impending_victory"] = "red hands holding wielding sword blade weapon hilt talent:impending_victory(warrior)",
	["spell_lfieblood"] = "green plant healing restoration nature natural magic herbalism herbs bubbles lifeblood nurturing nurture glowing",
	["spell_lifegivingseed"] = "green plant healing restoration nature natural magic life herbalism herbs seedling sapling budding emerging nurturing nurture glowing",
	["spell_lifegivingspeed"] = "foot human ankle fast speedy rushed rushing running dashing quick green glowing nature magic spell power increased swiftness",
	["spell_lightning_lightningbolt01"] = "purple blue lightning nature magic power crackle crackling thunderbolt jagged thunderstorm stormy cloudy clouds sky heavens electricity natural spell",
	["spell_mage_altertime"] = "yellow golden clock face warp bend speed rate spell:alter_time(mage)",
	["spell_mage_conjuredmanabuns"] = "brown cinnamon roll conjured food glaze icing bread",
	["spell_mage_conjuredmanapudding"] = "white mana pudding conjured rice bowl food spell:conjure_refreshment(mage)",
	["spell_mage_curtainoffrost"] = "blue apron snowflakes curtain shroud veil mystery ice icy cold frozen freezing frosty clothing magic spell power",
	["spell_mage_flameorb"] = "yellow orange living ember fire fiery burning hot heat flames flaming blazing scorching searing orb circle radiant radiance glowing magic spell power sun blast burst sphere",
	["spell_mage_frostbomb"] = "blue orb bomb nucleus nuclear atomic blast explosion ice icy cold frozen freezing frosty",
	["spell_mage_iceflows"] = "blue icy cold frozen freezing frosty conjuring conjured sweep swipe shards talent:ice_floes(mage)",
	["spell_mage_infernoblast"] = "yellow orange red fiery burning hot heat flames flaming blazing scorching searing fireball bolt missile nuke magic power spell:fire_blast(mage) spell:inferno_blast(mage)",
	["spell_mage_nethertempest"] = "purple bolts missiles nukes magic spells power arcane blast explosion",
	["spell_mage_presenceofmind"] = "purple mind control power brain waves radiant radiance glowing eyes psychic fortune teller",
	["spell_mage_runeofpower"] = "light blue cyan rune symbol power magic spell beams rays circle ring",
	["spell_mage_temporalshield"] = "blue ice icy cold frozen freezing frosty bubble barrier runes runic symbols talent:temporal_shield(mage)",
	["spell_magearmor"] = "blue mage's wizard's robe clothing fire fiery burning hot heat flames flaming blazing scorching searing ethereal arcane magic conjured protection defense defensive spell:mage_armor(mage)",
	["spell_magic_featherfall"] = "yellow orange purple slowfall magic levitation gravity wind flying floating falling drifting away winged feathered feathers animal creature bird's plucked spell:slow_fall(mage)",
	["spell_magic_greaterblessingofkings"] = "blue golden yellow glowing radiant radiance plate armor steel metal helmet defense protection lightbeams spell:blessing_of_kings(paladin)",
	["spell_magic_lesserinvisibilty"] = "blue bald human head duplicates copies person clones cloned copied duplicated replicated replication shadows figures images deception illusion magic arcane spell:mirror_image(mage) passive:succubus:_lesser_invisibility(warlock) spell:fade(priest)",
	["spell_magic_magearmor"] = "silver blue glowing radiant radiance plate armor steel metal helmet defense protection lightbeams",
	["spell_magic_managain"] = "blue stars wisps specks dots dust particles mana spring regeneration healing restoration holy spell power shiny sparkly sparkles spots points lights radiant radiance enchanted enchantment magical passive:wisdom_of_the_four_winds(mage) passive:wisdom_of_the_four_winds(death_knight) passive:wisdom_of_the_four_winds(druid) passive:wisdom_of_the_four_winds(warlock) passive:wisdom_of_the_four_winds(rogue) passive:wisdom_of_the_four_winds(hunter) passive:wisdom_of_the_four_winds(paladin) passive:wisdom_of_the_four_winds(warrior) passive:wisdom_of_the_four_winds(priest) passive:wisdom_of_the_four_winds(shaman)",
	["spell_magic_polymorphchicken"] = "white critter farm animal creature feathers feathered head beak companion:ancona_chicken",
	["spell_magic_polymorphpig"] = "brown gray grey pig farm animal hog tusks head creature spell:polymorph(mage)",
	["spell_magic_polymorphrabbit"] = "blue white rabbit furry creature animal critter bunny easter noblegarden head tongue sticking out lightning thunderbolt bulging eyes striking hitting spell:polymorph(mage)",
	["spell_misc_conjuremanajewel"] = "blue white shiny gemstone jewel diamond sparkly sparkles reflections reflective sharp racial_passive:gemcutting(draenei)",
	["spell_misc_drink"] = "silver steel metal mug cup mead root beer handle lid liquid drink alcoholic booze dwarven stout",
	["spell_misc_emotionafraid"] = "purple afraid frowning scared unsure nervous emoticon emotion food buff face human mask",
	["spell_misc_emotionangry"] = "red anger angry fury furious rage raging aggression frowning emoticon emotion food buff face human mask",
	["spell_misc_emotionhappy"] = "yellow happy pleased pleasant smiling grinning smile joyful emoticon emotion food buff face human mask",
	["spell_misc_emotionsad"] = "blue sad pouting frowning unhappy displeased disappointed emoticon emotion food buff face human mask",
	["spell_misc_food"] = "orange yellow swiss cheese with holes wedge slice triangle food snack buff",
	["spell_misc_hellifrepvpcombatmorale"] = "red arrows up diagonal circle battle plan map directions directional boosted morale high lifted fireworks aura magic spell power",
	["spell_misc_hellifrepvphonorholdfavor"] = "blue kingdom of lordaeron symbol emblem sigil seal banner alliance warship boat mast flag moon",
	["spell_misc_hellifrepvpthrallmarfavor"] = "red orgrimmar symbol emblem sigil seal banner horde warship boat mast flag moon",
	["spell_misc_petheal"] = "kitty feline furry fuzzy beast animal creature doctor patient bandaged wrapped bandages mummified mummification hurt injured pet",
	["spell_misc_warsongbrutal"] = "yellow green torn flag destroyed desecrated desecration ripped banner warship mast boat",
	["spell_misc_warsongfocus"] = "red torn flag destroyed desecrated desecration ripped banner warship mast boat spell:veteran_of_the_third_war(death_knight)",
	["spell_misc_wateringcan"] = "bluegreen cyan turquoise silver gray grey watering can tin metal",
	["spell_monk_brewmaster_spec"] = "yellow orange tauren bull horned horns furry fuzzy beast animal creature",
	["spell_monk_brewmastertraining"] = "root beer mead booze ale alcohol drink liquid brew book tome libram mug cup",
	["spell_monk_mistweaver_spec"] = "green cloud serpent dragon beast animal creature scaled scales scaly reptile outlined",
	["spell_monk_windwalker_spec"] = "gray grey tauren bull horned horns furry fuzzy beast animal creature yak",
	["spell_nature_abolishmagic"] = "wisp blue-green cyan magic spell blast burst explosion glowing nature power lightbeams radiant radiance",
	["spell_nature_acid_01"] = "blue water liquid acid poison toxic disease erosion corrosion goo nature magic spell power",
	["spell_nature_agitatingtotem"] = "totem shaman red skull bones weapon stick bloody voodoo",
	["spell_nature_ancestralguardian"] = "yellow orange red tauren bull horned horns furry fuzzy beast animal creature anger angry fury furious raging aggression intimidating grunting baring teeth mouth head magic power spell:berserker_rage(warrior)",
	["spell_nature_astralrecal"] = "purple portal swirly spiraling tunnel teleportation wormhole glowing magic power vortex maelstrom spell:astral_recall(shaman)",
	["spell_nature_astralrecalgroup"] = "purple portal swirly spiraling tunnel teleportation wormhole glowing magic spell power vortex maelstrom sparks sparkly sparkles lights radiant radiance",
	["spell_nature_bloodlust"] = "red yellow bloody shield armor defense protection blocking face skull spell:bloodthirst(warrior) spell:bloodlust(shaman)",
	["spell_nature_brilliance"] = "scepter staff weapon wand ranged blue ice icy cold frozen freezing frosty nature magic glowing radiant radiance lightbeams shard tip point spell:capacitor_totem(shaman)",
	["spell_nature_callstorm"] = "thrall warchief orcish calling summoning lightning thunderbolt gray grey green blue skies thunderstorm clouds cloudy bleak nature magic power crackle crackling natural spell:rolling_thunder(shaman)",
	["spell_nature_chainlightning"] = "blue orcish hand summoning calling thunderbolt nature magic power spell:chain_lightning(shaman)",
	["spell_nature_corrosivebreath"] = "green nature magic spell felblood venom poison toxic disease liquid blob goo snot mucus spit saliva",
	["spell_nature_crystalball"] = "light blue cyan orb ball globe sphere water liquid container seer magical future vision insight prediction bauble spell:omen_of_clarity(druid)",
	["spell_nature_cyclone"] = "gray grey windy gusts gusty winds leaves cyclone tornado torrential blowing hurling carrying lifting destroying tearing ripping nature magic powerful destructive force natural storm whirlwind swirly spell:hurricane(druid) spell:wind_shear(shaman)",
	["spell_nature_diseasecleansingtotem"] = "purple green totem shaman skull purifying nature shadow unholy magic glowing evil wand weapon ranged",
	["spell_nature_doublepolymorph1"] = "white two sheep lambs wool critters animals creatures",
	["spell_nature_doublepolymorph2"] = "white purple two sheep lambs wool critters animals creatures",
	["spell_nature_drowsy"] = "purple warlock hand casting shadow unholy magic glowing power light demonic evil stuck leaves tree wooden encased trapped transformed transformation spell:tranquilizing_shot(hunter)",
	["spell_nature_dryaddispelmagic"] = "yellow orange brown green leaves thorns vines log stick totem fire fiery burning hot heat flames flaming blazing scorching searing dispelling undoing removing cleansing nature natural magic spells wand weapon ranged",
	["spell_nature_earthbind"] = "blue grey gray storm gusty winds windy gusts tornado vortex nature natural powerful destruction destructive force whirlwind swirly magic spell:cyclone(druid)",
	["spell_nature_earthbindtotem"] = "golden yellow clenched fist totem green glowing radiant radiance nature magical powerful knuckles wand weapon ranged strength strong",
	["spell_nature_earthelemental_totem"] = "brown ground conjured summoned nature magic strength strong powerful defense stones skin rocks companion:pebble spell:earth_elemental_totem(shaman)",
	["spell_nature_earthquake"] = "brown ground sundering nature magic spell powerful earthquake explosion blast burst volcano lava magma fire fiery burning hot heat light mountain",
	["spell_nature_earthshock"] = "yellow brown mountains scenery landcape earthquake nature magic power spell:earth_shock(shaman)",
	["spell_nature_elementalabsorption"] = "gray grey hands holding clutching summoning absorbing taking red blue green take combined combining elements earth nature magical powers fire ice frost colorful light glowing clouds spell:conjure_familiar(mage) spell:spiritual_insight(shaman)",
	["spell_nature_elementalprecision_1"] = "red blue green take combined combining elements earth nature magical powers ice frost colorful light glowing clouds fireworks explosion blast burst spell:elemental_precision(shaman)",
	["spell_nature_elementalprecision_2"] = "red blue green take combined combining earth nature magical powers fire ice frost colorful light glowing clouds yellow orange hands conjuring summoning releasing gasses poisons toxins toxic spell:mastery:_enhanced_elements(shaman)",
	["spell_nature_elementalshields"] = "green sun circle felfireball fiery burning hot heat flames flaming blazing scorching searing barrier shield armor defense protection magic spell nature poison toxic disease demonic power planet eclipse sphere",
	["spell_nature_enchantarmor"] = "silver glowing blue steel metal helmet armor defense protection talent:presence_of_mind(mage)",
	["spell_nature_eyeofthestorm"] = "blue gray grey hurricane storm clouds whirlwind cyclone tornado center middle eye swirly torrential windy gusts gusty winds nature magic spell power",
	["spell_nature_faeriefire"] = "purple warlock hand casting shadow unholy magic glowing power light demonic evil spell:faerie_fire(druid)",
	["spell_nature_farsight"] = "orange vision portal seer magic spell:far_sight(shaman)",
	["spell_nature_focusedmind"] = "glowing green eyes human face raised eyebrow emerald evil deception cunning sneaky driven",
	["spell_nature_forceofnature"] = "glowing light blue cyan animal creature bird winged feathered feathers beak silhouette outline nature magic power head spell:moonkin_form(druid)",
	["spell_nature_giftofthewaterspirit"] = "outline silhouette eagle light blue cyan emblem sigil seal bird winged feathered feathers animal creature wings restoration magic power spell:healing_rain(shaman)",
	["spell_nature_giftofthewild"] = "purple circle magic power nature pawprints animals creatures pact union bond connection",
	["spell_nature_groundingtotem"] = "purple chains ethereal ghostly ghastly white imprisonment bindings shackles shackled imprisoned bound shadow magic power glowing light radiant radiance spell:grounding_totem(shaman)",
	["spell_nature_guardianward"] = "orange viper cobra fangs teeth venomous poisonous toxic deadly bite mouth animal creature reptile nature magic spell glowing striking strike head companion:albino_snake companion:black_kingsnake companion:brown_snake companion:crimson_snake",
	["spell_nature_healingtouch"] = "blue green fallen leaf water ice icy cold frozen freezing frosty nature natural magic power glowing life spell:healing_touch(druid) tab:restoration(druid) spell:mastery:_deep_healing(shaman)",
	["spell_nature_healingwavegreater"] = "green blue orcish hand water liquid nature magic power healing restoration reaching casting spell:chain_heal(shaman)",
	["spell_nature_healingwavelesser"] = "green purple orcish hand water liquid nature magic power restoration reaching casting spell:greater_healing_wave(shaman)",
	["spell_nature_healingway"] = "green orcish hand holding conjuring yellow ball of light magic power restoration casting spell:mastery:_harmony(druid) spell:healing_surge(shaman)",
	["spell_nature_heavypolymorph1"] = "white dizzy sheep confused confusion bumped head knocked out",
	["spell_nature_heavypolymorph2"] = "white dizzy sheep confused confusion head",
	["spell_nature_insect_swarm2"] = "yellow swarm bees wasps insects",
	["spell_nature_insectswarm"] = "green grasshopper insects bugs winged wings red eyes swarming group flying flight locusts army six legs magic spell power",
	["spell_nature_invisibilitytotem"] = "blue totem staff weapon lightbeams glowing radiant radiance magical power arcane spell:grace_of_air(shaman)",
	["spell_nature_invisibilty"] = "bald human head pink deception mirror images swiftness speed increased fast illusion magic spell arcane conjured shadow",
	["spell_nature_lightning"] = "blue magic powerful thunderbolt nature natural crackle crackling jagged electricity spell:arcane_power(mage) spell:innervate(druid) spell:lightning_bolt(shaman)",
	["spell_nature_lightningbolt"] = "green blue orcish hand summoning calling chain lightning thunderbolt nature magic power shaman spell",
	["spell_nature_lightningoverload"] = "blue lightning nature magic arcane power natural hands fingers thunderbolt crackle crackling striking hitting electricity spell:mastery:_elemental_overload(shaman)",
	["spell_nature_lightningshield"] = "purple thunder barrier armor defense protection magic blocking black obsidian nature power spell:sanctuary(paladin) spell:lightning_shield(shaman)",
	["spell_nature_magicimmunity"] = "green orcish hand water liquid nature magic power restoration reaching casting spell:healing_wave(shaman)",
	["spell_nature_manaregentotem"] = "purple totem shining lightbeams radiant radiance mana spell power arcane regeneration healing restoration eyeball stick magic",
	["spell_nature_massteleport"] = "yellow pink silhouettes outlines warriors enchanted enchantment glowing teleportation displacement black swords blades weapons plate armor steel metal magic spell power",
	["spell_nature_mentalquickness"] = "green brain speedy fast agility running dashing smart intelligent passive:mental_quickness(shaman)",
	["spell_nature_mirrorimage"] = "green orcish shadow mirror image tusks teeth deception conjured illusion magic spell",
	["spell_nature_moonglow"] = "purple starlight moonlight starfire moonbeams blast burst comet shooting meteor rocket launch sky nature magic spell power",
	["spell_nature_moonkey"] = "silver key unlocks spell:pick_lock(rogue)",
	["spell_nature_natureblessing"] = "blue-green leaf fallen nature magic spell natural healing restoration talent:renewal(druid)",
	["spell_nature_natureguardian"] = "blue starry sky landscape scenery background silhouette shadows tree branches leaves stars starlight shining sparkling shiny sparkly nighttime spell:celestial_alignment(druid) talent:nature's_guardian(shaman)",
	["spell_nature_natureresistancetotem"] = "green nature magic scepter weapon staff totem wand ranged leaves",
	["spell_nature_naturesblessing"] = "brown stump wooden tree bark",
	["spell_nature_natureswrath"] = "light blue cyan talons claws bird winged feathered feathers animal creature glowing enchanted enchantment foot spell:nature's_grasp(druid)",
	["spell_nature_naturetouchdecay"] = "brown wood bark tree branch bough stick leaves dead dying barren decaying nature natural",
	["spell_nature_naturetouchgrow"] = "brown wood bark tree branch bough stick leaves green growing life berries nature natural professions:herb_gathering",
	["spell_nature_nullifydisease"] = "green purple skull undeath undead disease plague toxic deadly head glowing enchanted enchantment power scourge corpse spell:mind-numbing_poison(rogue) spell:dispel_magic(priest)",
	["spell_nature_nullifypoison"] = "green black spike claw poison toxic venom glowing lightbeams radiant radiance enchanted enchantment magic spell power",
	["spell_nature_nullifypoison_02"] = "green black spike claw poison toxic venom glowing leaves lightbeams radiant radiance nature magic spell power",
	["spell_nature_nullward"] = "purple glowing enchanted enchantment mask tiki face brown red bones totem stick",
	["spell_nature_poisoncleansingtotem"] = "green snake viper venomous poisonous toxic king cobra fangs mouth teeth animal creature nature magic spell power totem wand weapon ranged head",
	["spell_nature_polymorph"] = "sheep furry critter creature head animal white grass eating gnawing spell:polymorph(mage)",
	["spell_nature_polymorph_cow"] = "cow furry creature critter animal black white grass eating gnawing head",
	["spell_nature_preservation"] = "purple leaf star green nature magic spell power glowing enchanted enchantment healing restoration life",
	["spell_nature_protectionformnature"] = "green leaf nature magic spell life growing enchanted enchantment circles aura",
	["spell_nature_purge"] = "lightning thunder striking strike purple blue black shadowy figure radiant radiance nature natural power magic destruction spell:evocation(mage) spell:purge(shaman)",
	["spell_nature_ravenform"] = "green emblem talons claws foot bird winged feathered feathers animal creature eagle outline silhouette magic power talent:nature's_swiftness(druid) spell:aspect_of_the_hawk(hunter)",
	["spell_nature_regenerate"] = "green blue hand life healing restoration nature magic power glowing enchanted enchantment radiant radiance racial_passive:regeneration(troll) spell:ancestral_spirit(shaman)",
	["spell_nature_regeneration"] = "purple pawprint creature animal black outline silhouette magic nature power spell:mark_of_the_wild(druid)",
	["spell_nature_regeneration_02"] = "green nature life magic spell tranquility healing restoration sparkles sparkly blue eyeball",
	["spell_nature_reincarnation"] = "red orange tauren bull horned horns furry fuzzy beast animal creature anger angry fury furious rage raging aggression shouting yelling intimidating mouth spell:rebirth(druid) spell:taunt(warrior)",
	["spell_nature_rejuvenation"] = "purple leaf magic nature power glowing enchanted enchantment healing restoration life spell:rejuvenation(druid)",
	["spell_nature_removecurse"] = "purple green rune symbol radiant radiance lightbeams hand grasping clutching fingers shadow unholy magic evil warlock power fel spell:remove_curse(mage)",
	["spell_nature_removedisease"] = "green purple skull glowing enchanted enchantment magic spell undead undeath disease poison toxic plague deadly head power scourge corpse",
	["spell_nature_resistmagic"] = "green orcish hand water liquid nature magic spell power healing restoration reaching casting",
	["spell_nature_resistnature"] = "green leaf magic nature power glowing enchanted enchantment healing restoration life spell:regrowth(druid)",
	["spell_nature_riptide"] = "blue turbulent waves water liquid sea ocean stormy elements elemental nature natural magic power surface spell:riptide(shaman)",
	["spell_nature_rockbiter"] = "green mace weapon enchanted enchantment spikes spiked spiky swinging moving motion striking strike attacking",
	["spell_nature_rune"] = "light green glowing enchanted enchantment rune symbol magic spell power nature healing restoration radiant radiance",
	["spell_nature_sentinal"] = "blue black sky moonlight owl creature animal feathered feathers bird winged hawk guardian nighttime watching looking cliff silhouette shadowy shadows figure outline",
	["spell_nature_shamanrage"] = "red silhouette outline anger angry fury furious raging aggression berserked glowing radiant radiance warrior horned horns spell:dark_command(death_knight) spell:shamanistic_rage(shaman)",
	["spell_nature_sicklypolymorph"] = "green white sickly sheep wool",
	["spell_nature_skinofearth"] = "green brown nature magic glowing enchanted enchantment radiant radiance plates armor plated skin defense protection blocking barrier spell:earth_shield(shaman)",
	["spell_nature_sleep"] = "pink human sleeping eyes closed napping dozing off unconsciousness dead death eyebrows eyelids spell:hibernate(druid) spell:meditation(druid) spell:meditation(shaman)",
	["spell_nature_slow"] = "red orange golden yellow glowing fire fiery burning hot heated enchanted enchantment radiant radiance forged forging magic power spell:slow(mage) spell:shackle_undead(priest)",
	["spell_nature_slowingtotem"] = "green totem wand weapon ranged glowing radiant radiance enchanted enchantment skull horns horned sparkly sparkles magic spell power",
	["spell_nature_slowpoison"] = "purple poison toxic needle point sharp tip blade dagger weapon knife",
	["spell_nature_spiritarmor"] = "green silver magic shield armor defense protection blocking riveted eagle creature flying animal winged wings feathered feathers racial_passive:nature_resistance(tauren) racial_passive:nature_resistance(night_elf)",
	["spell_nature_spiritwolf"] = "canine dog worg animal furry fuzzy creature beast purple teeth snarling fangs head spell:dismiss_pet(hunter) spell:ghost_wolf(shaman)",
	["spell_nature_starfall"] = "purple bolt nuke magic power star spell:arcane_missiles(mage) spell:moonfire(druid) tab:balance(druid)",
	["spell_nature_stoneclawtotem"] = "red yellow orange furry animal creature claws paw hand wooden totem wand weapon ranged green glowing radiant radiance fire fiery burning hot heat nature magic power spell:barkskin(druid)",
	["spell_nature_stoneskintotem"] = "purple arcane rune symbol obelisk obsidian glowing radiant radiance stone metal point tip pyramid lightning totem magic spell power",
	["spell_nature_stormreach"] = "blue lightning storm thunderbolt jagged nature natural magic power impacting hitting striking crackle crackling electricity spell:elemental_reach(shaman)",
	["spell_nature_stranglevines"] = "green brown nature vines entangled hand arm fist trapped magic knuckles spell:entangling_roots(druid) spell:entrapment(hunter) talent:earthgrab_totem(shaman)",
	["spell_nature_strength"] = "pink human muscles arm flexing strength strong power bodybuilding fist hand naked",
	["spell_nature_strengthofearthtotem02"] = "gray grey blue stone rock chains chained glowing radiant radiance enchanted enchantment nature magic strong strength powerful spell:earthbind_totem(shaman)",
	["spell_nature_swiftness"] = "blue spell:master_riding(mage) spell:master_riding(death_knight) spell:apprentice_riding(druid) spell:journeyman_riding(druid) spell:expert_riding(druid) spell:artisan_riding(druid) spell:master_riding(druid) spell:apprentice_riding(warlock) spell:journeyman_riding(warlock) spell:expert_riding(warlock) spell:artisan_riding(warlock) spell:master_riding(warlock) spell:apprentice_riding(rogue) spell:journeyman_riding(rogue) spell:expert_riding(rogue) spell:artisan_riding(rogue) spell:master_riding(rogue) spell:apprentice_riding(hunter) spell:journeyman_riding(hunter) spell:expert_riding(hunter) spell:artisan_riding(hunter) spell:master_riding(hunter) spell:apprentice_riding(paladin) spell:journeyman_riding(paladin) spell:expert_riding(paladin) spell:artisan_riding(paladin) spell:master_riding(paladin) spell:apprentice_riding(warrior) spell:journeyman_riding(warrior) spell:expert_riding(warrior) spell:artisan_riding(warrior) spell:master_riding(warrior) spell:apprentice_riding(priest) spell:journeyman_riding(priest) spell:expert_riding(priest) spell:artisan_riding(priest) spell:master_riding(priest) spell:apprentice_riding(shaman) spell:journeyman_riding(shaman) spell:expert_riding(shaman) spell:artisan_riding(shaman) spell:master_riding(shaman) spell:apprentice_riding(mage) spell:journeyman_riding(mage) spell:expert_riding(mage) spell:artisan_riding(mage)",
	["spell_nature_thorns"] = "green brown thorns vines roots sharp spikes spiky nature magic spell power growing",
	["spell_nature_thunderclap"] = "orcish smashing striking impacting hitting thor's hammer mace weapon lightning power magic enchanted enchantment blow swinging moving motion hand arm wristband gray grey blue spell:thunder_clap(warrior)",
	["spell_nature_timestop"] = "orange golden yellow pocketwatch clock hands time chronos fast slowed stopped halted tempo speed rate magic spell power fire fiery burning hot heat flames flaming blazing scorching searing pet_command:stay(hunter)",
	["spell_nature_tranquility"] = "blue sky landscape scenery background daytime clouds lightning thunder water liquid reflections surface lake sea ocean ripples waves spell:tranquility(druid)",
	["spell_nature_tremortotem"] = "yellow gray grey hammer mace weapon glowing enchanted enchantment golden stone symbol emblem rune of power spell:tremor_totem(shaman)",
	["spell_nature_undyingstrength"] = "green orcish hand leather armor wristband holding clutching grasping weapon fist steel metal sword blade spear polearm glowing enchanted enchantment strength strong",
	["spell_nature_unleashedrage"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing anger angry fury furious raging aggression sword blade weapon outline silhouette magic power spell:unleashed_rage(shaman)",
	["spell_nature_unrelentingstorm"] = "blue lightning thunderbolt magic power shockwave crater ocean sea liquid water splash tsunami tidal impact blast burst explosion spikes spiky bear trap striking hitting electricity natural surface spell:fulmination(shaman)",
	["spell_nature_unyeildingstamina"] = "yellow bull horned horns furry fuzzy beast animal creature cow silhouette outline strong strength magic power head racial_passive:endurance(tauren) spell:leader_of_the_pack(druid)",
	["spell_nature_web"] = "purple web glowing enchanted enchantment nature natural spider's silk mesh grid",
	["spell_nature_windfury"] = "blue purple yellow totem wand weapon ranged glowing radiant radiance light magic spell power casting nature",
	["spell_nature_wispheal"] = "blue yellow orange lightning thunderbolt impacting hitting striking jagged electricity nature natural magic spell power crackle crackling",
	["spell_nature_wispsplode"] = "blue exploding blasting supernova shockwave circle power magic starlight flare implosion point burst spot spell:arcane_explosion(mage) racial_passive:wisp_spirit(night_elf) racial_passive:arcane_resistance(gnome)",
	["spell_nature_wispsplodegreen"] = "green nature arcane explosion exploding blasting supernova shockwave circle power magic spell starlight flare wisp implosion point burst professions:lifeblood",
	["spell_nature_wrathofair_totem"] = "blue totem runes symbols flames fire magic spell power",
	["spell_nature_wrathv2"] = "magic bolt missile nuke power ball light glowing dust sun rays sand spark spell:wrath(druid)",
	["spell_paladin_clarityofpurpose"] = "golden yellow lightbeams radiant radiance glowing enchanted enchantment holy magic spell divine power armor steel metal chestplate defense protection shoulders human paladin healing restoration",
	["spell_paladin_divinecircle"] = "golden yellow radiant glowing bright lightbeams book reading pages paper magic divine power knowledge intellect aura spot point enlightenment spell:holy_radiance(paladin)",
	["spell_paladin_executionsentence"] = "gizmo gadget execution death sentence condemned battleaxe blade weapon",
	["spell_paladin_hammerofwrath"] = "blue green cyan gray grey silver mace weapon enchanted enchantment glowing swinging spell:hammer_of_wrath(paladin)",
	["spell_paladin_holyprism"] = "yellow golden glass prismatic light beam ray filter lantern spectrum triangle",
	["spell_paladin_inquisition"] = "golden yellow lightbeams arrows pointing upward outlines silhouettes toward sky sunlight holy magic divine power raise lift tall figures radiant radiance spell:inquisition(paladin)",
	["spell_paladin_lightofdawn"] = "yellow golden fire fiery burning hot heat flames flaming blazing scorching searing holy magic divine power lightbeams bright intense circles aura sunlight sunrise radiant radiance healing restoration spell:light_of_dawn(paladin)",
	["spell_paladin_lightshammer"] = "yellow golden hammer mace weapon radiant radiance",
	["spell_paladin_templarsverdict"] = "yellow silver gray grey steel metal hammer mace weapon rune of power symbol glowing circles lines wrapped entangled enchanted enchantment aura magic spell:templar's_verdict(paladin)",
	["spell_priest_angelicbulwark"] = "white yellow golden emblem sigil crest shield winged wings",
	["spell_priest_burningwill"] = "yellow golden blast explosion light rays waves radiant radiance mind mental psychic fortune teller control willpower",
	["spell_priest_chakra"] = "red yellow golden cross symbol emblem sigil seal holy divine magic spell power radiant radiance enchanted enchantment lightbeams circles aura healing restoration",
	["spell_priest_divinestar"] = "yellow golden rays point sunlight stardust swirling explosion blast source",
	["spell_priest_divinestar_holy"] = "brown yellow golden star fire fiery burning hot heat flames flaming blazing scorching searing emblem sigil crest symbol",
	["spell_priest_divinestar_shadow"] = "purple violet star fire fiery burning hot heat flames flaming blazing scorching searing emblem sigil crest symbol",
	["spell_priest_divinestar_shadow2"] = "purple violet rays point sunlight stardust swirling explosion blast source",
	["spell_priest_finalprayer"] = "yellow candle light fire fiery burning hot heat flames flaming blazing scorching searing",
	["spell_priest_mindspike"] = "purple shadow magical wand weapon ranged glowing enchanted enchantment diamond cloudy liquid gemstone spear light striking thrusting spell:mind_spike(priest)",
	["spell_priest_pathofdevout"] = "yellow shoe human foot stepping walking running speed flight flying",
	["spell_priest_psyfiend"] = "blue mouth teeth undead undeath unholy talent:psyfiend(priest)",
	["spell_priest_shadoworbs"] = "purple magic circle sphere power spell:shadow_orbs(priest)",
	["spell_priest_voidshift"] = "purple dna strand shadow spell:void_shift(priest)",
	["spell_priest_voidtendrils"] = "purple tentacle spiraling swirling octopus squid talent:void_tendrils(priest)",
	["spell_priest_vowofunity"] = "yellow human hands fists bump touching together unity cooperation",
	["spell_shadow_abominationexplosion"] = "red bloody vomiting dying zombie undead undeath unholy poisoned toxic diseased plagued teeth mouth head evil scourge corpse spell:corruption(warlock)",
	["spell_shadow_animatedead"] = "green moonlight hand reaching raised fingers clutching grasping undead undeath unholy zombie raising white nighttime midnight necromancy magic glowing enchanted enchantment evil scourge corpse spell:raise_dead(death_knight) spell:master_of_ghouls(death_knight)",
	["spell_shadow_antimagicshell"] = "green rune of power symbol emblem 999 666 unlucky unholy hex voodoo demonic evil superstition superstitious squiggles lines glowing racial_passive:arcane_resistance(blood_elf) spell:anti-magic_shell(death_knight)",
	["spell_shadow_antishadow"] = "purple magic spell protection defense summoned demonic creature seal emblem sigil shadowy figure unholy runes symbols evil racial_passive:aberration(worgen) passive:voidwalker:_shadow_bulwark(warlock)",
	["spell_shadow_auraofdarkness"] = "whites green eyeballs glowing eyes flaming demonic creatures evil scary scared haunted nervous ghosts demons apparitions fearful terrified terror horror horrified afraid of the dark unholy passive:metamorphosis:_doom(warlock)",
	["spell_shadow_blackplague"] = "purple floating flying skulls undeath magic spell enchanted enchantment shadow demonic creatures fel power necromancy glowing eyes unholy evil undead scourge corpses spirits ghosts apparitions heads passive:voidwalker:_suffering(warlock)",
	["spell_shadow_bloodboil"] = "red boiling liquid bubbles surface pool unholy evil passive:imp:_blood_pact(warlock)",
	["spell_shadow_brainwash"] = "purple brain knowledge intellect internal organ memory mind control noggin noodle magic spell power",
	["spell_shadow_burningspirit"] = "black purple hawk eagle bird winged feathered feathers animal creature flight flying head wings tail beak shadow unholy magic power nature natural evil spell:life_tap(warlock) passive:avoidance(hunter)",
	["spell_shadow_callofbone"] = "yellow white bones skulls totems voodoo unholy evil",
	["spell_shadow_carrionswarm"] = "blue white moths bugs swarming insects flying group flight unholy",
	["spell_shadow_charm"] = "purple magic power glowing eyes demonic creature evil smile grin face shadowy figure unholy raised eyebrows vision sight spell:vengeance(death_knight) spell:vengeance(druid) racial:every_man_for_himself(human) spell:vengeance(warrior)",
	["spell_shadow_chilltouch"] = "purple hand casting shadow unholy magic spell bones arm undead undeath ghoul apparition skeleton skeletal claws ghastly ghostly ethereal reaching grasping spirit scary haunting evil scourge corpse",
	["spell_shadow_coneofsilence"] = "purple blue glowing ghoul shadow unholy magic bright light x-ray xray medical teeth fangs skull skeleton skeletal undead undeath ethereal figure ghostly ghastly apparition spirit bones eye sockets mouth haunting scary evil scourge corpse",
	["spell_shadow_contagion"] = "blue white moths bugs swarming insects flying group flight talent:unholy_blight(death_knight)",
	["spell_shadow_corpseexplode"] = "red bloody skull skeleton skeletal undeath undead unholy exploding explode explosion blowing up corpse body sticky liquid blast burst evil scourge bones",
	["spell_shadow_creepingplague"] = "yellow cliff rocks injury wound broken shattered torn ripped skin earth breaking down dissolved dissolving unholy",
	["spell_shadow_cripple"] = "green lightbeam of talons claws bird winged feathered feathers animal creature demonic glowing hand foot fingernails shadow unholy nature magic power blast burst evil spell:banish(warlock)",
	["spell_shadow_curse"] = "purple red sword blade weapon cursed skull partitioned line division divided aura glowing enchanted enchantment unholy magic spell power",
	["spell_shadow_curseofachimonde"] = "blue arcane intellect brilliance lightning thunderbolts cursed crossed out canceled cancelled negated broken undone destroyed revoked glowing thunderstorm sky torn rift disenchanted unholy magic spell power passive:soulburn:_curse(warlock)",
	["spell_shadow_curseofmannoroth"] = "green silver glowing poison steel metal sword blade weapon crossed out cursed canceled cancelled destroyed broken undone negated revoked torn rift disenchanted unholy magic spell power",
	["spell_shadow_curseofsargeras"] = "yellow red orange silver lightning thunderbolts cursed crossed out canceled cancelled negated broken undone destroyed revoked glowing thunderstorm sky torn rift shield armor defense protection blocking barrier magic steel metal disenchanted unholy power spell:agony(warlock)",
	["spell_shadow_curseoftounges"] = "yellow orange red summoned demonic fire fiery burning hot heat flames flaming blazing scorching searing creature imp cursed glowing mouth teeth eyes pointy pointed ears unholy evil magic spell power head face",
	["spell_shadow_darkritual"] = "blue ice icy cold frozen freezing frosty corpse hand lightbeam casting magic spell skeletal skeleton bones claws undead undeath demonic creature evil necromancy unholy",
	["spell_shadow_darksummoning"] = "silver glowing green undead undeath warriors skeletal army of the raised enchanted enchantment spears weapons plume helmets armor defense protection plate steel metal necromancy magic spell powerful unholy evil scourge corpses",
	["spell_shadow_deadofnight"] = "glowing green yellow moon white undead undeath raised necromancy enchanted enchantment hand reaching grasping magic powerful unholy evil scourge corpse spell:raise_ally(death_knight)",
	["spell_shadow_deathanddecay"] = "red rose flower herbalism mageroyal wilting decaying falling apart petals dropping green stem life dying withering rotting unholy spell:death_and_decay(death_knight)",
	["spell_shadow_deathcoil"] = "green glowing skull skeleton skeletal demonic creature melting corpse undead undeath fire fiery burning hot heat flames flaming blazing scorching searing evil mouth eyes socket dying decaying unholy magic power scourge bones head spell:death_coil(death_knight)",
	["spell_shadow_deathpact"] = "red orange yellow fist power strength fire fiery burning hot heat clenched glowing enchanted enchantment knuckles unholy magic spell strong",
	["spell_shadow_deathscream"] = "red skull skeletal skeleton bloodlust bloody tusks teeth mouth open yelling screaming roaring intimidating anger angry fury furious rage raging aggression terrifying corpse undeath undead unholy evil scourge bones",
	["spell_shadow_deathsembrace"] = "purple face banshee undeath undead spirit ghost apparition haunting phantom glowing eyes open mouth ethereal trapped rooted held place pale anger angry fury furious rage raging aggression unholy evil magic spell power scourge corpse",
	["spell_shadow_demonbreath"] = "white blue eyes bubbles water liquid demonic creature evil magic power breating lurking monster unholy spell:unending_breath(warlock)",
	["spell_shadow_demonform"] = "purple demonic evil creature horned horns glowing eyes pointed ears fel shadow magic power summoned unholy head spell:metamorphosis(warlock)",
	["spell_shadow_demoniccirclesummon"] = "green of power runes symbols ritual seance gathering spirits ghosts glowing lights ethereal essences summoning casting fel shadow magic aura unholy evil spell:demonic_circle:_summon(warlock)",
	["spell_shadow_demoniccircleteleport"] = "green of power runes symbols ritual seance gathering spirits ghosts glowing lights ethereal essences summoning casting fel shadow magic aura unholy evil spell:demonic_circle:_teleport(warlock) passive:soulburn:_demonic_circle:_teleport(warlock)",
	["spell_shadow_demonicempathy"] = "cursed purple skull skeletal glowing eyes demonic creature shadow power magic spell undeath undead enchanted enchantment symbol emblem unholy evil scourge corpse head",
	["spell_shadow_demonicfortitude"] = "red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing symbol emblem skull demonic creature power magic glowing eyes enchanted enchantment undead undeath scythe unholy evil scourge corpse head spell:shadow_word:_death(priest)",
	["spell_shadow_demonicpact"] = "yellow orange red gray grey demonic summoned creature felguard glowing green eyes fire fiery burning hot heat flames flaming blazing scorching searing helmet armor defense protection explosion blast burst mouth head horn shadow magic power spell unholy evil",
	["spell_shadow_demonictactics"] = "purple red yellow creature fel shadowy magic power skull skeletal undead undeath mind deceit horns cracked unholy evil scourge head corpse spell:unending_resolve(warlock) passive:demonic_rebirth(warlock)",
	["spell_shadow_destructivesoul"] = "yellow red orange hands holding grasping casting summoning fire fiery burning hot heat magic spell power demonic destruction blast burst explosion unholy",
	["spell_shadow_detectinvisibility"] = "green black fel glowing lightbeams explosion blast burst magic spell power radiant radiance circle aura eye sight vision bubbles liquid racial_passive:shadow_resistance(draenei) racial_passive:shadow_resistance(undead)",
	["spell_shadow_detectlesserinvisibility"] = "blue-green cyan black arcane water liquid depth charge barrier shield armor defense protection magic spell ward glowing lightbeams explosion blast burst power radiant radiance circle aura eye sight vision bubbles",
	["spell_shadow_devouringplague"] = "purple demonic creature evil skull glowing eyes skeletal mouth open teeth breath undeath undead poison diseased pestilence unholy magic power scourge corpse head spell:devouring_plague(priest)",
	["spell_shadow_devouringplague."] = "purple demonic creature evil skull glowing eyes skeletal mouth open teeth breath undeath undead poison plague diseased pestilence unholy magic spell power scourge corpse head",
	["spell_shadow_dispersion"] = "purple arcane magic shadow power remove curse disease poison purify cleanse sparkles sparkly swirly spiraling light fade fading disperse dispersing vortex spell:dispersion(priest)",
	["spell_shadow_enslavedemon"] = "blue cyan hand fingers shadow magic fel power demonic evil sign runes of pale symbols unholy passive:control_demon(warlock) spell:enslave_demon(warlock)",
	["spell_shadow_evileye"] = "yellow white eyeball bloody evil demonic shadow fel magic power unholy spell:eye_of_kilrogg(warlock)",
	["spell_shadow_felarmour"] = "green shadow magic spell power demonic creature shield defense protection blocking barrier ward skull mouth breath skeletal undead undeath poison toxic plague pestilence disease unholy evil scourge corpse face passive:fel_armor(warlock)",
	["spell_shadow_felmending"] = "purple red shadowy hand reaching grasping demonic creature fingers evil claws warlock magic spell power bubbles boiling liquid raise undeath undead risen unholy",
	["spell_shadow_fingerofdeath"] = "claws fingernails hand undeath green glowing enchanted enchantment magic spell power casting shadow fel demonic creature evil pointing unholy racial_passive:touch_of_the_grave(undead)",
	["spell_shadow_focusedpower"] = "blue light divided ripped power magic spell fragmented splitting dividing fragmenting ripping shadowy figure arcane",
	["spell_shadow_fumble"] = "green silver brown hand grasping steel metal sword blade weapon plate armor glowing enchanted enchantment shadow magic fel demonic power hilt unholy",
	["spell_shadow_gathershadows"] = "purple shadow fel demonic magic spell power skull skeletal ghostly ghastly apparitions ethereal spirits demons creatures gathering circling swirling swirly spiraling unholy evil vortex maelstrom",
	["spell_shadow_ghostkey"] = "purple key unlocks",
	["spell_shadow_grimward"] = "white red skull shield armor defense protection blocking skeletal undeath undead sticks totem teeth eyes sockets unholy evil spell:curse_of_exhaustion(warlock) spell:disarm_trap(rogue)",
	["spell_shadow_haunting"] = "blue ethereal ghosts apparitions spirits undeath undead gathering dissolving dust melting exorcism floating skeletons skeletal mouths eyes sockets unholy evil magic bones power scourge corpses spell:drain_soul(warlock)",
	["spell_shadow_impphaseshift"] = "purple yellow glowing imp goblin warlock summoned demonic creature fel shadow magic spell power hands head ears mouth eyes fingernails fingers casting summoning unholy evil",
	["spell_shadow_improvedvampiricembrace"] = "purple bloody hand symbol of power bloodlust ritualistic spattered spattering splattered splattering fingers glowing enchanted enchantment unholy magic spell",
	["spell_shadow_lastingaffliction"] = "green yellow golden melting pocketwatch clock acid dissolving goo bubbles metal corrosion time chronos speed tempo fast slowed stopped sped up destroyed destroying halted magic spell power",
	["spell_shadow_lastingafflictions"] = "green cyan turquoise pocketwatch clock fire fiery burning hot heat flames flaming blazing scorching searing steel metal time chronos speed tempo fast slowed stopped sped up halted magic spell power",
	["spell_shadow_lifedrain"] = "red dripping droplets drops falling raining dropping liquid unholy spell:vampiric_blood(death_knight) spell:health_funnel(warlock) passive:soulburn:_health_funnel(warlock) spell:hemorrhage(rogue)",
	["spell_shadow_lifedrain02"] = "green glowing eyes mouth spirits ghosts ethereal posession posessed demonic creature evil exorcism human head shadow fel magic power draining stealing sucking siphoning force unholy apparition spell:drain_life(warlock)",
	["spell_shadow_manaburn"] = "blue hand claws clutching casting arcane magic power point spot star ball of light fingernails spell:elemental_focus(shaman)",
	["spell_shadow_manafeed"] = "purple blue glowing eyes face human blood elf shadows magic spell power mana star shine shiny shining sparkle sparkly sparkling head",
	["spell_shadow_metamorphosis"] = "purple orange red yellow demonic creature glowing fire fiery burning hot heat flames flaming blazing scorching searing eyes frowning fel shadow magic power spell sight vision unholy evil head",
	["spell_shadow_mindbomb"] = "yellow metal shiny shining bomb explosion explosives blast burst boom fuse lit fire fiery burning hot heat lightning electricity sparks ball round spherical sphere",
	["spell_shadow_mindflay"] = "purple face shadowy webbed exploding explosion mouth spattered spattering splattered splattering opened head mind unholy evil magic spell power",
	["spell_shadow_mindrot"] = "pink red tauren hand fingers holding clutching squeezing brain mind bloody internal organs guts unholy passive:felhunter:_spell_lock(warlock)",
	["spell_shadow_mindshear"] = "purple eyebrows night elf fel demonic creature magic power shadowy figure unholy spell:mind_sear(priest)",
	["spell_shadow_mindsteal"] = "brown pink human blinded eyebrow lazy eyed scarred cut stabbed wounded injured face passive:succubus:_seduction(warlock) spell:blind(rogue)",
	["spell_shadow_mindtwisting"] = "purple shadow magic spell power spiraling swirly swirls lightbeams radiant radiance glowing tunnel entrance vortex",
	["spell_shadow_misery"] = "green hand migraine headache misery stressful agony pain helmet armor",
	["spell_shadow_nethercloak"] = "purple shadowy cloaked hooded figure evil demonic creature ghost spirit warlock vile unholy apparition spell:cloak_of_shadows(rogue)",
	["spell_shadow_netherprotection"] = "green red yellow orange fire fiery burning hot heat flames flaming blazing scorching searing nature shadow magic spell power barrier shield armor defense protection",
	["spell_shadow_nightofthedead"] = "green moonlight hand reaching raised fingers clutching grasping undead undeath unholy zombie raising ghoul white nighttime midnight necromancy magic spell glowing enchanted enchantment evil scourge corpse",
	["spell_shadow_painandsuffering"] = "purple face mouth pain suffering agony terror fear horror night elf glowing green eyes eyebrows unholy",
	["spell_shadow_painfulafflictions"] = "light blue-green cyan purple melting pocketwatch clock acid dissolving goo bubbles metal corrosion time chronos speed tempo fast slowed stopped sped up destroyed destroying halted magic power spell:pandemic(warlock)",
	["spell_shadow_painspike"] = "purple arrowhead weapon spearhead pierce piercing puncture puncturing magic power spell:sudden_doom(death_knight) spell:black_arrow(hunter)",
	["spell_shadow_plaguecloud"] = "green posion goo vomiting dying zombie undead undeath unholy poisoned toxic diseased plagued teeth mouth head evil scourge corpse spell:pestilence(death_knight)",
	["spell_shadow_possession"] = "purple white gray grey pale glowing eyes demonic creature shadow fel evil power magic sight vision posessed ghost spirit undead undeath head unholy scourge corpse apparition spell:fear(warlock) spell:improved_fear(warlock) spell:premeditation(rogue)",
	["spell_shadow_psychichorrors"] = "purple white gray grey pale yellow glowing eyes ghoul corpse undeath undead raised risen forsaken scourge head mouth yelling shouting screaming anger angry fury furious rage raging aggression unholy evil spell:psychic_horror(priest)",
	["spell_shadow_psychicscream"] = "blue wind air magic shouting screaming yelling horrified terrified afraid swirls spirals mouth power unholy spell:psychic_scream(priest)",
	["spell_shadow_ragingscream"] = "purple red shield armor defense protection blocking barrier magic spell blood drip droplet skull teeth mouth undeath undead demonic creature power unholy evil scourge corpse face",
	["spell_shadow_rainoffire"] = "yellow orange red fiery burning hot heat flames flaming blazing scorching searing bolts missiles nukes magic spells power raining sky falling bombardment array fireballs spell:rain_of_fire(warlock)",
	["spell_shadow_raisedead"] = "white gray grey skeleton skeletal skull undeath evil demonic creature necromancy raised risen bones magic spell power unholy scourge corpse head talent:lichborne(death_knight) racial:will_of_the_forsaken(undead)",
	["spell_shadow_requiem"] = "green fel demonic creature magic power hand claws fingers reaching grasping undead undeath glowing raise risen unholy evil spell:shadow_infusion(death_knight)",
	["spell_shadow_ritualofsacrifice"] = "purple gray grey silver shadow magic power fel steel metal dagger blade weapon knife stabbing striking holding hand arm glowing enchanted enchantment unholy evil spell:sinister_strike(rogue)",
	["spell_shadow_rune"] = "purple shadow demonic magic power spell rune of symbol glowing enchanted enchantment",
	["spell_shadow_sacrificialshield"] = "red yellow bloody x cross winged wings flight flying canceled cancelled undone revoked destroyed removed shadow magic spell power divine unholy",
	["spell_shadow_scourgebuild"] = "blue pale gray grey silver hand arm fingernails holding ball of lightbeams point spot casting undead scourge symbol emblem sigil seal flag unholy spell:shadowburn(warlock)",
	["spell_shadow_sealofkings"] = "blue purple runes of power symbols circle ritual crown emblem sigil seal shadow magic spell royal passive:metamorphosis:_cursed_auras(warlock)",
	["spell_shadow_seedofdestruction"] = "green purple stone glowing enchanted enchantment plant sowing death destruction destructive shadow demonic fel magic power unholy spell:seed_of_corruption(warlock) passive:soulburn:_seed_of_corruption(warlock)",
	["spell_shadow_shadesofdarkness"] = "purple trees brought to life shadowy magic power outlines silhouettes glowing eyes humanoid figures arms limbs summoned demonic creatures evil felwood forest unholy corrupted apparitions spell:create_soulwell(warlock)",
	["spell_shadow_shadetruesight"] = "purple yellow floating flying skulls undeath magic enchanted enchantment shadow demonic creatures fel power necromancy glowing eyes unholy evil undead scourge corpses spirits ghosts apparitions heads spell:reaping(death_knight)",
	["spell_shadow_shadowandflame"] = "yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing mouth breathing glowing eyes shadowy shadows figure masked belching burping exhaling unholy evil magic spell power",
	["spell_shadow_shadowbolt"] = "purple green shadowbolt felfire missile nuke magic power demonic unholy spell:shadow_bolt(warlock)",
	["spell_shadow_shadowembrace"] = "blue hands arms light aura glowing shadow magic spell power casting holding conjuring",
	["spell_shadow_shadowfiend"] = "purple glowing claws arm bones fel magic power swiping striking blightbeast flesh monster creature mana regeneration restoration unholy spell:shadowfiend(priest)",
	["spell_shadow_shadowform"] = "purple green glowing eyes face head shadowy figure demonic summoned creature magic power enchanted enchantment aura unholy evil spell:shadowform(priest)",
	["spell_shadow_shadowfury"] = "purple shadow magic spell power swirly spiraling spikes spiky bear trap impact explosion blast burst tidal wave tsunami crater ocean sea liquid water splash surface vortex maelstrom",
	["spell_shadow_shadowmend"] = "purple shadow magic spell power spiraling swirly spirals twisting snowing snowflakes snowy sparkly sparkles vortex",
	["spell_shadow_shadowpact"] = "purple brown shadow fel demonic magic power voodoo doll hexed cursed straw scarecrow evil figure unholy spell:mastery:_master_demonologist(warlock)",
	["spell_shadow_shadowpower"] = "purple blue shadow magic spell power head mouth glowing eyes lightbeams enchanted enchantment vision sight deathly undead undeath corpse unholy evil scourge",
	["spell_shadow_shadowward"] = "pink purple skull shadowy magic power symbol shadows images deception illusion unholy spell:evasion(rogue)",
	["spell_shadow_shadowworddominate"] = "yellow purple orange spiraling eyeball face glowing empowered shadow magic controlled vision sight hypnotism hypnotised unholy spell:adrenaline_rush(rogue) talent:dominate_mind(priest)",
	["spell_shadow_shadowwordpain"] = "red orange yellow skull magic power undeath undead fire fiery burning hot heat flames flaming blazing scorching searing unholy evil scourge corpse spell:shadow_word:_pain(priest)",
	["spell_shadow_siphonmana"] = "blue mana glowing eyes mouth spirits ghosts ethereal posession posessed demonic creature evil exorcism human head shadow fel magic power draining stealing sucking siphoning life force unholy apparition spell:mind_flay(priest)",
	["spell_shadow_skull"] = "purple shadowy evil green glowing eyes face demonic creature magic power skull unholy sockets mouth undeath undead scourge corpse",
	["spell_shadow_soothingkiss"] = "pink green female lipstick kisssing of death poisonous toxic succubus glowing breathing exhaling whispering mouth seduction sexy unholy evil magic spell power",
	["spell_shadow_soulgem"] = "purple shadow orb shiny glowing ball sphere circle gemstone conjured magic power aura spell:soulstone(warlock)",
	["spell_shadow_soulleech"] = "red bloody soul vomiting puking guts pink coughing mouth face unholy magic power spell:blood_parasite(death_knight)",
	["spell_shadow_soulleech_1"] = "green unholy shadow magic spell power swirly spiraling vortex maelstrom liquid water stormy",
	["spell_shadow_soulleech_2"] = "blue shadow magic spell power swirly spiraling vortex maelstrom liquid water stormy",
	["spell_shadow_soulleech_3"] = "purple glowing green eyes fangs mouth teeth shadowy figure magic power corpse undeath undead spirit demonic fel creature unholy evil scourge head spell:strangulate(death_knight)",
	["spell_shadow_spectralsight"] = "yellow purple orange spiraling eyeball face glowing empowered shadow magic spell mind controlled vision sight hypnotism hypnotised unholy",
	["spell_shadow_summonfelguard"] = "red demonic creature shadow magic power summoned horn head unholy evil spell:summon_felguard(warlock)",
	["spell_shadow_summonfelhunter"] = "red bloody shadow magic demonic creature summoned beast teeth mouth unholy evil head spell:summon_felhunter(warlock)",
	["spell_shadow_summonimp"] = "red yellow green goblin teeth tusks eyes pointy nose mouth open yelling shouting evil chaotic chaos fire fiery burning hot heat flames flaming blazing scorching searing demonic fel power magic shadow summoned creature face unholy spell:summon_imp(warlock)",
	["spell_shadow_summoninfernal"] = "green yellow summoned creature demonic shadow fel magic power rocks rocky fire fiery burning hot heat flames flaming blazing scorching searing head stone unholy evil spell:summon_infernal(warlock)",
	["spell_shadow_summonsuccubus"] = "purple summoned creature demonic shadow fel magic power seduction sexy female horned horns glowing yellow eyes lipstick unholy evil head face spell:summon_succubus(warlock)",
	["spell_shadow_summonvoidwalker"] = "purple glowing eyes face head shadowy figure demonic summoned creature magic power enchanted enchantment aura unholy evil spell:summon_voidwalker(warlock)",
	["spell_shadow_teleport"] = "blue magic spot point blast burst explosion starlight lightbeams implosion spell power radiant radiance racial:arcane_torrent(blood_elf)",
	["spell_shadow_twilight"] = "purple landscape scenery background sky nighttime trees mountains hills clouds stars moon twilight dusk starlight spell:ritual_of_summoning(warlock) spell:nightfall(warlock)",
	["spell_shadow_twistedfaith"] = "purple blue shadow magic spell power naked body humanoid glowing lightbeams radiant radiance ghost spirit phantom apparition",
	["spell_shadow_unholyfrenzy"] = "corpse undead undeath evil scourge yellow purple orange spiraling eyeball face glowing empowered shadow magic controlled vision sight hypnotism hypnotised spell:unholy_frenzy(death_knight) spell:mind_blast(priest)",
	["spell_shadow_unholystrength"] = "silver red orange steel metal gauntlet glove armor hand fingers defense protection enchanted enchantment spell:unholy_might(death_knight) racial:stoneform(dwarf)",
	["spell_shadow_unstableaffliction_1"] = "light blue-green cyan purple melting pocketwatch clock acid dissolving goo bubbles metal corrosion time chronos speed tempo fast slowed stopped sped up destroyed destroying halted magic spell power",
	["spell_shadow_unstableaffliction_2"] = "green yellow golden melting pocketwatch clock acid dissolving goo bubbles metal corrosion time chronos speed tempo fast slowed stopped sped up destroyed destroying halted magic spell power",
	["spell_shadow_unstableaffliction_3"] = "red yellow orange humanoid yelling shouting mouth screaming pain suffering agony impact face unholy evil magic power spell:unstable_affliction(warlock)",
	["spell_shadow_unstableafllictions"] = "afflictions green yellow golden melting pocketwatch clock acid dissolving goo bubbles metal corrosion time chronos speed tempo fast slowed stopped sped up destroyed destroying halted magic spell power",
	["spell_shadow_unsummonbuilding"] = "blue runes of power shadow magic hand symbols gray grey pale unholy spell:vampiric_embrace(priest)",
	["spell_shadow_vampiricaura"] = "bloody tooth fang claw animal creature orange lightbeams dripping soaked tusk unholy spell:shred(druid)",
	["spell_shaman_ancestralawakening"] = "blue ethereal glowing ghost spirit ancestor naked phantom magic power light humanoid arms chest apparition spell:ancestral_awakening(shaman)",
	["spell_shaman_astralshift"] = "blue ethereal glowing tauren cow bull horned horns furry fuzzy beast animal creature male nosering ghost spirit ancestor head phantom magic spell power light apparition",
	["spell_shaman_bindelemental"] = "purple green chained chains missile nuke magic power bound bindings felfire shadowbolt shards stopped prevented talent:binding_shot(hunter) spell:bind_elemental(shaman)",
	["spell_shaman_blessingofeternals"] = "green healing restoration blessing gift touch bright lightbeams radiant radiance enchanted enchantment aura circle female humanoid maiden glowing magic spell power life",
	["spell_shaman_blessingoftheeternals"] = "blue healing restoration blessing gift touch bright lightbeams radiant radiance enchanted enchantment aura circle female humanoid maiden glowing magic spell power life",
	["spell_shaman_convection"] = "green explosion blast burst boom lightbeams sparks nature magic spell power",
	["spell_shaman_dropall_01"] = "red totem brown wooden placed elemental elements magic spell power",
	["spell_shaman_dropall_02"] = "blue totem brown wooden placed elemental elements magic spell power",
	["spell_shaman_dropall_03"] = "purple totem brown wooden placed elemental elements magic spell power",
	["spell_shaman_earthlivingweapon"] = "green silver nature natural steel metal growing life flowering blossoming blooming leaves vines entangling enchanted enchantment empowered sword blade weapon dagger knife glowing spell magic",
	["spell_shaman_earthquake"] = "brown ground cracked torn asunder tear rift cracks jagged ripped apart open rocks rocky dirt soil shaking magic power spell:earthquake(shaman)",
	["spell_shaman_elementaloath"] = "blue lightbeams glowing ritual cloaked hooded shadowy figures gathered gathering spirits ghosts apparitions phantoms robes magic powerful unholy evil spell:elemental_oath(shaman)",
	["spell_shaman_feralspirit"] = "wolves canines dogs worgs furry fuzzy beasts animals creatures purple blue glowing snarling growling anger angry fury furious rage raging aggression baring teeth heads spell:feral_spirit(shaman)",
	["spell_shaman_firenova"] = "red orange yellow fiery burning hot heat flames flaming blazing scorching searing magic power blast burst explosion rocks spell:fire_nova(shaman)",
	["spell_shaman_focusedstrikes"] = "blue silver lightning magic spell nature imbued enchanted enchantment empowered steel metal hammer mace weapon thunder electricity",
	["spell_shaman_giftearthmother"] = "red pink green yellow flower blossoming blooming leaves stem glowing enchanted enchantment bright light radiant radiance life growing healing restoration magic power spell:malfurion's_gift(druid)",
	["spell_shaman_hex"] = "green toad eyes surprised transformed transformation animal creature magic voodoo power head companion:tree_frog spell:hex(shaman)",
	["spell_shaman_improvedearthshield"] = "yellow red glowing radiant radiance nature magic spell power elemental shield barrier enchanted enchantment skin armor defense protection blocking hardened plated plates",
	["spell_shaman_improvedfirenova"] = "red orange yellow fire fiery burning hot heat flames flaming blazing scorching searing magic spell power blast burst explosion rocks",
	["spell_shaman_improvedreincarnation"] = "purple flexing humanoid shadowy figure outline silhouette strong radiant radiance enchanted enchantment powerful glowing muscles bodybuilding empowered magic spell passive:reincarnation(shaman)",
	["spell_shaman_improvedstormstrike"] = "purple silver steel metal mace weapon enchanted enchantment empowered magic storm spell:unleash_elements(shaman)",
	["spell_shaman_improvelavalash"] = "hammer mace weapon fire fiery burning hot heat flames flaming blazing scorching searing ice icy cold frozen freezing frosty elements elemental smashing swinging moving motion nature's power magic enchanted enchantment red yellow orange blue",
	["spell_shaman_lavaburst"] = "yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing magma rocks rocky molten explosion blast earth erupting magic power volcano spell:lava_burst(shaman)",
	["spell_shaman_lavaflow"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing molten lava volcano rocky rocks mountain flowing liquid erupting magic spell power",
	["spell_shaman_lavasurge"] = "yellow orange fire fiery burning hot heat flames flaming blazing scorching searing magma liquid exploding blowing blasting bursting rocks erupting volcano magic power spell:lava_surge(shaman)",
	["spell_shaman_maelstromweapon"] = "purple blue scepter staff magic enchanted enchantment empowered lightning thunder elemental electricity spell:maelstrom_weapon(shaman)",
	["spell_shaman_measuredinsight"] = "blue circles spiraling glowing light ball sphere liquid water magic spell power vortex maelstrom",
	["spell_shaman_primalstrike"] = "silver blue steel metal hammer mace weapon hitting impacting emblem spell:primal_strike(shaman)",
	["spell_shaman_spectraltransformation"] = "purple glowing threads of fate stars cosmos cosmic space swirling twisting nether spectre phantom lights magic spell power",
	["spell_shaman_spiritlink"] = "green glowing ring ritual pact friendship bond hands tauren troll humanoid magic power spell:spirit_link_totem(shaman)",
	["spell_shaman_spiritwalkersgrace"] = "green boots glowing enchanted enchantment lightbeams fast running walking speed swiftness magic power spell:spiritwalker's_grace(shaman)",
	["spell_shaman_staticshock"] = "blue summoned lightning golem thunder humanoid creature electricity power magic stunned shocked shocking surrounded crackle crackling spell:static_shock(shaman)",
	["spell_shaman_stormearthfire"] = "red yellow orange firestorm fiery burning hot heat flames flaming blazing scorching searing tornado vortex whirlwind jagged earth shards rocks windy gusts gusty winds elemental magic spell power",
	["spell_shaman_thunderstorm"] = "blue orange sky lightning thunderbolt jagged electricity shock elemental magic power crackle crackling striking hitting spell:thunderstorm(shaman)",
	["spell_shaman_tidalwaves"] = "blue moonlight tsunami ocean sea sky landscape scenerey background nighttime magic power spell:tidal_waves(shaman)",
	["spell_shaman_totemrecall"] = "gray grey brown wooden placed elemental elements magic power recalled recalling windy winds spell:totemic_recall(shaman)",
	["spell_shaman_unleashweapon_earth"] = "earth ground brown silver hammer mace steel metal glowing enchanted enchantment impacting hitting magic power spell:rockbiter_weapon(shaman)",
	["spell_shaman_unleashweapon_flame"] = "fire fiery burning hot heat flames flaming blazing scorching searing red silver hammer mace steel metal glowing enchanted enchantment impacting hitting magic power spell:flametongue_weapon(shaman)",
	["spell_shaman_unleashweapon_frost"] = "ice icy cold frozen freezing frosty blue silver hammer mace steel metal glowing enchanted enchantment impacting hitting magic power spell:frostbrand_weapon(shaman)",
	["spell_shaman_unleashweapon_life"] = "green nature healing restoration life silver hammer mace steel metal glowing enchanted enchantment impacting hitting magic power spell:earthliving_weapon(shaman)",
	["spell_shaman_unleashweapon_wind"] = "white blue winds windy air silver hammer mace steel metal glowing enchanted enchantment impacting hitting magic power spell:windfury_weapon(shaman)",
	["spell_totem_wardofdraining"] = "red white bloody totem skull bones voodoo",
	["spell_unused"] = "green glowing lightbeams division line poison toxic magic spell power",
	["spell_unused2"] = "blue humanoid creature blowing noise sound moonlight nighttime midnight starry stars sky horned horns calling summoning passive:shamanism(shaman)",
	["spell_warlock_darkregeneration"] = "purple shadow hand claw undead undeath unholy demonic evil grasping creature talent:dark_regeneration(warlock)",
	["spell_warlock_demonicportal_green"] = "green shadow portal unholy spell:demonic_gateway(warlock)",
	["spell_warlock_demonicportal_purple"] = "purple shadow demonic portal gateway unholy",
	["spell_warlock_demonsoul"] = "purple shadowy unholy figure outline silhouette fire fiery burning hot heat flames flaming blazing scorching searing magic power enchanted enchantment spirit phantom ghost apparition humanoid creature evil spell:dark_soul:_misery(warlock) spell:dark_soul:_knowledge(warlock) spell:dark_soul:_instability(warlock)",
	["spell_warlock_focusshadow"] = "purple hands holding ball of light magic power shadowflame essence spirit ghost wisp apparition spell:dark_intent(warlock)",
	["spell_warlock_harvestoflife"] = "green drain exorcism glowing light beams talent:harvest_life(warlock)",
	["spell_warlock_soulburn"] = "purple skull undead undeath fire fiery burning hot heat flames flaming blazing scorching searing fel shadow magic power apparition phantom ghost spirit eyes sockets teeth demonic creature unholy evil scourge corpse face head spell:soulburn(warlock)",
	["spell_warlock_summonabyssal"] = "yellow orange red fire fiery burning hot heat flames flaming blazing scorching searing abyssal demonic infernal creature summoned",
	["spell_warlock_summonimpoutland"] = "green creature summoned demonic evil spell:summon_fel_imp(warlock)",
	["spell_warlock_summonterrorguard"] = "orange terror felguard summoned creature demonic evil",
	["spell_warlock_summonwrathguard"] = "purple summoned creature demonic evil spell:summon_wrathguard(warlock)",
	["spell_warrior_wildstrike"] = "red dagger blade weapon knife bloody spell:wild_strike(warrior)",
	["spell_yorsahj_bloodboil_black"] = "black white grey gray liquid blood boiling bubbles bubbling brew goo pool steaming magic heated boiled surface",
	["spell_yorsahj_bloodboil_blue"] = "blue liquid blood boiling bubbles bubbling brew goo pool steaming magic heated boiled surface",
	["spell_yorsahj_bloodboil_green"] = "green poison disease liquid blood boiling bubbles bubbling brew goo pool steaming magic heated boiled surface",
	["spell_yorsahj_bloodboil_purple"] = "purple curse poison liquid blood boiling bubbles bubbling brew goo pool steaming magic heated boiled surface",
	["spell_yorsahj_bloodboil_yellow"] = "yellow liquid blood boiling bubbles bubbling brew goo pool steaming magic heated boiled surface",
}
